/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.common;

/**
 * Models a calculation entry within a projection. A calculation
 * entry consists of a name and the corresponding operation.
 *
 * @author Marc Seidemann
 */
public class CalculationAttribute<O extends Comparable> {

    /**
     * The name of the resulting attribute.
     */
    protected String name;
    /**
     * The operation to be calculated
     */
    protected CustomOperation<O> operation;

    /**
     * Creates a new calculation attribute.
     *
     * @param name the name of the resulting attribute
     * @param operation the operation to be calculated
     */
    public CalculationAttribute(String name, CustomOperation<O> operation) {
        this.name = name;
        this.operation = operation;
    }

    /**
     * Returns the name of this calculated attribute.
     *
     * @return the name of the resulting attribute
     */
    public String getName() {
        return name;
    }

    /**
     * Sets the name for this calculated attribute.
     *
     * @param name the name of the attribute
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Returns the operation that calculates this attribute.
     *
     * @return the operation to be calculated
     */
    public CustomOperation<O> getOperation() {
        return operation;
    }

    @Override
    public String toString() {
        return name+"="+operation.toString();
    }
}
