/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.common;

/**
 * Represents a sort attribute within a sort operator.
 *
 * @author Marc Seidemann
 */
public class SortAttribute {

    /**
     * Represents sorting orders.
     */
    public static enum Mode {
        ASCENDING,
        DESCENDING
    }

    /**
     * The name of the sorted attribute.
     */
    protected String name;
    /**
     * The ordering mode.
     */
    protected Mode mode;

    /**
     * Creates a new sort attribute.
     *
     * @param name the name of the attribute that should be sorted
     * @param mode the ordering mode
     */
    public SortAttribute(String name, Mode mode) {
        this.name = name;
        this.mode = mode;
    }

    /**
     * Creates a new sort attribute with ascending ordering mode.
     *
     * @param name the name of the attribute that should be sorted
     */
    public SortAttribute(String name) {
        this(name, Mode.ASCENDING);
    }

    /**
     * Returns the name of the sorted attribute.
     *
     * @return the name of the sorted attribute
     */
    public String getName() {
        return name;
    }

    /**
     * Sets the name of the sorted attribute.
     *
     * @param name the name of the attribute
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Returns the ordering mode of the sorted attribute.
     *
     * @return the sorting mode of this sorted attribute
     */
    public Mode getMode() {
        return mode;
    }

    @Override
    public String toString() {
        return name+" : "+mode.toString();
    }
}
