package de.umr.ep.event;

import java.util.List;

import de.umr.ep.common.Attribute;
import de.umr.ep.common.SchemaUtils;

/**
 * Available time representation in streams.
 *
 * @author Marc Seidemann
 */
public enum TimeRepresentation {

    /**
     * Represents events with time intervals, indicating the time the event is valid
     */
    INTERVAL,

    /**
     * Represents events with validity during a single point in time
     */
    POINT,

    /**
     * Represents events without time stamps. Therefore, the current time is used.
     */
    NONE;

    public static TimeRepresentation fromSchema(List<Attribute> schema){
        if (SchemaUtils.ATTR_TEND.equalsIgnoreCase(schema.get(schema.size()-1).getName()))
            return INTERVAL;
        else if(SchemaUtils.ATTR_TSTART.equalsIgnoreCase(schema.get(schema.size()-1).getName()))
            return POINT;
        else
            return NONE;
    }
}