/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.operators;


import de.umr.ep.common.Attribute;
import de.umr.ep.common.CalculationAttribute;
import de.umr.ep.queryengine.LogicalOperator;

import java.util.ArrayList;
import java.util.List;

/**
 * Implementation of a logical calculation operator node.
 *
 * @author Marc Seidemann
 */
public class Calculation extends LogicalOperator {

    /**
     * The input operator.
     */
    protected LogicalOperator input;
    /**
     * The calculations to be done.
     */
    protected List<CalculationAttribute<?>> operations;

    /**
     * Creates a new calculation operator.
     *
     * @param input the input operator
     * @param operations the calculation operations to be done
     */
    public Calculation(LogicalOperator input, List<CalculationAttribute<?>> operations) {
        this.input = input;
        this.operations = operations;
    }

    @Override
    public List<Attribute> getSchema() {
        List<Attribute> s = input.getSchema();
        List<Attribute> result = new ArrayList<>(s.size()+operations.size());
        for (int i = 0; i < s.size(); i++) {
            result.add(new Attribute(s.get(i)));
        }
        for(int i = 0; i < operations.size(); i++)
            result.add(new Attribute(operations.get(i).getName(), operations.get(i).getOperation().getType(s)));
        return result;
    }

    @Override
    public void setInput(LogicalOperator input, int i) {
        this.input = input;
    }

    @Override
    public int getInputCount() {
        return 1;
    }

    @Override
    public LogicalOperator getInput(int i) {
        return input;
    }

    /**
     * Returns the input operator.
     *
     * @return the input operator
     */
    public LogicalOperator getInput() {
        return input;
    }

    /**
     * Returns the calculations to be done.
     *
     * @return the calculations to be done.
     */
    public List<CalculationAttribute<?>> getOperations() {
        return operations;
    }

    @Override
    public String toString() {
        return "c"+operations.toString();
    }
}
