/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.operators;

import de.umr.ep.common.Attribute;
import de.umr.ep.common.SchemaUtils;
import de.umr.ep.queryengine.LogicalOperator;

import java.util.List;

/**
 * Implementation of a logical cross product operator.
 *
 * @author Marc Seidemann
 */
public class CrossProduct extends LogicalOperator {

    /**
     * The left input operator.
     */
    protected LogicalOperator leftInput;
    /**
     * The right input operator.
     */
    protected LogicalOperator rightInput;

    /**
     * Returns a new logical cross product operator.
     *
     * @param leftInput the left input operator
     * @param rightInput the right input operator
     */
    public CrossProduct(LogicalOperator leftInput, LogicalOperator rightInput) {
        this.leftInput = leftInput;
        this.rightInput = rightInput;
    }

    @Override
    public List<Attribute> getSchema() {
        return SchemaUtils.join(leftInput.getSchema(), rightInput.getSchema());
    }

    @Override
    public void setInput(LogicalOperator input, int i) {
        if (i == 0)
            leftInput = input;
        else
            rightInput = input;
    }

    @Override
    public int getInputCount() {
        return 2;
    }

    @Override
    public LogicalOperator getInput(int i) {
        if (i == 0)
            return leftInput;
        else
            return rightInput;
    }

    @Override
    public String toString() {
       return "\u00D7";
    }
    
    
}
