/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.operators;

import de.umr.ep.common.Attribute;
import de.umr.ep.queryengine.LogicalOperator;

import java.util.Arrays;
import java.util.List;

/**
 * Implementation of a logical distinct operator node.
 * The input of this operator has to be sorted!
 *
 * @author Marc Seidemann
 */
public class Distinct extends LogicalOperator {

    /**
     * The sorted input operator.
     */
    protected LogicalOperator input;

    /**
     * The attributes required for distinction.
     */
    protected String[] distinctAttributes;

    /**
     * Creates a new distinct operator on the given input.
     *
     * @param input the sorted input operator
     * @param distinctAttributes the attributes for distinction
     */
    public Distinct(LogicalOperator input, String ... distinctAttributes) {
        this.input = input;
        this.distinctAttributes = distinctAttributes;
    }

    @Override
    public List<Attribute> getSchema() {
        return input.getSchema();
    }

    @Override
    public void setInput(LogicalOperator input, int i) {
        this.input = input;
    }

    @Override
    public int getInputCount() {
        return 1;
    }

    @Override
    public LogicalOperator getInput(int i) {
        return input;
    }

    /**
     * Returns the input operator.
     *
     * @return the input operator
     */
    public LogicalOperator getInput() {
        return input;
    }

    /**
     * Returns all attributes required for distinction.
     *
     * @return the distinction attributes
     */
    public String[] getDistinctAttributes() {
        return distinctAttributes;
    }

    @Override
    public String toString() {
        return "D "+ Arrays.toString(distinctAttributes);
    }
}
