/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.operators;

import de.umr.ep.common.Attribute;
import de.umr.ep.queryengine.LogicalOperator;

import java.util.Arrays;
import java.util.List;

/**
 * Implementation of a logical group by operator node.
 *
 * @author Marc Seidemann
 */
public class Grouping extends LogicalOperator {

    /**
     * The input to be grouped.
     */
    protected LogicalOperator input;
    /**
     * List of attributes that are used for grouping.
     */
    protected List<String> groupAttributes;

    /**
     * Creates a new logical grouping operator.
     *
     * @param input the grouping input
     * @param groupAttributes the grouping attributes
     */
    public Grouping(LogicalOperator input, String ... groupAttributes) {
        this(input, Arrays.asList(groupAttributes));
    }

    /**
     * Creates a new logical grouping operator.
     *
     * @param input the grouping input
     * @param groupAttributes the grouping attributes
     */
    public Grouping(LogicalOperator input, List<String> groupAttributes) {
        this.input = input;
        this.groupAttributes = groupAttributes;
    }

    /**
     * Returns all attributes that are used for grouping.
     *
     * @return a list containing all grouping attributes
     */
    public List<String> getGroupAttributes() {
        return groupAttributes;
    }

    /**
     * Returns the input node.
     *
     * @return the input node
     */
    public LogicalOperator getInput() {
        return input;
    }

    @Override
    public List<Attribute> getSchema() {
        return input.getSchema();
    }

    @Override
    public void setInput(LogicalOperator input, int i) {
        this.input = input;
    }

    @Override
    public String toString() {
        return '\u03B3' + groupAttributes.toString()+"";
    }

    @Override
    public int getInputCount() {
        return 1;
    }

    @Override
    public LogicalOperator getInput(int i) {
        return getInput();
    }
}
