/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.operators;

import de.umr.ep.common.Attribute;
import de.umr.ep.queryengine.LogicalOperator;

import java.util.List;

/**
 * Implementation of a logical limit operator node.
 *
 * @author Marc Seidemann
 */
public class Limit extends LogicalOperator {

    /**
     * The input to be limited.
     */
    protected LogicalOperator input;
    /**
     * The limit, i.e. the maximum number of output events
     */
    protected long n;

    /**
     * Creates a new limit on the given input
     * @param input the input to be limited
     * @param n the maximum number of output events
     */
    public Limit(LogicalOperator input, long n) {
        this.input = input;
        this.n = n;
    }

    @Override
    public List<Attribute> getSchema() {
        return input.getSchema();
    }

    @Override
    public void setInput(LogicalOperator input, int i) {
        this.input = input;
    }

    /**
     * Returns the operator`s input.
     *
     * @return the operator`s input
     */
    public LogicalOperator getInput() {
        return input;
    }

    /**
     * Returns the maximum number of output events.
     *
     * @return the maximum number of output events
     */
    public long getNumber() {
        return n;
    }

    @Override
    public int getInputCount() {
        return 1;
    }

    @Override
    public LogicalOperator getInput(int i) {
        return getInput();
    }
}
