/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.operators;


import de.umr.ep.common.Attribute;
import de.umr.ep.queryengine.LogicalOperator;
import de.umr.ep.queryengine.predicates.BooleanExpression;

import java.util.List;

/**
 * Implementation of a logical selection operator node.
 *
 * @author Marc Seidemann
 */
public class Selection extends LogicalOperator {

    /**
     * The selection input.
     */
    protected LogicalOperator input;
    /**
     * The boolean expression.
     */
    protected BooleanExpression expression;

    /**
     * Creates a new selection on the given input with
     * the given boolean predicate.
     *
     * @param input the input operand
     * @param expression the predicate
     */
    public Selection(LogicalOperator input, BooleanExpression expression) {
        this.input = input;
        this.expression = expression;
    }

    /**
     * Returns the input node.
     *
     * @return the input node
     */
    public LogicalOperator getInput() {
        return input;
    }

    /**
     * Sets the input node.
     *
     * @param node the new input node
     */
    public void setInput(LogicalOperator node) {
        this.input = node;
    }

    /**
     * Returns the predicate.
     *
     * @return the boolean predicate
     */
    public BooleanExpression getExpression() {
        return expression;
    }

    /**
     * Sets the predicate.
     *
     * @param exp the new predicate
     */
    public void setExpression(BooleanExpression exp) {
        this.expression = exp;
    }

    @Override
    public List<Attribute> getSchema() {
        return input.getSchema();
    }

    @Override
    public void setInput(LogicalOperator input, int i) {
        setInput(input);
    }

    @Override
    public String toString() {
        return '\u03B4'+"_["+expression.toString()+"]";
    }

    @Override
    public int getInputCount() {
        return 1;
    }

    @Override
    public LogicalOperator getInput(int i) {
        return getInput();
    }
}
