/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.operators;


import de.umr.ep.common.Attribute;
import de.umr.ep.common.SortAttribute;
import de.umr.ep.queryengine.LogicalOperator;

import java.util.Arrays;
import java.util.List;

/**
 * Implementation of a logical ORDER BY operator node.
 *
 * @author Marc Seidemann
 */
public class Sort extends LogicalOperator {

    /**
     * The input operator node.
     */
    protected LogicalOperator input;
    /**
     * The list of sorted attributes.
     */
    protected List<SortAttribute> sortedAttributes;

    /**
     * Creates a new sort operator node.
     *
     * @param input the input operator node to be sorted
     * @param sortedAttributes the attributes used for sorting
     */
    public Sort(LogicalOperator input, SortAttribute ... sortedAttributes) {
        this.input = input;;
        this.sortedAttributes = Arrays.asList(sortedAttributes);
    }

    /**
     * Returns the input node.
     *
     * @return the input node
     */
    public LogicalOperator getInput() {
        return input;
    }

    /**
     * Returns the list of sorted attributes.
     *
     * @return the list of sorted attributes
     */
    public List<SortAttribute> getSortAttributes() {
        return sortedAttributes;
    }

    @Override
    public List<Attribute> getSchema() {
        return input.getSchema();
    }

    @Override
    public void setInput(LogicalOperator input, int i) {
        this.input = input;
    }

    @Override
    public String toString() {
        return "s"+sortedAttributes.toString();
    }

    @Override
    public int getInputCount() {
        return 1;
    }

    @Override
    public LogicalOperator getInput(int i) {
        return getInput();
    }
}
