/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.operators;

import de.umr.ep.common.Attribute;
import de.umr.ep.common.SchemaUtils;
import de.umr.ep.queryengine.LogicalOperator;

import java.util.List;

/**
 * Models a subquery.
 *
 * @author Marc Seidemann
 */
public class Subquery extends LogicalOperator {

    /**
     * The root node of the subquery
     */
    private LogicalOperator input;

    /**
     * Defines the identifier variable for this subquery
     */
    private String identifier;

    /**
     * Creates a new subquery.
     *
     * @param input the root node of the subquery.
     */
    public Subquery(LogicalOperator input, String identifier) {
        this.input = input;
        this.identifier = identifier;
    }

    @Override
    public List<Attribute> getSchema() {
        List<Attribute> s = SchemaUtils.copy(input.getSchema());
        if (identifier != null)
            SchemaUtils.setIdentifier(s, identifier);
        return s;
    }

    @Override
    public void setInput(LogicalOperator input, int i) {
        this.input = input;
    }

    /**
     * Returns the subquery`s root node.
     *
     * @return the subquery`s root node
     */
    public LogicalOperator getInput() {
        return input;
    }

    /**
     * Returns the subquery`s identifier.
     *
     * @return the identifier of this subquery
     */
    public String getIdentifier() {
        return identifier;
    }

    /**
     * Sets the subquery's identifier.
     *
     * @param identifier the identifier
     */
    public void setIdentifier(String identifier) {
        this.identifier = identifier;
    }

    @Override
    public int getInputCount() {
        return 1;
    }

    @Override
    public LogicalOperator getInput(int i) {
        return getInput();
    }

    @Override
    public String toString() {
        return input.toString();
    }
}
