/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.operators;

import de.umr.ep.common.Attribute;
import de.umr.ep.queryengine.LogicalOperator;

import java.util.List;

/**
 * Implementation of a logical UNION operator node.
 *
 * @author Marc Seidemann
 */
public class Union extends LogicalOperator {

    /**
     * The left union input.
     */
    protected LogicalOperator left;
    /**
     * The right union input.
     */
    protected LogicalOperator right;

    /**
     * Creates a new logical UNION operator.
     *
     * @param left the left union input
     * @param right the right union input
     */
    public Union(LogicalOperator left, LogicalOperator right) {
        this.left = left;
        this.right = right;
    }

    @Override
    public List<Attribute> getSchema() {
        return left.getSchema();
    }

    @Override
    public void setInput(LogicalOperator input, int i) {
        if (i == 0)
            left = input;
        else
            right = input;
    }

    @Override
    public int getInputCount() {
        return 2;
    }

    @Override
    public LogicalOperator getInput(int i) {
        if (i == 0)
            return left;
        else
            return right;
    }

    /**
     * Returns the left input operator.
     *
     * @return the left input operator
     */
    public LogicalOperator getLeft() {
        return left;
    }

    /**
     * Returns the right input operator.
     *
     * @return the right input operator
     */
    public LogicalOperator getRight() {
        return right;
    }
}
