/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.operators;

import de.umr.ep.common.Attribute;
import de.umr.ep.operators.windows.Window;
import de.umr.ep.queryengine.LogicalOperator;

import java.util.List;

/**
 * Implementation of a logical window operator.
 *
 * @author Marc Seidemann
 */
public class WindowGrouping extends LogicalOperator {

    /** The window for aggregation */
    private Window window;
    /** The input operator */
    private LogicalOperator input;

    /**
     * Creates a new window grouping operator.
     *
     * @param input the input operator
     * @param window the window
     */
    public WindowGrouping(LogicalOperator input, Window window) {
        this.window = window;
        this.input = input;
    }

    /**
     * Returns the input operator.
     *
     * @return the input operator
     */
    public LogicalOperator getInput() {
        return input;
    }

    /**
     * Returns the window.
     * @return the window
     */
    public Window getWindow() {
        return window;
    }

    @Override
    public List<Attribute> getSchema() {
        return input.getSchema();
    }

    @Override
    public void setInput(LogicalOperator input, int i) {
        this.input = input;
    }

    @Override
    public int getInputCount() {
        return 1;
    }

    @Override
    public LogicalOperator getInput(int i) {
        return input;
    }
}
