/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.operators.windows;

import de.umr.ep.queryengine.predicates.arithmetic.ArithmeticOperation;

/**
 * Implementation of a jumping window.
 *
 * @author Marc Seidemann
 */
public class JumpingWindow implements Window {
    /** The size of the window */
    private ArithmeticOperation size;
    /** The jump size */
    private ArithmeticOperation jumpSize;

    /**
     * Creates a new fixed size jumping window.
     *
     * @param size the size of the window (is also the jump size)
     */
    public JumpingWindow(ArithmeticOperation size) {
        this.size = size;
        this.jumpSize = size;
    }


    @Override
    public ArithmeticOperation getSize() {
        return size;
    }

    @Override
    public ArithmeticOperation getJumpSize() {
        return jumpSize;
    }
}
