/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.queryengine;


import de.umr.ep.common.Attribute;

import java.util.List;

/**
 * Models an abstract operator node within
 * a logical query plan.
 *
 * @author Marc Seidemann
 */
public abstract class LogicalOperator {

    /**
     * Returns the schema of the operator`s
     * output data.
     *
     * @return the operator`s output schema
     */
    public abstract List<Attribute> getSchema();

    /**
     * Sets the i-th input for this operator.
     *
     * @param input the input operator
     * @param i the number of the input
     */
    public abstract void setInput(LogicalOperator input, int i);

    /**
     * Returns the number of inputs of this operator.
     *
     * @return the number of this input operators
     */
    public abstract int getInputCount();

    /**
     * Returns the requested input operator.
     * The first input operator has index 0.
     * Operators are ordered ascending from left to right.
     *
     * @param i the index of the operator
     * @return The requested input operator
     */
    public abstract LogicalOperator getInput(int i);
}
