package de.umr.ep.queryengine.optimization;


import de.umr.ep.common.Attribute;
import de.umr.ep.common.SchemaUtils;
import de.umr.ep.common.SortAttribute;
import de.umr.ep.operators.*;
import de.umr.ep.queryengine.LogicalOperator;
import de.umr.ep.queryengine.predicates.*;
import de.umr.ep.queryengine.predicates.arithmetic.ArithmeticOperation;
import de.umr.ep.queryengine.predicates.arithmetic.ArithmeticOperationTerm;
import de.umr.ep.queryengine.predicates.arithmetic.ArithmeticTerm;
import de.umr.ep.queryengine.predicates.arithmetic.NestedOperation;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

public class CorrelationOptimizer {

    Stack<LogicalOperator> stack = new Stack<>();
    int numberRenames =0;

    public LogicalOperator optimize(LogicalOperator input) {

        getSelectionNodes(input);

        while (!stack.isEmpty()) {
            LogicalOperator node = stack.pop();
            //  if the input of the selection is no Subquery it must be a From and there should be no need for a general unnesting
            if (node instanceof Selection && ((Selection) node).getInput() instanceof Subquery)
                startDecorrelation((Selection) node);
        }
        return input;
    }


    private void getSelectionNodes(LogicalOperator n) {
		// Correlation can only be caused by a selection, not a join since subqueries aren't allowed in the USING-clause
        if (n instanceof Selection)
            stack.add(n);
        for (int i = 0; i < n.getInputCount(); i++) {
            getSelectionNodes(n.getInput(i));
        }
    }

    private void startDecorrelation(Selection n) {
        n.setExpression(startDecorrelation(n, n.getExpression()));
    }

    private BooleanExpression startDecorrelation(Selection n, BooleanExpression b) {

        if (b instanceof Comparison) {
            Comparison e = ((Comparison) b);
            Variable left = new Variable("");
            ScalarSubquery right = new ScalarSubquery(null);
            if (e.getLeft() instanceof Variable && e.getRight() instanceof ScalarSubquery) {
                if ( ((ScalarSubquery) e.getRight()).isCorrelated(n.getInput().getSchema()) ) {
                    left = (Variable) e.getLeft();
                    right = (ScalarSubquery) e.getRight();
                } else
                    return b;
            } else if (e.getLeft() instanceof ScalarSubquery && e.getRight() instanceof Variable) {
                if ( ((ScalarSubquery) e.getLeft()).isCorrelated(n.getInput().getSchema()) ) {
                    right = (ScalarSubquery) e.getLeft();
                    left = (Variable) e.getRight();
                } else
                    return b;
            } else if (e.getLeft() instanceof ScalarSubquery && e.getRight() instanceof ScalarSubquery) {
//                TODO two subqueries are compared (Solution: create a join and for each side of the join do the pushDownSteps)
            } else
                return b;

            Subquery selecSub = (Subquery) n.getInput();
            String q = selecSub.getIdentifier();
            if (q == null) {
                q = "q";
                // TODO find a sensible Identifier
            }
            // TODO find a sensible Identifier
            String identifierNew = "q2";
            selecSub.setIdentifier(identifierNew);

            List<String> dependent = right.getCorrelatedAttributes(selecSub.getSchema());
//          here we remove the identifier in front of the dependent attributes
            List<String> free = new ArrayList<String>();
            for (String s : dependent) {
                free.add(s.split("\\.")[1]);
            }

//			creates the SetProjection from the Decorrelation alogrithm
            Subquery D = createSetProjection(selecSub, free, identifierNew);

            // Step 1: rearranging the operatorgrapph
            // This is the first step of the decorrelation algorithm
            // TODO find a sensible IdentifierName
            String subqueryname = "z";
            // first the intersection between dependent and free has to be set equal
            BooleanExpression predicateD = EqualPredicateDefiner(free, free.size(), subqueryname, identifierNew);
            // Create new Join (Decorrelation alogrithm)
            Subquery depSubquery = new Subquery(new Join(D, right.getSubquery()), subqueryname);
            n.setInput(new Subquery(new Join(selecSub, depSubquery, predicateD), q));

            // Step 2: keep the connection in the expression of the selection
            // here we make the assumption that the first operator of the subquery  of the (correlated) scalarsubquery    is a projection
            // this assumption is sensible since it is a scalarsubquery and to obtain a scalar value a projection or an aggregation is needed
            // and an aggregation operator has always a projection successor
            // in case of an aggregation we need a renaming operator to fix a variable which can be injected into the superior selection
            List<String> scalarVariableList = ((Projection) right.getSubquery().getInput()).getAttributes();
            String scalarVariable = scalarVariableList.get(0);
            if (((Projection) right.getSubquery().getInput()).getInput() instanceof Aggregation) {
                LogicalOperator temp = ((Projection) right.getSubquery().getInput()).getInput();
                String newName = "rrr"+numberRenames;
                numberRenames++;
                Renaming rename = new Renaming(temp, new Renaming.RenameItem(((Projection) right.getSubquery().getInput()).getAttributes().get(0), newName));
                ((Projection) right.getSubquery().getInput()).setInput(rename);
                ((Projection) right.getSubquery().getInput()).getAttributes().clear();
                ((Projection) right.getSubquery().getInput()).getAttributes().add(newName);
                scalarVariable = newName;
            }


            // Step 3: PushDown the DependentJoin
            firstPushDownDependentJoinStep(depSubquery, free, dependent);
            // Here the new variable introduced above must be connected with the Subquery through the selection
            if(left.name.split("\\.").length==1) {
                ((Comparison) b).setLeft(new Variable(q + "." + left.name));
            }
            else
                ((Comparison) b).setLeft(new Variable(q + "." + left.name.split("\\.")[1]));
            ((Comparison) b).setRight(new Variable(q + "." + scalarVariable));
            return b;
        } else if (b.getArity() == 2) {
            b.setInput(0, startDecorrelation(n, b.getInput(0)));
            b.setInput(1, startDecorrelation(n, b.getInput(0)));
        } else if (b instanceof NestedExpression)
            return new NestedExpression(startDecorrelation(n, b.getInput(0)));
//
        else if (b instanceof Not) {
            return new Not(startDecorrelation(n, b.getInput(0)));
        } else if (b instanceof Exists) {
//            TODO how do we handle Exists clauses?
        }
        return b;

    }


    /**
     * @param depquery the dependent Subquery where the dependentjoin should be pushed down
     * @param f        List of the free variables
     * @param g        List of the dependent variables
     * @return
     */
    private void firstPushDownDependentJoinStep(Subquery depquery, List<String> f, List<String> g) {
        // the first PushDown is a special case, since a new Subquery has to be created

        Projection pi = ((Projection) ((Subquery) ((Join) ((Subquery) depquery).getInput()).getRight()).getInput());

        // Step 1: Pushing down the dependent subquery
        // TODO find a sensible IdentifierName
        String newIdentifier = "z2";
        Subquery depnew = new Subquery(((Subquery) depquery).getInput(), newIdentifier);
        depquery.setInput(pi, 1);
        LogicalOperator temp2 = pi.getInput();
        pi.setInput(depnew);
        ((Subquery) ((Join) depnew.getInput()).getRight()).setInput(temp2, 1);

        // Step 2 : check if the projection attributes are set appropriate when the the first pushdown happens

        // all  attributes inside the porjection have to be enriched with the prefix newIdentifier, additionally all dependent attributes have to be added to the projection
        for (String s : pi.getAttributes()) {
            if (s.split("\\.").length == 1) {
                String newName = newIdentifier + "." + s;
                pi.getAttributes().set(pi.getAttributes().indexOf(s), newName);
            } else {
                String newName = newIdentifier + "." + s.split("\\.");
                pi.getAttributes().set(pi.getAttributes().indexOf(s), newName);
            }

        }
        for (String s : f) {
            if (!pi.getAttributes().contains(newIdentifier + "." + s))
                pi.getAttributes().add(newIdentifier + "." + s);
        }

        // Step 3: Continue with the pushdown
        pushDownDependentJoinStep(pi, f, g, Mode.SCALAR);
    }

    /**
     * Represents the type resp. which direction of the previous  LogicalOperator was chosen
     */
    public static enum Mode {
        LEFT,
        RIGHT,
        SCALAR
    }

    /**
     * pushes the dependent join iteratively down the graph until the dependent join can be tranformed into a usual join
     * checks also if a substitution is possible in order to eliminate the remaining decorrelated join
     *
     * @param prevDep the successor LogicalOperator of the dependent Query
     * @param f       List of the free variables
     * @param g       List of the dependent variables
     * @param M       parameter which shows which of the input of prevDep need to be considered
     */
    private void pushDownDependentJoinStep(LogicalOperator prevDep, List<String> f, List<String> g, Mode M) {
        int direction = 0;
        if (M.equals(M.RIGHT)) {
            direction = 1;
        }
        //   a case study is needed for all possible LogicalOperators
        if (((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1) instanceof Aggregation) {
            Aggregation x = ((Aggregation) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1));
            ArrayList<String> temp = new ArrayList<String>();
            for (String s : f) {
                temp.add(((Subquery) prevDep.getInput(direction)).getIdentifier() + "." + s);
            }
            // If a grouping operator already exists then extend the grouping attributes, else create a grouping operator
            // in both cases push aggregation and grouping upwards
            if (x.getInput() instanceof Grouping) {
                ((Grouping) x.getInput()).getGroupAttributes().addAll(temp);
                ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).setInput(((Grouping) x.getInput()).getInput(), 1);
                x.getInput().setInput(prevDep.getInput(direction), 1);
                prevDep.setInput(x, direction);

            } else {
                ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).setInput(x.getInput(), 1);
                x.setInput(new Grouping(prevDep.getInput(direction), temp), 1);
                prevDep.setInput(x, direction);
            }

            pushDownDependentJoinStep(x.getInput(), f, g, Mode.SCALAR);

        } else if (((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1) instanceof Calculation) {
            // TODO rename the calculationAttributes in the calculation operator, here a setName method is needed for CalculationAttributes
//             a distinct attribute has to be renamed since it has to refer to the identifier of the subquery which will be pushed down
//            List<CalculationAttribute> temp= ((Calculation) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1)).getOperations();
//            for(String s:g){
//                if(temp.)
////                        temp[i]=    ((Subquery) prevDep.getInput(direction)).getIdentifier()+"."+s.split("\\.")[1];
//
//            }
//            pushDownScalarOperator(((Sort) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1)), prevDep);
//            return pushDownDependentJoinStep(prevDep.getInput(direction), f, g, Mode.Scalar);
        } else if (((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1) instanceof CrossProduct) {
//               TODO CrossProduct pushdown
        } else if (((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1) instanceof Distinct) {
//            a distinct attribute has to be renamed since it has to refer to the identifier of the subquery which will be pushed down
            String[] temp = ((Distinct) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1)).getDistinctAttributes();
            for (int i = 0; i < temp.length; i++) {
                if (temp[i].split("\\.").length == 1) {
                    temp[i] = ((Subquery) prevDep.getInput(direction)).getIdentifier() + "." + temp[i];
                } else
                    temp[i] = ((Subquery) prevDep.getInput(direction)).getIdentifier() + "." + temp[i].split("\\.")[1];
            }

            pushDownScalarOperator(prevDep, direction);
            pushDownDependentJoinStep(prevDep.getInput(direction), f, g, Mode.SCALAR);

        } else if (((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1) instanceof Except) {
            pushDownNonscalarOperator(prevDep, direction);
            pushDownDependentJoinStep(prevDep.getInput(direction).getInput(0), f, g, Mode.LEFT);
            pushDownDependentJoinStep(prevDep.getInput(direction).getInput(0), f, g, Mode.RIGHT);
        }
        // since a group by statement is always connected to an aggregate a special treatment of groupings is not needed
        // if a single grouping appears it is the same as a distinct
        else if (((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1) instanceof Grouping) {

        } else if (((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1) instanceof Join) {
//            TODO Join pushdown

            Join j = ((Join) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1));
            PredicatePartitionResult partitionResult = partitionJoinPredicate(j.getPredicate(), j.getLeft().getSchema(), j.getRight().getSchema());

        } else if (((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1) instanceof Limit) {
            pushDownScalarOperator(prevDep, direction);
            pushDownDependentJoinStep(prevDep.getInput(direction), f, g, Mode.SCALAR);

        } else if (((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1) instanceof Projection) {
            Projection pi= ((Projection) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1));
            String newIdentifier = ((Subquery) prevDep.getInput(direction)).getIdentifier();
            for (String s : pi.getAttributes()) {
                if (s.split("\\.").length == 1) {
                    String newName = newIdentifier + "." + s;
                    pi.getAttributes().set(pi.getAttributes().indexOf(s), newName);
                } else {
                    String newName = newIdentifier + "." + s.split("\\.");
                    pi.getAttributes().set(pi.getAttributes().indexOf(s), newName);
                }

            }
            for (String s : f) {
                if (!pi.getAttributes().contains(newIdentifier + "." + s))
                    pi.getAttributes().add(newIdentifier + "." + s);
            }
            pushDownScalarOperator(prevDep, direction);
            pushDownDependentJoinStep(prevDep.getInput(direction), f, g, Mode.SCALAR);
        } else if (((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1) instanceof Renaming) {
            for (String s : ((Renaming) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1)).getRenaming().keySet()) {
                if (((Renaming) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1)).getRenaming().get(s).split("\\.").length == 1) {
                    String temp = ((Renaming) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1)).getRenaming().get(s);
                    ((Renaming) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1)).getRenaming().remove(s);
                    ((Renaming) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1)).getRenaming().put(s, ((Subquery) prevDep.getInput(direction)).getIdentifier() + "." + temp);
                } else {
                    String temp = ((Renaming) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1)).getRenaming().get(s).split("\\.")[2];
                    ((Renaming) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1)).getRenaming().remove(s);
                    ((Renaming) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1)).getRenaming().put(s, ((Subquery) prevDep.getInput(direction)).getIdentifier() + "." + temp);
                }
            }
            pushDownScalarOperator(prevDep, direction);
            pushDownDependentJoinStep(prevDep.getInput(direction), f, g, Mode.SCALAR);

        } else if (((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1) instanceof Selection) {
            Selection x = ((Selection) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1));
//            checks if a renaming is needed, if so, then the selection will be pushed into the dependent join
            boolean renamingNeeded = false;
            for (String s : g)
                renamingNeeded = renamingNeeded || renameVariable(x.getExpression(), s, ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getLeft()).getIdentifier() + "." + s.split("\\.")[1]);
//            if dependent variables are in the expression of the selection the selection is pushed into the expression of the dependent join
            if (renamingNeeded) {
                ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).setInput(x.getInput(), 0);
//          If this is the first selection the expression in the dependent join will be null, hence a special treatment is needed
                if (((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getPredicate() == null) {
                    ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).setPredicate(x.getExpression());
                } else
                    ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).setPredicate(new And(((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getPredicate(), x.getExpression()));
                if (direction == 1)
                    pushDownDependentJoinStep(prevDep, f, g, Mode.RIGHT);
                else
                    pushDownDependentJoinStep(prevDep, f, g, Mode.LEFT);
            }
//            if no dependent variable is in the expression of the selection the selection is just an operator which can be pushed up
            else {
                pushDownScalarOperator(prevDep, direction);
                pushDownDependentJoinStep(prevDep.getInput(direction), f, g, Mode.SCALAR);
            }

        } else if (((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1) instanceof Sort) {
            // TODO rename the sortedAttributes in the sort operator, here a setName is needed for SortAttributes
////             a distinct attribute has to be renamed since it has to refer to the identifier of the subquery which will be pushed down
//            List<SortAttribute> temp= ((Sort) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1)).getSortAttributes();
//
//            for(int i=0;i<temp.size();i++){
//                  temp.get(i).setName()
////                       temp[i]=    ((Subquery) prevDep.getInput(direction)).getIdentifier()+"."+s.split("\\.")[1];
//
//            }
//            pushDownScalarOperator(((Sort) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1)), prevDep, direction);
//            return pushDownDependentJoinStep(prevDep.getInput(direction), f, g, Mode.SCALAR);

        } else if (((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1) instanceof Subquery) {
            pushDownScalarOperator(prevDep, direction);
            pushDownDependentJoinStep(prevDep.getInput(direction), f, g, Mode.SCALAR);

        } else if (((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1) instanceof Union) {
            pushDownNonscalarOperator(prevDep, direction);
            pushDownDependentJoinStep(prevDep.getInput(direction).getInput(0), f, g, Mode.LEFT);
            pushDownDependentJoinStep(prevDep.getInput(direction).getInput(0), f, g, Mode.RIGHT);
        }
        // This is the case where the dependent join can be transformed into a normal join
        else if (((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1) instanceof From) {
//           an optional Substitution
            boolean sub = substitutionChecker(((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getPredicate(), f, g, ((From) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1)).getAlias(), ((Subquery) prevDep.getInput(direction)).getIdentifier());
            if (sub) {
//                If Substitution is possible we have to introduce a renaming operator which renames the attributes of the input stream in order to
//                guarantee that above in the graph all attributes are defined (their prefix is the identifier of the subquery which will be eliminated now)
                Renaming.RenameItem[] r = new Renaming.RenameItem[g.size()];
                String id = "";
                if (!(((From) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1)).getAlias().equals(null))) {
                    id = ((From) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1)).getAlias();
                    int i = 0;
                    for (String s : f) {
                        r[i++] = new Renaming.RenameItem(id + "." + s, ((Subquery) prevDep.getInput(direction)).getIdentifier() + "." + s);
                    }
                } else {
                    int i = 0;
                    for (String s : f) {
                        r[i++] = new Renaming.RenameItem(s, ((Subquery) prevDep.getInput(direction)).getIdentifier() + "." + s);
                    }
                }
                BooleanExpression b = ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getPredicate();
//                the predicates which defined the equivalence classes  have to be removed, since they contain an attribute which will no longer exist after elimination of the decorrelated join
                b = removeEquivalenceClasses(((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getPredicate(), f, g, ((From) ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1)).getAlias(), ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getLeft()).getIdentifier());
                Selection s = new Selection(prevDep, b);
                prevDep.setInput(new Selection(new Renaming(((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1), r), b), direction);
            } else
                ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).setRight(((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1));
        }

    }

    private void pushDownNonscalarOperator(LogicalOperator prevDep, int direction) {
        LogicalOperator left = ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1).getInput(0);
        LogicalOperator right = ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1).getInput(1);
    //            TODO sensible identifier
        String newIdentifier = "D1";
        ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1).setInput(new Subquery(prevDep.getInput(direction), newIdentifier), 0);
        ((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1).setInput(new Subquery(prevDep.getInput(direction), newIdentifier), 1);
        prevDep.getInput(direction).setInput(((Subquery) ((Join) ((Subquery) prevDep.getInput(direction)).getInput()).getRight()).getInput(1), 0);

    }


    ///////////////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////// Helper /////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * @param selecSub the SubQuery which builts the ProjectionSet
     * @param f        list of free variablenames
     * @param newName  new name of the uncorrelated subquery
     * @return the ProjectSet for the Decorrelation alogrithm
     */
    private Subquery createSetProjection(Subquery selecSub, List<String> f, String newName) {
        ArrayList<String> temp1 = new ArrayList<String>();
        SortAttribute[] temp2 = new SortAttribute[f.size()];
        String[] temp3 = new String[f.size()];
        int i = 0;
        for (String s : f) {
            temp1.add(newName + "." + s);
            temp3[i] = s;
            temp2[i++] = new SortAttribute(s, SortAttribute.Mode.ASCENDING);
        }
        Distinct d = new Distinct(new Sort(new Projection(selecSub, f), temp2), temp3);
        return new Subquery(d, "D");
    }

    /**
     * this method creates recursively the join expression in the first step of the decorrelation algorithm
     *
     * @param free          list of the free variables
     * @param size          is the variable which drives the recursion
     * @param subqueryname  the name of the dependent subquery
     * @param identifierNew the name of the binded subquery
     * @return the BooleanExpression which represents the join predicate in the first step of the decorrelation algorithm
     */

    private BooleanExpression EqualPredicateDefiner(List<String> free, int size, String subqueryname, String identifierNew) {
        Variable var1 = new Variable(identifierNew + "." + free.get(free.size() - size));
        Variable var2 = new Variable(subqueryname + "." + free.get(free.size() - size));
        if (size == 1) {
            return new Equal(var1, var2);
        } else
            return new And(new Equal(var1, var2), EqualPredicateDefiner(free, size - 1, subqueryname, identifierNew));
    }


    private void pushDownScalarOperator(LogicalOperator p, int direction) {
        LogicalOperator x = ((Subquery) ((Join) ((Subquery) p.getInput(direction)).getInput()).getRight()).getInput(0);
        ((Subquery) ((Join) ((Subquery) p.getInput(direction)).getInput()).getRight()).setInput(x.getInput(0), 0);
        x.setInput(p.getInput(direction), 0);
        p.setInput(x, direction);
    }

    private boolean substitutionChecker(BooleanExpression b, List<String> f, List<String> g, String fromid, String subid) {
        if (b == null) {
            return false;
        }
        if (b instanceof Comparison) {
            if (!(b instanceof Equal)) {
                Term bl = ((Comparison) b).getLeft();
                Term br = ((Comparison) b).getRight();
                // TODO attributeNames has to be checked properly
                if (bl instanceof Variable && br instanceof Variable) {
                    if (((Variable) bl).name.split("\\.").length == 1 && ((Variable) br).name.split("\\.").length == 2 && ((Variable) br).name.split("\\.")[0].equals(subid)) {
                        if (f.contains(((Variable) bl).name)) {
                            return false;
                        }
                    } else {
                        if (((Variable) bl).name.split("\\.")[0].equals(fromid) && ((Variable) br).name.split("\\.").length == 2 && ((Variable) br).name.split("\\.")[0].equals(subid)) {
                            if (f.contains(((Variable) bl).name.split("\\.")[1])) {
                                return false;
                            }
                        }
                    }
                    if (((Variable) br).name.split("\\.").length == 1 && ((Variable) bl).name.split("\\.").length == 2 && ((Variable) bl).name.split("\\.")[0].equals(subid)) {
                        if (f.contains(((Variable) br).name)) {
                            return false;
                        }
                    } else {
                        if (((Variable) br).name.split("\\.")[0].equals(fromid) && ((Variable) bl).name.split("\\.").length == 2 && ((Variable) bl).name.split("\\.")[0].equals(subid)) {
                            if (f.contains(((Variable) br).name.split("\\.")[1])) {
                                return false;
                            }
                        }
                    }
                }
            }
        } else if (b.getArity() == 2) {
            return (true && (substitutionChecker(b.getInput(0), f, g, fromid, subid) | substitutionChecker(b.getInput(1), f, g, fromid, subid)));
        } else if (b instanceof NestedExpression) {
            return (true && substitutionChecker((b.getInput(0)), f, g, fromid, subid));
    //
        }
    //        TODO Not is a special case, as (not) \neq is an equivalence class
    // else if (b instanceof Not) {
    //            return (true && substitutionChecker(((Not) b).getExpression(), f, g, fromid, subid));
    //        }
        return true;
    }

    private BooleanExpression removeEquivalenceClasses(BooleanExpression b, List<String> f, List<String> g, String alias, String identifier) {
        if (b instanceof Comparison) {
            if (b instanceof Equal) {
                Term bl = ((Comparison) b).getLeft();
                Term br = ((Comparison) b).getRight();
                if (bl instanceof Variable && br instanceof Variable) {
                    // This is the case when the alias is null
                    if (((Variable) bl).name.split("\\.").length == 1 && ((Variable) br).name.split("\\.").length == 2 && ((Variable) br).name.split("\\.")[0].equals(identifier)) {
                        if (f.contains(((Variable) bl).name)) {
                            return new True();
                        }
                    } else {
                        if (((Variable) bl).name.split("\\.")[0].equals(alias) && ((Variable) br).name.split("\\.").length == 2 && ((Variable) br).name.split("\\.")[0].equals(identifier)) {
                            if (f.contains(((Variable) bl).name.split("\\.")[1])) {
                                return new True();
                            }
                        }
                    }
                    if (((Variable) br).name.split("\\.").length == 1 && ((Variable) bl).name.split("\\.").length == 2 && ((Variable) bl).name.split("\\.")[0].equals(identifier)) {
                        if (f.contains(((Variable) br).name)) {
                            return new True();
                        }
                    } else {
                        if (((Variable) br).name.split("\\.")[0].equals(alias) && ((Variable) bl).name.split("\\.").length == 2 && ((Variable) bl).name.split("\\.")[0].equals(identifier)) {
                            if (f.contains(((Variable) br).name.split("\\.")[1])) {
                                return new True();
                            }
                        }
                    }
                    return b;
                }
            }
            return b;
        } else if (b.getArity() == 2) {
            b.setInput(0, removeEquivalenceClasses(b.getInput(0), f, g, alias, identifier));
            b.setInput(1, removeEquivalenceClasses(b.getInput(1), f, g, alias, identifier));
        } else if (b instanceof NestedExpression)
            return new NestedExpression(removeEquivalenceClasses((b.getInput(0)), f, g, alias, identifier));
    //
        return b;
    }

    /**
     * Renames a variable in the given boolean expression.
     *
     * @param exp    the boolean expression
     * @param var    the (old) name of the variable to be renamed
     * @param newVar the new name of the variable
     */
    private boolean renameVariable(BooleanExpression exp, String var, String newVar) {
        if (exp instanceof Comparison) {
            return (renameVariable(((Comparison) exp).getLeft(), var, newVar) || renameVariable(((Comparison) exp).getRight(), var, newVar));
        } else if (exp.getArity() == 2) {
            return (renameVariable(exp.getInput(0), var, newVar) || renameVariable(exp.getInput(1), var, newVar));
        } else if (exp instanceof Not) {
            return renameVariable(((Not) exp).getInput(0), var, newVar);
        }
        return false;
    }

    /**
     * Renames a variable in the given term.
     *
     * @param term   the term
     * @param var    the (old) name of the variable to be renamed
     * @param newVar the new name of the variable
     * @return true if there is a variable to be renamed, else false
     */
    private boolean renameVariable(Term term, String var, String newVar) {
        if (term instanceof Variable) {
            Variable v = (Variable) term;
            if (v.name.equals(var)) {
                v.name = newVar;
                return true;
            }
        }
        return false;
    }

    /**
     * Partitions the given boolean expression in such a way that its
     * result consists of three different expressions:
     * 1. all expressions that can be validated on the first schema (s1) only
     * 2. all expressions that can be validated on the second schema  only
     * 3. all expressions, that have to evaluated on both schemata
     *
     * @param b the boolean expression to be partitioned
     * @param s1 the first schema
     * @param s2 the second schema
     * @return the result of the partitioned join predicate
     */
    private PredicatePartitionResult partitionJoinPredicate(BooleanExpression b, List<Attribute> s1, List<Attribute> s2) {
        PredicateNode root = PredicateNode.createNode(b, s1, s2);
        root.mark();
        PredicatePartitionResult result = new PredicatePartitionResult();

        Stack<PredicateNode> tree = new Stack<PredicateNode>();
        tree.push(root);

        while(!tree.isEmpty()) {
            PredicateNode n = tree.pop();

            if (n.getType() == PartitionType.P1) {
                result.add(n.getExpression(), PartitionType.P1);
            } else if (n.getType() == PartitionType.P2) {
                result.add(n.getExpression(), PartitionType.P2);
            } else if (n.getType() == PartitionType.P3) {

                if (n instanceof BinaryPredicateNode) { // Binary node
                    BinaryPredicateNode dbn = (BinaryPredicateNode)n;
                    if (dbn.expression instanceof Or) {
                        result.add(dbn.expression, PartitionType.P3);
                    } else {
                        // And => can be splitted!!
                        tree.push(dbn.rightNode);
                        tree.push(dbn.leftNode);
                    }
                } else {
                    UnaryPredicateNode mn = (UnaryPredicateNode)n;
                    if (mn.expression instanceof Not)
                        tree.push(mn.childNode);
                    else
                        result.add(mn.expression, PartitionType.P3);
                }

            }
        }
        return result;
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////// Helper Classes /////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * The result of a predicate partitioning.
     * The expression p1 refers to the given schema (in case of a join: the left schema),
     * the expression p2 refers to a not defined schema
     * (in case of a join: the right schema)
     * and expression p3 refers to not only to p1
     */
    private static class PredicatePartitionResult {
        /**
         * Expression referring only to the given schema.
         */
        BooleanExpression p1;
        /**
         * Expression referring not at all to the given schema.
         */
        BooleanExpression p2;
        /**
         * Expression referring partially to the given schema.
         */
        BooleanExpression p3;

        /**
         * Adds the given expression to the partition result.
         *
         * @param b the expression
         * @param type defines to which expression the given one
         *             should be added to
         */
        public void add(BooleanExpression b, PartitionType type) {
            switch (type) {
                case P1:
                    if (p1 == null)
                        p1 = b;
                    else
                        p1 = new And(p1,b);
                    break;
                case P2:
                    if (p2 == null)
                        p2 = b;
                    else
                        p2 = new And(p2,b);
                    break;
                case P3:
                    if (p3 == null)
                        p3 = b;
                    else
                        p3 = new And(p3,b);
                    break;
            }
        }
    }

    /**
     * Defines the type of a partition.
     */
    private enum PartitionType {
        P1,P2,P3,CONSTANT
    }

    /**
     * Models an abstract node in the predicate tree.
     */
    private static abstract class PredicateNode {

        protected PartitionType type;
        protected List<Attribute> s1;
        protected List<Attribute> s2;
        public abstract PartitionType mark();

        /**
         * Creates a new PredicateNode for the given schema.
         *
         * @param s1 the schema to partition by
         * @param s2 the second schema
         */
        public PredicateNode(List<Attribute> s1, List<Attribute> s2) {
            this.s1 = s1;
            this.s2 = s2;
        }

        /**
         * Returns this node`s partition type.
         *
         * @return the type of the partition
         */
        public PartitionType getType() {
            return type;
        }

        /**
         * Creates a new Predicate node for the given boolean expression.
         *
         * @param be the boolean expression
         * @param s1 the schema for the first partition
         * @param s2 the second schema
         * @return the predicate node corresponding to the given expression
         */
        public static PredicateNode createNode(BooleanExpression be, List<Attribute> s1, List<Attribute> s2) {
            if (be.getArity() == 2)
                return new BinaryPredicateNode(be,s1,s2);
            else
                return new UnaryPredicateNode(be,s1,s2);
        }

        /**
         * Returns the boolean expression that corresponds to this node.
         *
         * @return the boolean (sub-)expression corresponding to this node
         */
        public abstract BooleanExpression getExpression();

        /**
         * Determines the partition type from to inputs.
         *
         * @param t1 the first partition type
         * @param t2 the second partition type
         * @return the resulting partition type
         */
        protected PartitionType merge(PartitionType t1, PartitionType t2) {
            if (t1 == t2)
                return t1;
            else if (t1 == PartitionType.CONSTANT)
                return t2;
            else if (t2 == PartitionType.CONSTANT)
                return t1;
            else
                return PartitionType.P3;
        }
    }

    /**
     * Models an unary node in the predicate tree.
     */
    private static class UnaryPredicateNode extends PredicateNode {

        protected PredicateNode childNode;
        protected BooleanExpression expression;

        public UnaryPredicateNode(BooleanExpression exp, List<Attribute> s1, List<Attribute> s2) {
            super(s1,s2);
            this.expression = exp;
        }

        @Override
        public PartitionType mark() {

            if (expression instanceof Comparison) {
                Comparison c = (Comparison)expression;
                type = merge( mark(c.getLeft(),true), mark(c.getRight(),false) );
            } else if (expression instanceof Not) {
                Not n = (Not)expression;
                childNode = createNode(n.getInput(0), s1, s2);
                type = childNode.mark();
            }
            return type;
        }

        /**
         * Returns the partition type of a term.
         *
         * @param t the term which partition type should
         *          be determined
         * @return the partition type of t
         */
        protected PartitionType mark(Term t, boolean left) {
            Variable v;
            if (t instanceof Constant)
                return PartitionType.CONSTANT;
            if (t instanceof ArithmeticTerm) {
                ArithmeticTerm at = (ArithmeticTerm)t;
                if (at.getOperation() instanceof ArithmeticOperationTerm) {
                    ArithmeticOperationTerm aot = (ArithmeticOperationTerm) at.getOperation();
                    return mark(aot.getTerm(), left);
                } else
                    return PartitionType.P3; // Cannot be partitioned
            } else
                v = (Variable)t;

            if (left) {
                if (SchemaUtils.containsAttribute(s1, v.name))
                    return PartitionType.P1;
                else
                    return PartitionType.P2;
            } else {
                if (SchemaUtils.containsAttribute(s2, v.name))
                    return PartitionType.P2;
                else
                    return PartitionType.P1;
            }
        }

        @Override
        public BooleanExpression getExpression() {
            return expression;
        }
    }

    /**
     * Models a binary node in the predicate tree.
     */
    private static class BinaryPredicateNode extends PredicateNode {

        protected BooleanExpression expression;
        protected PredicateNode leftNode;
        protected PredicateNode rightNode;

        public BinaryPredicateNode(BooleanExpression exp, List<Attribute> s1, List<Attribute> s2) {
            super(s1,s2);
            this.expression = exp;
        }

        @Override
        public BooleanExpression getExpression() {
            return expression;
        }

        @Override
        public PartitionType mark() {
            leftNode = createNode(expression.getInput(0), s1, s2);
            rightNode = createNode(expression.getInput(1), s1, s2);
            type = merge(leftNode.mark(), rightNode.mark());
            return type;
        }
    }

    // ==============================================================

}


