package de.umr.ep.queryengine.optimization;

import de.umr.ep.operators.Selection;
import de.umr.ep.queryengine.LogicalOperator;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by seidemann on 03.02.2017.
 */
public class LogicalOptimizer {

    private Map<Class, Optimization> optimizations;

    public LogicalOptimizer() {
        optimizations = new HashMap<>();
        init();
    }

    public LogicalOperator optimizePlan(LogicalOperator plan) {
        optimize(plan);
        plan = new CorrelationOptimizer().optimize(plan);
        plan = new SelectionPushDownOptimizer().optimize(plan);
        return plan;
    }

    void optimize(LogicalOperator operator) {
        if (optimizations.containsKey(operator.getClass()))
            optimizations.get(operator.getClass()).optimize(operator, this);
        for (int i = 0; i < operator.getInputCount(); i++) {
            optimize(operator.getInput(i));
        }
    }

    private void init() {
        optimizations.put(Selection.class, new Optimization<Selection>() {
            @Override
            public void optimize(Selection operator, LogicalOptimizer optimizer) {
                operator.setExpression(new BooleanExpressionOptimizer().optimize(operator.getExpression()));
            }
        });
    }

}
