package de.umr.ep.queryengine.predicates;

/**
 * Boolean AND operator.
 *
 * @author Marc Seidemann
 */
public class And implements BooleanExpression {

    private BooleanExpression left;
    private BooleanExpression right;

    /**
     * Creates the expression left AND right.
     * @param left the left predicate
     * @param right the right predicate
     */
    public And(BooleanExpression left, BooleanExpression right) {
        this.left = left;
        this.right = right;
    }

    @Override
    public int getArity() {
        return 2;
    }

    @Override
    public BooleanExpression getInput(int i) {
        switch (i) {
            case 0: return left;
            case 1: return right;
            default: throw new IndexOutOfBoundsException();
        }
    }

    @Override
    public void setInput(int i, BooleanExpression expression) {
        if (i < 0 || i >= getArity())
            throw new IndexOutOfBoundsException();
        else if (i == 0)
            left = expression;
        else
            right = expression;
    }

    @Override
    public String toString() {
        return left.toString() + " AND " + right.toString();
    }
}
