/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.queryengine.predicates;

/**
 * Models a binary comparison operator.
 *
 * @author Marc Seidemann
 */
public abstract class Comparison extends Predicate {

    /** The left term */
    protected Term left;
    /** The right term */
    protected Term right;

    /**
     * Creates a new comparison for the given terms.
     *
     * @param left the left term
     * @param right the right term
     */
    public Comparison(Term left, Term right) {
        this.left = left;
        this.right = right;
    }

    /**
     * Returns the left term.
     *
     * @return the left term
     */
    public Term getLeft() {
        return left;
    }

    /**
     * Returns the right term.
     *
     * @return the right term
     */
    public Term getRight() {
        return right;
    }

    /**
     * Sets the left term.
     *
     * @param left the left term
     */
    public void setLeft(Term left) {
        this.left = left;
    }

    /**
     * Sets the right term
     *
     * @param right the right term
     */
    public void setRight(Term right) {
        this.right = right;
    }

    @Override
    public int getArity() {
        return 0;
    }

    @Override
    public BooleanExpression getInput(int i) {
        throw new IndexOutOfBoundsException();
    }

    @Override
    public void setInput(int i, BooleanExpression expression) {
        throw new IndexOutOfBoundsException();
    }
}
