/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.queryengine.predicates;

/**
 * Boolean EXISTS operator.
 *
 * @author Marc Seidemann
 */
public class Exists extends Predicate {

    /**
     * The subquery.
     */
    protected ScalarSubquery subquery;

    /**
     * Creates a new exists operator with the given subquery.
     *
     * @param subquery the subquery for the exists operator
     */
    public Exists(ScalarSubquery subquery) {
        this.subquery = subquery;
    }

    @Override
    public BooleanExpression clone() {return new Exists(subquery);}

    /**
     * Returns the subquery of the exists clause.
     *
     * @return the subquery
     */
    public ScalarSubquery getSubquery() {
        return subquery;
    }

    @Override
    public int getArity() {
        return 0;
    }

    @Override
    public BooleanExpression getInput(int i) {
        return null;
    }
}
