/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.queryengine.predicates;

/**
 * Boolean NOT operator.
 *
 * @author Marc Seidemann
 */
public class Not implements BooleanExpression {

    /** The predicate predicate */
    protected BooleanExpression expression;

    /**
     * Creates the expression NOT predicate.
     * @param expression the predicate
     */
    public Not(BooleanExpression expression) {
        this.expression = expression;
    }

    @Override
    public String toString() {
        return "NOT ("+expression.toString()+")";
    }


    @Override
    public int getArity() {
        return 1;
    }

    @Override
    public BooleanExpression getInput(int i) {
        if (i == 0)
            return expression;
        else throw new IndexOutOfBoundsException();
    }

    @Override
    public void setInput(int i, BooleanExpression expression) {
        if (i == 0)
            this.expression = expression;
        else throw new IndexOutOfBoundsException();
    }
}
