/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.queryengine.predicates;

/**
 * Boolean OR operator.
 *
 * @author Marc Seidemann
 */
public class Or implements BooleanExpression {

    private BooleanExpression left;
    private BooleanExpression right;

    /**
     * Creates the expression left OR right.
     * @param left the left predicate
     * @param right the right predicate
     */
    public Or(BooleanExpression left,BooleanExpression right) {
        this.left = left;
        this.right = right;
    }

    @Override
    public String toString() {
        return "("+left.toString()+") OR ("+right.toString()+")";
    }

    @Override
    public int getArity() {
        return 2;
    }

    @Override
    public BooleanExpression getInput(int i) {
        switch (i) {
            case 0: return left;
            case 1: return right;
            default: throw new IndexOutOfBoundsException();
        }
    }

    @Override
    public void setInput(int i, BooleanExpression expression) {
        if (i < 0 || i >= getArity())
            throw new IndexOutOfBoundsException();
        else if (i == 0)
            left = expression;
        else
            right = expression;
    }
}
