/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.queryengine.predicates.arithmetic;

import de.umr.ep.common.Attribute;
import de.umr.ep.common.ComparableNumber;
import de.umr.ep.common.SchemaUtils;
import de.umr.ep.queryengine.predicates.Constant;
import de.umr.ep.queryengine.predicates.Term;
import de.umr.ep.queryengine.predicates.Variable;

import java.util.List;

/**
 * Represents an arithmetic term.
 *
 * @author Marc Seidemann
 */
public class ArithmeticOperationTerm extends ArithmeticOperation {

    /**
     * Stores the term.
     */
    protected Term term;

    /**
     * Creates a new arithmetic term.
     *
     * @param term the term
     */
    public ArithmeticOperationTerm(Term term) {
        this.term = term;
    }

    /**
     * Returns the nested term.
     *
     * @return the term
     */
    public Term getTerm() {
        return term;
    }

    @Override
    public Attribute.DataType getType(List<Attribute> s) {
        if (term instanceof Variable) {
            Variable v = (Variable) term;
            return SchemaUtils.getAttribute(s, v.name).getType();
        } else {
            Constant c = (Constant) term;
            Number n = ((ComparableNumber)c.getConstantValue()).getNumber();
            if (n instanceof Double)
                return Attribute.DataType.DOUBLE;
            else
                return Attribute.DataType.LONG;
        }
    }

    @Override
    public ArithmeticOperation clone() {
        return new ArithmeticOperationTerm(term);
    }

    @Override
    public int getArity() {
        return 0;
    }

    @Override
    public ArithmeticOperation getInput(int i) {
        throw new IndexOutOfBoundsException();
    }

    @Override
    public void setInput(int i, ArithmeticOperation operation) {
        throw new IndexOutOfBoundsException();
    }

    @Override
    public String toString() {
        return term.toString();
    }
}
