/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.queryengine.predicates.arithmetic;

import de.umr.ep.common.Attribute;
import de.umr.ep.queryengine.predicates.Term;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Represents an arithmetic term.
 *
 * @author Marc Seidemann
 */
public class ArithmeticTerm extends Term {

    /**
     * The arithmetic operation
     */
    private ArithmeticOperation operation;

    /**
     * Creates a new arithmetic term with the given arithmetic operation.
     *
     * @param operation the arithmetic operation
     */
    public ArithmeticTerm(ArithmeticOperation operation) {
        this.operation = operation;
    }

    /**
     * Returns the arithmetic operation.
     *
     * @return the arithmetic operation
     */
    public ArithmeticOperation getOperation() {
        return operation;
    }


    @Override
    public Term clone() {
        return new ArithmeticTerm(operation.clone());
    }

    @Override
    public String toString() {
        return operation.toString();
    }
}
