/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.queryengine.predicates.arithmetic;

import de.umr.ep.common.Attribute;

import java.util.List;

/**
 * Models a nested operation.
 *
 * @author Marc Seidemann
 */
public class NestedOperation extends ArithmeticOperation {

    /**
     * The nested operation.
     */
    protected ArithmeticOperation nested;

    /**
     * Creates a new nested operation.
     *
     * @param nested the nested operation
     */
    public NestedOperation(ArithmeticOperation nested) {
        this.nested = nested;
    }

    /**
     * Returns the nested operation.
     *
     * @return the nested operation
     */
    public ArithmeticOperation getNested() {
        return nested;
    }

    @Override
    public Attribute.DataType getType(List<Attribute> s) {
        return nested.getType(s);
    }


    @Override
    public ArithmeticOperation clone() {
        return new NestedOperation(nested.clone());
    }

    @Override
    public int getArity() {
        return 1;
    }

    @Override
    public ArithmeticOperation getInput(int i) {
        if (i == 0)
            return nested;
        else
            throw new IndexOutOfBoundsException();
    }

    @Override
    public void setInput(int i, ArithmeticOperation operation) {
        if (i == 0)
            this.nested = operation;
        else
            throw new IndexOutOfBoundsException();
    }

    @Override
    public String toString() {
        return "("+nested.toString()+")";
    }
}
