/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.queryengine.predicates.arithmetic;


import de.umr.ep.common.Attribute;

import java.util.List;

/**
 * Models a subtraction operation.
 *
 * @author Marc Seidemann
 */
public class Subtract extends ArithmeticOperation {

    protected ArithmeticOperation left, right;

    /**
     * Creates a new subtraction operation.
     *
     * @param left the minuend
     * @param right the subtrahend
     */
    public Subtract(ArithmeticOperation left, ArithmeticOperation right) {
        this.left = left;
        this.right = right;
    }

    @Override
    public Attribute.DataType getType(List<Attribute> s) {
        if (left.getType(s) == right.getType(s))
            return left.getType(s);
        else
            return Attribute.DataType.DOUBLE;
    }

    @Override
    public ArithmeticOperation clone() {
        return new Subtract(left.clone(), right.clone());
    }

    @Override
    public int getArity() {
        return 2;
    }

    @Override
    public ArithmeticOperation getInput(int i) {
        switch(i) {
            case 0: return left;
            case 1: return right;
            default: throw new IndexOutOfBoundsException();
        }
    }

    @Override
    public void setInput(int i, ArithmeticOperation operation) {
        switch(i) {
            case 0: this.left = operation; break;
            case 1: this.right = operation; break;
            default: throw new IndexOutOfBoundsException();
        }
    }

    @Override
    public String toString() {
        return left.toString()+"-"+right.toString();
    }
}
