/*
 * Copyright (c) 2015
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.queryengine.predicates.pattern;

import java.util.List;

/**
 * Implementation of the Kleene star operator.
 *
 * @author Marc Seidemann
 */
public class KleeneStar extends Pattern {

    /**
     * The pattern to be nested.
     */
    private Pattern pattern;

    /**
     * Creates a Kleene star, i.e. (pattern)*.
     *
     * @param pattern the pattern to be nested
     */
    public KleeneStar(Pattern pattern) {
        this.pattern = pattern;
    }

    @Override
    public String getPattern() {
        return "("+pattern.getPattern()+")*";
    }

    /**
     * Returns the nested pattern.
     *
     * @return the nested pattern
     */
    public Pattern getNestedPattern() {
        return pattern;
    }

    /**
     * Returns the symbol definitions used in this pattern.
     *
     * @return the symbol definitions used in this pattern.
     */
    public List<Symbol> getSymbolDefinition() {
        return pattern.getSymbolDefinition();
    }

    @Override
    public Pattern copy() {
        return new KleeneStar(pattern.copy());
    }
}
