/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.queryengine.predicates.pattern;

/**
 * Models an attribute declaration in the measures clause.
 *
 * @author Marc Seidemann
 */
public class MeasureAttribute {

    /**
     * The measurement functions.
     */
    public enum MeasureFunction {
        LAST,
        FIRST
    }

    /**
     * The name of the symbol this attribute refers to.
     */
    private String symbolName;
    /**
     * The identifier of the symbol`s attribute
     */
    private String attributeName;
    /**
     * The optional renaming.
     */
    private String renaming;
    /**
     * The optional measurement function.
     */
    private MeasureFunction function;

    public MeasureAttribute(String symbolName, String attributeName, String renaming) {
        this.symbolName = symbolName;
        this.attributeName = attributeName;
        this.renaming = renaming;
    }

    /**
     * Returns the symbol name.
     *
     * @return the name of the symbol
     */
    public String getSymbolName() {
        return symbolName;
    }

    public void setSymbolName(String symbolName) {
        this.symbolName = symbolName;
    }

    /**
     * Returns the attribute`s name.
     *
     * @return the name of the attribute
     */
    public String getAttributeName() {
        return attributeName;
    }

    public void setAttributeName(String attributeName) {
        this.attributeName = attributeName;
    }

    public String getRenaming() {
        return renaming;
    }

    public void setRenaming(String renaming) {
        this.renaming = renaming;
    }

    public MeasureFunction getFunction() {
        return function;
    }

    public void setFunction(MeasureFunction function) {
        this.function = function;
    }
}
