/*
 * Copyright (c) 2015
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.queryengine.predicates.pattern;

import java.util.List;

/**
 * Abstraction of a pattern.
 *
 * @author Marc Seidemann
 */
public abstract class Pattern {

    /**
     * Returns the string representation of the pattern as
     * regular expression (for automaton transformation).
     *
     * @return the regular expression describing the pattern
     */
    public abstract String getPattern();

    /**
     * Returns the set of symbols contained in the given pattern, ordered ascending by occurence
     *
     * @return the set of contained symbols
     */
    public abstract List<Symbol> getSymbolDefinition();

    /**
     * Deep-copies the given pattern
     * @return
     */
    public abstract Pattern copy();

}
