package de.umr.ep.queryengine.predicates.pattern;

import java.util.ArrayList;
import java.util.List;

/**
 * Represents a pattern definition, i.e. a sequence of patterns.
 *
 * @author Marc Seidemann
 */
public class PatternDefinition extends Pattern {

    /**
     * The patterns of this pattern definition
     */
    protected List<Pattern> patterns;

    /**
     * Creates a new pattern definition.
     *
     *  @param patterns the pattern sequence
     */
    public PatternDefinition(List<Pattern> patterns) {
        this.patterns = patterns;
    }

    public PatternDefinition(Pattern ... patterns) {
        this.patterns = new ArrayList<>(patterns.length);
        for (int i = 0; i < patterns.length; i++) {
            this.patterns.add(patterns[i]);
        }
    }

    /**
     * Returns the sequence of patterns represented by this pattern
     *
     * @return the pattern sequence.
     */
    public List<Pattern> getPatterns() {
        return patterns;
    }

    /**
     * Returns the string representation of this pattern.
     *
     * @return the string representation of the pattern
     */
    public String getPattern() {
        StringBuilder res = new StringBuilder();
        for (int i = 0; i < patterns.size(); i++) {
            res.append(patterns.get(i).getPattern());
        }
        return res.toString();
    }


    @Override
    public List<Symbol> getSymbolDefinition() {
        List<Symbol> result = new ArrayList<>();
        for (int i = 0; i < patterns.size(); i++) {
            result.addAll(patterns.get(i).getSymbolDefinition());
        }
        return result;
    }

    @Override
    public Pattern copy() {
        List<Pattern> copied = new ArrayList<>(patterns.size());
        for (int i = 0; i < patterns.size(); i++) {
            copied.add(patterns.get(i).copy());
        }
        return new PatternDefinition(copied);
    }
}
