/*
 * Copyright (c) 2015
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.queryengine.predicates.pattern;

/**
 * Implementation of a pattern variable.
 *
 * @author Marc Seidemann
 */
public class PatternVariable {
    /**
     * The name of the variable
     */
    private String name;
    /**
     * The bound attribute
     */
    private String attribute;

    /**
     * Creates a variable for the given attribute
     * @param name the name of the variable
     * @param attribute the attribute to be bound
     */
    public PatternVariable(String name, String attribute) {
        this.name = name;
        this.attribute = attribute;
    }

    /**
     * Returns the identifier of the variable.
     *
     * @return the identifier (name) of the variable
     */
    public String getName() {
        return name;
    }

    /**
     * Returns the attribute to be bound.
     *
     * @return the identifier of the attribute to be bound
     */
    public String getAttribute() {
        return attribute;
    }

    @Override
    public String toString() {
        return name+" <- "+attribute;
    }
}
