/*
 * Copyright (c) 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.ep.queryengine.predicates.spatial;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.io.ParseException;
import com.vividsolutions.jts.io.WKTReader;
import de.umr.ep.queryengine.predicates.Constant;

/**
 * Models a constant for geometry values.
 *
 * @author Marc Seidemann
 */
public class GeometryConstant extends Constant {

    /**
     * Creates a new geometry constant value
     *
     * @param value the constant value
     */
    public GeometryConstant(String value) {
        super(value);
        WKTReader reader = new WKTReader();
        try {
            this.value = reader.read(value);
        } catch (ParseException e) {
            throw new RuntimeException("Error parsing geometry constant: "+value);
        }
    }

    /**
     * Creates a new geometry constant value
     * @param value
     */
    public GeometryConstant(Geometry value) {
        super(value);
    }

    @Override
    public Geometry getConstantValue() {
        return (Geometry) value;
    }
}
