/**
 * 
 */
package de.umr.ep.queryengine.ql.grammar;

import org.antlr.v4.runtime.ParserRuleContext;

import de.umr.ep.queryengine.ql.generated.EPQLBaseListener;
import de.umr.ep.queryengine.ql.generated.EPQLParser.*;

/**
 * @author Michael Körber
 *
 */
public class LoggingEPQLListener extends EPQLBaseListener {

    private int depth = 0;

    private void logEnter() {
        log(false);
    }

    private void logExit() {
        log(true);
    }

    private void log(boolean exit) {
        if ( exit ) depth--;
        StringBuilder message = new StringBuilder();
        for (int i = 0; i < 2 * depth; i++)
            message.append(' ');
        message.append(exit ? "Exit:  " : "Enter: ").append(Thread.currentThread().getStackTrace()[3].getMethodName());
        System.out.println(message.toString());
        if ( !exit ) depth++;
    }

    @Override
    public void enterStart(StartContext ctx) {
        logEnter();
    }

    @Override
    public void exitStart(StartContext ctx) {

        logExit();
    }

    @Override
    public void enterStatement(StatementContext ctx) {
        logEnter();
    }

    @Override
    public void exitStatement(StatementContext ctx) {

        logExit();
    }

    @Override
    public void enterSelectClause(SelectClauseContext ctx) {
        logEnter();
    }

    @Override
    public void exitSelectClause(SelectClauseContext ctx) {

        logExit();
    }

    @Override
    public void enterFromClause(FromClauseContext ctx) {
        logEnter();
    }

    @Override
    public void exitFromClause(FromClauseContext ctx) {

        logExit();
    }

    @Override
    public void enterJoinClause(JoinClauseContext ctx) {
        logEnter();
    }

    @Override
    public void exitJoinClause(JoinClauseContext ctx) {

        logExit();
    }

    @Override
    public void enterBoundIdentifier(BoundIdentifierContext ctx) {
        logEnter();
    }

    @Override
    public void exitBoundIdentifier(BoundIdentifierContext ctx) {

        logExit();
    }

    @Override
    public void enterIdentifier(IdentifierContext ctx) {
//        logEnter();
    }

    @Override
    public void exitIdentifier(IdentifierContext ctx) {

//        logExit();
    }

    @Override
    public void enterAttribute(AttributeContext ctx) {
        logEnter();
    }

    @Override
    public void exitAttribute(AttributeContext ctx) {

        logExit();
    }

    @Override
    public void enterOrderByEntry(OrderByEntryContext ctx) {
        logEnter();
    }

    @Override
    public void exitOrderByEntry(OrderByEntryContext ctx) {

        logExit();
    }

    @Override
    public void enterOrderByList(OrderByListContext ctx) {

        logEnter();
    }

    @Override
    public void exitOrderByList(OrderByListContext ctx) {

        logExit();
    }

    @Override
    public void enterWhereClause(WhereClauseContext ctx) {

        logEnter();
    }

    @Override
    public void exitWhereClause(WhereClauseContext ctx) {

        logExit();
    }

    @Override
    public void enterGroupByAttribute(GroupByAttributeContext ctx) {

        logEnter();
    }

    @Override
    public void exitGroupByAttribute(GroupByAttributeContext ctx) {

        logExit();
    }

    @Override
    public void enterGroupByClause(GroupByClauseContext ctx) {

        logEnter();
    }

    @Override
    public void exitGroupByClause(GroupByClauseContext ctx) {

        logExit();
    }

    @Override
    public void enterHavingClause(HavingClauseContext ctx) {

        logEnter();
    }

    @Override
    public void exitHavingClause(HavingClauseContext ctx) {

        logExit();
    }

    @Override
    public void enterLimitClause(LimitClauseContext ctx) {

        logEnter();
    }

    @Override
    public void exitLimitClause(LimitClauseContext ctx) {

        logExit();
    }

    @Override
    public void enterOrderByClause(OrderByClauseContext ctx) {

        logEnter();
    }

    @Override
    public void exitOrderByClause(OrderByClauseContext ctx) {

        logExit();
    }

    @Override
    public void enterWithinClause(WithinClauseContext ctx) {

        logEnter();
    }

    @Override
    public void exitWithinClause(WithinClauseContext ctx) {

        logExit();
    }

    @Override
    public void enterWindowClause(WindowClauseContext ctx) {

        logEnter();
    }

    @Override
    public void exitWindowClause(WindowClauseContext ctx) {

        logExit();
    }

    @Override
    public void enterNowTime(NowTimeContext ctx) {

        logEnter();
    }

    @Override
    public void exitNowTime(NowTimeContext ctx) {

        logExit();
    }

    @Override
    public void enterNumberTime(NumberTimeContext ctx) {

        logEnter();
    }

    @Override
    public void exitNumberTime(NumberTimeContext ctx) {

        logExit();
    }

    @Override
    public void enterExpressionTime(ExpressionTimeContext ctx) {

        logEnter();
    }

    @Override
    public void exitExpressionTime(ExpressionTimeContext ctx) {

        logExit();
    }

    @Override
    public void enterTimeNumber(TimeNumberContext ctx) {

        logEnter();
    }

    @Override
    public void exitTimeNumber(TimeNumberContext ctx) {

        logExit();
    }

    @Override
    public void enterStream(StreamContext ctx) {

        logEnter();
    }

    @Override
    public void exitStream(StreamContext ctx) {

        logExit();
    }

    @Override
    public void enterAggregateIdentifier(AggregateIdentifierContext ctx) {

        logEnter();
    }

    @Override
    public void exitAggregateIdentifier(AggregateIdentifierContext ctx) {

        logExit();
    }

    @Override
    public void enterAggregate(AggregateContext ctx) {

        logEnter();
    }

    @Override
    public void exitAggregate(AggregateContext ctx) {

        logExit();
    }

    @Override
    public void enterOuterQuery(OuterQueryContext ctx) {

        logEnter();
    }

    @Override
    public void exitOuterQuery(OuterQueryContext ctx) {

        logExit();
    }

    @Override
    public void enterUnionQuery(UnionQueryContext ctx) {

        logEnter();
    }

    @Override
    public void exitUnionQuery(UnionQueryContext ctx) {

        logExit();
    }

    @Override
    public void enterExceptQuery(ExceptQueryContext ctx) {

        logEnter();
    }

    @Override
    public void exitExceptQuery(ExceptQueryContext ctx) {

        logExit();
    }

    @Override
    public void enterOuterQueryPart(OuterQueryPartContext ctx) {

        logEnter();
    }

    @Override
    public void exitOuterQueryPart(OuterQueryPartContext ctx) {

        logExit();
    }

    @Override
    public void enterSubQueryRule(SubQueryRuleContext ctx) {

        logEnter();
    }

    @Override
    public void exitSubQueryRule(SubQueryRuleContext ctx) {

        logExit();
    }

    @Override
    public void enterJoin(JoinContext ctx) {

        logEnter();
    }

    @Override
    public void exitJoin(JoinContext ctx) {

        logExit();
    }

    @Override
    public void enterStreamRule(StreamRuleContext ctx) {

        logEnter();
    }

    @Override
    public void exitStreamRule(StreamRuleContext ctx) {

        logExit();
    }

    @Override
    public void enterSubqueryStatement(SubqueryStatementContext ctx) {

        logEnter();
    }

    @Override
    public void exitSubqueryStatement(SubqueryStatementContext ctx) {

        logExit();
    }

    @Override
    public void enterSubqueryJoin(SubqueryJoinContext ctx) {

        logEnter();
    }

    @Override
    public void exitSubqueryJoin(SubqueryJoinContext ctx) {

        logExit();
    }

    @Override
    public void enterAggAtt(AggAttContext ctx) {

        logEnter();
    }

    @Override
    public void exitAggAtt(AggAttContext ctx) {

        logExit();
    }

    @Override
    public void enterPlainAtt(PlainAttContext ctx) {

        logEnter();
    }

    @Override
    public void exitPlainAtt(PlainAttContext ctx) {

        logExit();
    }

    @Override
    public void enterArithAtt(ArithAttContext ctx) {

        logEnter();
    }

    @Override
    public void exitArithAtt(ArithAttContext ctx) {

        logExit();
    }

    @Override
    public void enterSpatialAtt(SpatialAttContext ctx) {

        logEnter();
    }

    @Override
    public void exitSpatialAtt(SpatialAttContext ctx) {

        logExit();
    }

    @Override
    public void enterProjectionList(ProjectionListContext ctx) {

        logEnter();
    }

    @Override
    public void exitProjectionList(ProjectionListContext ctx) {

        logExit();
    }

    @Override
    public void enterAtom(AtomContext ctx) {

        logEnter();
    }

    @Override
    public void exitAtom(AtomContext ctx) {

        logExit();
    }

    @Override
    public void enterAtomAttribute(AtomAttributeContext ctx) {

        logEnter();
    }

    @Override
    public void exitAtomAttribute(AtomAttributeContext ctx) {

        logExit();
    }

    @Override
    public void enterPrevAtomAttribute(PrevAtomAttributeContext ctx) {

        logEnter();
    }

    @Override
    public void exitPrevAtomAttribute(PrevAtomAttributeContext ctx) {

        logExit();
    }

    @Override
    public void enterAtomVal(AtomValContext ctx) {

        logEnter();
    }

    @Override
    public void exitAtomVal(AtomValContext ctx) {

        logExit();
    }

    @Override
    public void enterAtomString(AtomStringContext ctx) {

        logEnter();
    }

    @Override
    public void exitAtomString(AtomStringContext ctx) {

        logExit();
    }

    @Override
    public void enterAtomNum(AtomNumContext ctx) {

        logEnter();
    }

    @Override
    public void exitAtomNum(AtomNumContext ctx) {

        logExit();
    }

    @Override
    public void enterGeometryAtom(GeometryAtomContext ctx) {

        logEnter();
    }

    @Override
    public void exitGeometryAtom(GeometryAtomContext ctx) {

        logExit();
    }

    @Override
    public void enterAtomGeometry(AtomGeometryContext ctx) {

        logEnter();
    }

    @Override
    public void exitAtomGeometry(AtomGeometryContext ctx) {

        logExit();
    }

    @Override
    public void enterAExpList(AExpListContext ctx) {

        logEnter();
    }

    @Override
    public void exitAExpList(AExpListContext ctx) {

        logExit();
    }

    @Override
    public void enterArithOpExp(ArithOpExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitArithOpExp(ArithOpExpContext ctx) {

        logExit();
    }

    @Override
    public void enterArithAtomExp(ArithAtomExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitArithAtomExp(ArithAtomExpContext ctx) {

        logExit();
    }

    @Override
    public void enterArithmeticFunc(ArithmeticFuncContext ctx) {

        logEnter();
    }

    @Override
    public void exitArithmeticFunc(ArithmeticFuncContext ctx) {

        logExit();
    }

    @Override
    public void enterArithParExp(ArithParExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitArithParExp(ArithParExpContext ctx) {

        logExit();
    }

    @Override
    public void enterArithmeticFunction(ArithmeticFunctionContext ctx) {

        logEnter();
    }

    @Override
    public void exitArithmeticFunction(ArithmeticFunctionContext ctx) {

        logExit();
    }

    @Override
    public void enterIntervalList(IntervalListContext ctx) {

        logEnter();
    }

    @Override
    public void exitIntervalList(IntervalListContext ctx) {

        logExit();
    }

    @Override
    public void enterIntervalRange(IntervalRangeContext ctx) {

        logEnter();
    }

    @Override
    public void exitIntervalRange(IntervalRangeContext ctx) {

        logExit();
    }

    @Override
    public void enterASubQueryExp(ASubQueryExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitASubQueryExp(ASubQueryExpContext ctx) {

        logExit();
    }

    @Override
    public void enterArithExpRule(ArithExpRuleContext ctx) {

        logEnter();
    }

    @Override
    public void exitArithExpRule(ArithExpRuleContext ctx) {

        logExit();
    }

    @Override
    public void enterArithSubQueryRule(ArithSubQueryRuleContext ctx) {

        logEnter();
    }

    @Override
    public void exitArithSubQueryRule(ArithSubQueryRuleContext ctx) {

        logExit();
    }

    @Override
    public void enterHavingArithExpRule(HavingArithExpRuleContext ctx) {

        logEnter();
    }

    @Override
    public void exitHavingArithExpRule(HavingArithExpRuleContext ctx) {

        logExit();
    }

    @Override
    public void enterHavingArithSubQueryRule(HavingArithSubQueryRuleContext ctx) {

        logEnter();
    }

    @Override
    public void exitHavingArithSubQueryRule(HavingArithSubQueryRuleContext ctx) {

        logExit();
    }

    @Override
    public void enterUnaryBoolSpatialExp(UnaryBoolSpatialExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitUnaryBoolSpatialExp(UnaryBoolSpatialExpContext ctx) {

        logExit();
    }

    @Override
    public void enterBoolExistsExp(BoolExistsExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitBoolExistsExp(BoolExistsExpContext ctx) {

        logExit();
    }

    @Override
    public void enterBoolOpExp(BoolOpExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitBoolOpExp(BoolOpExpContext ctx) {

        logExit();
    }

    @Override
    public void enterBoolRelExp(BoolRelExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitBoolRelExp(BoolRelExpContext ctx) {

        logExit();
    }

    @Override
    public void enterBoolInExp(BoolInExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitBoolInExp(BoolInExpContext ctx) {

        logExit();
    }

    @Override
    public void enterTrueBoolExp(TrueBoolExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitTrueBoolExp(TrueBoolExpContext ctx) {

        logExit();
    }

    @Override
    public void enterBoolBetweenExp(BoolBetweenExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitBoolBetweenExp(BoolBetweenExpContext ctx) {

        logExit();
    }

    @Override
    public void enterBoolParExp(BoolParExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitBoolParExp(BoolParExpContext ctx) {

        logExit();
    }

    @Override
    public void enterBoolSpatialExp(BoolSpatialExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitBoolSpatialExp(BoolSpatialExpContext ctx) {

        logExit();
    }

    @Override
    public void enterFalseBoolExp(FalseBoolExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitFalseBoolExp(FalseBoolExpContext ctx) {

        logExit();
    }

    @Override
    public void enterNegatedBoolOp(NegatedBoolOpContext ctx) {

        logEnter();
    }

    @Override
    public void exitNegatedBoolOp(NegatedBoolOpContext ctx) {

        logExit();
    }

    @Override
    public void enterHavingArithParExp(HavingArithParExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitHavingArithParExp(HavingArithParExpContext ctx) {

        logExit();
    }

    @Override
    public void enterHavingaggregateExpression(HavingaggregateExpressionContext ctx) {

        logEnter();
    }

    @Override
    public void exitHavingaggregateExpression(HavingaggregateExpressionContext ctx) {

        logExit();
    }

    @Override
    public void enterHavingArithOpExp(HavingArithOpExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitHavingArithOpExp(HavingArithOpExpContext ctx) {

        logExit();
    }

    @Override
    public void enterHavingArithAtomExp(HavingArithAtomExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitHavingArithAtomExp(HavingArithAtomExpContext ctx) {

        logExit();
    }

    @Override
    public void enterHavingNegatedBoolOp(HavingNegatedBoolOpContext ctx) {

        logEnter();
    }

    @Override
    public void exitHavingNegatedBoolOp(HavingNegatedBoolOpContext ctx) {

        logExit();
    }

    @Override
    public void enterHavingBoolParExp(HavingBoolParExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitHavingBoolParExp(HavingBoolParExpContext ctx) {

        logExit();
    }

    @Override
    public void enterHavingFalseBoolExp(HavingFalseBoolExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitHavingFalseBoolExp(HavingFalseBoolExpContext ctx) {

        logExit();
    }

    @Override
    public void enterHavingBoolOpExp(HavingBoolOpExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitHavingBoolOpExp(HavingBoolOpExpContext ctx) {

        logExit();
    }

    @Override
    public void enterHavingBoolInExp(HavingBoolInExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitHavingBoolInExp(HavingBoolInExpContext ctx) {

        logExit();
    }

    @Override
    public void enterHavingBoolRelExp(HavingBoolRelExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitHavingBoolRelExp(HavingBoolRelExpContext ctx) {

        logExit();
    }

    @Override
    public void enterHavingTrueBoolExp(HavingTrueBoolExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitHavingTrueBoolExp(HavingTrueBoolExpContext ctx) {

        logExit();
    }

    @Override
    public void enterHavingBoolBetweenExp(HavingBoolBetweenExpContext ctx) {

        logEnter();
    }

    @Override
    public void exitHavingBoolBetweenExp(HavingBoolBetweenExpContext ctx) {

        logExit();
    }

    @Override
    public void enterSpatialFunction(SpatialFunctionContext ctx) {

        logEnter();
    }

    @Override
    public void exitSpatialFunction(SpatialFunctionContext ctx) {

        logExit();
    }

    @Override
    public void enterMatchRecognizeClause(MatchRecognizeClauseContext ctx) {

        logEnter();
    }

    @Override
    public void exitMatchRecognizeClause(MatchRecognizeClauseContext ctx) {

        logExit();
    }

    @Override
    public void enterPatternClause(PatternClauseContext ctx) {

        logEnter();
    }

    @Override
    public void exitPatternClause(PatternClauseContext ctx) {

        logExit();
    }

    @Override
    public void enterPatternDefinition(PatternDefinitionContext ctx) {

        logEnter();
    }

    @Override
    public void exitPatternDefinition(PatternDefinitionContext ctx) {

        logExit();
    }

    @Override
    public void enterPatternList(PatternListContext ctx) {

        logEnter();
    }

    @Override
    public void exitPatternList(PatternListContext ctx) {

        logExit();
    }

    @Override
    public void enterPatternSymbol(PatternSymbolContext ctx) {

        logEnter();
    }

    @Override
    public void exitPatternSymbol(PatternSymbolContext ctx) {

        logExit();
    }

    @Override
    public void enterDefineClause(DefineClauseContext ctx) {

        logEnter();
    }

    @Override
    public void exitDefineClause(DefineClauseContext ctx) {

        logExit();
    }

    @Override
    public void enterSymbolDefinition(SymbolDefinitionContext ctx) {

        logEnter();
    }

    @Override
    public void exitSymbolDefinition(SymbolDefinitionContext ctx) {

        logExit();
    }

    @Override
    public void enterMeasureClause(MeasureClauseContext ctx) {

        logEnter();
    }

    @Override
    public void exitMeasureClause(MeasureClauseContext ctx) {

        logExit();
    }

    @Override
    public void enterMeasureAttributeList(MeasureAttributeListContext ctx) {

        logEnter();
    }

    @Override
    public void exitMeasureAttributeList(MeasureAttributeListContext ctx) {

        logExit();
    }

    @Override
    public void enterLastMeasureAttribute(LastMeasureAttributeContext ctx) {

        logEnter();
    }

    @Override
    public void exitLastMeasureAttribute(LastMeasureAttributeContext ctx) {

        logExit();
    }

    @Override
    public void enterFirstMeasureAttribute(FirstMeasureAttributeContext ctx) {

        logEnter();
    }

    @Override
    public void exitFirstMeasureAttribute(FirstMeasureAttributeContext ctx) {

        logExit();
    }

    @Override
    public void enterPlainMeasureAttribute(PlainMeasureAttributeContext ctx) {

        logEnter();
    }

    @Override
    public void exitPlainMeasureAttribute(PlainMeasureAttributeContext ctx) {

        logExit();
    }

    @Override
    public void enterDropStreamStatement(DropStreamStatementContext ctx) {

        logEnter();
    }

    @Override
    public void exitDropStreamStatement(DropStreamStatementContext ctx) {

        logExit();
    }

    @Override
    public void enterCreateStreamStatement(CreateStreamStatementContext ctx) {

        logEnter();
    }

    @Override
    public void exitCreateStreamStatement(CreateStreamStatementContext ctx) {

        logExit();
    }

    @Override
    public void enterAttributeDefinitionList(AttributeDefinitionListContext ctx) {

        logEnter();
    }

    @Override
    public void exitAttributeDefinitionList(AttributeDefinitionListContext ctx) {

        logExit();
    }

    @Override
    public void enterAttributeDefinition(AttributeDefinitionContext ctx) {

        logEnter();
    }

    @Override
    public void exitAttributeDefinition(AttributeDefinitionContext ctx) {

        logExit();
    }

    @Override
    public void enterIntegerDataType(IntegerDataTypeContext ctx) {

        logEnter();
    }

    @Override
    public void exitIntegerDataType(IntegerDataTypeContext ctx) {

        logExit();
    }

    @Override
    public void enterByteDataType(ByteDataTypeContext ctx) {

        logEnter();
    }

    @Override
    public void exitByteDataType(ByteDataTypeContext ctx) {

        logExit();
    }

    @Override
    public void enterShortDataType(ShortDataTypeContext ctx) {

        logEnter();
    }

    @Override
    public void exitShortDataType(ShortDataTypeContext ctx) {

        logExit();
    }

    @Override
    public void enterLongDataType(LongDataTypeContext ctx) {

        logEnter();
    }

    @Override
    public void exitLongDataType(LongDataTypeContext ctx) {

        logExit();
    }

    @Override
    public void enterFloatDataType(FloatDataTypeContext ctx) {

        logEnter();
    }

    @Override
    public void exitFloatDataType(FloatDataTypeContext ctx) {

        logExit();
    }

    @Override
    public void enterDoubleDataType(DoubleDataTypeContext ctx) {

        logEnter();
    }

    @Override
    public void exitDoubleDataType(DoubleDataTypeContext ctx) {

        logExit();
    }

    @Override
    public void enterStringDataType(StringDataTypeContext ctx) {

        logEnter();
    }

    @Override
    public void exitStringDataType(StringDataTypeContext ctx) {

        logExit();
    }

    @Override
    public void enterInsertEventStatement(InsertEventStatementContext ctx) {

        logEnter();
    }

    @Override
    public void exitInsertEventStatement(InsertEventStatementContext ctx) {

        logExit();
    }

    @Override
    public void enterAttributeList(AttributeListContext ctx) {

        logEnter();
    }

    @Override
    public void exitAttributeList(AttributeListContext ctx) {

        logExit();
    }

//    @Override
//    public void enterEveryRule(ParserRuleContext ctx) {
//        logEnter();
//    }
//
//    @Override
//    public void exitEveryRule(ParserRuleContext ctx) {
//
//        logExit();
//    }
}
