/**
 * 
 */
package de.umr.ep.util.io;

import java.io.BufferedInputStream;
import java.io.Closeable;
import java.io.DataInputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardOpenOption;
import java.util.Iterator;

import de.umr.ep.event.Event;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.impl.SimpleEvent;

/**
 * @author Michael Körber
 *
 */
public class BinaryEventReader implements Iterator<Event>, Closeable {
    
    private final EventSchema schema;
    
    private final DataInputStream in;
    
    private Event next;
    
    
    public BinaryEventReader( EventSchema schema, Path src ) throws IOException {
        this.schema = schema;
        this.in = new DataInputStream( new BufferedInputStream( Files.newInputStream(src, StandardOpenOption.READ) ));
        this.next = readNext();
    }
    
    private Event readNext() {
        try {
            Object[] payload = new Object[schema.getNumAttributes()]; 
            for ( int i = 0; i < schema.getNumAttributes(); i++ ) {
                switch ( schema.getAttribute(i).getType() ) {
                case BYTE:
                    payload[i] = in.readByte(); break;
                case SHORT:
                    payload[i] = in.readShort(); break;
                case INTEGER:
                    payload[i] = in.readInt(); break;
                case LONG:
                    payload[i] = in.readLong(); break;
                case FLOAT:
                    payload[i] = in.readFloat(); break;
                case DOUBLE:
                    payload[i] = in.readDouble(); break;
                case STRING:
                    payload[i] = in.readUTF(); break;
                default:
                    throw new IllegalArgumentException("Unknown data-type: " + schema.getAttribute(i).getType() );
                }
            }
            return new SimpleEvent(payload, in.readLong(), in.readLong());
        } catch ( IOException e ) {
            closeSilently();
            return null;
        }
    }
    
    private void closeSilently() {
        try {
            in.close();
        } catch (IOException e1) {
        }
    }
    
    
    public void close() throws IOException {
        closeSilently();
        next = null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean hasNext() {
        return next != null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Event next() {
        Event result = next;
        next = readNext();
        return result;
    }
    

}
