/**
 * 
 */
package de.umr.ep.util.io;

import java.io.BufferedOutputStream;
import java.io.Closeable;
import java.io.DataOutputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardOpenOption;

import de.umr.ep.event.Event;
import de.umr.ep.event.EventSchema;

/**
 * @author Michael Körber
 *
 */
public class BinaryEventWriter implements Closeable {
    
    private final EventSchema schema;
    
    private final DataOutputStream out;
    
    
    public BinaryEventWriter( EventSchema schema, Path dest ) throws IOException {
        this.schema = schema;
        this.out = new DataOutputStream( new BufferedOutputStream( Files.newOutputStream(dest, StandardOpenOption.CREATE_NEW, StandardOpenOption.WRITE) ));
    }
    
    public void writeEvent( Event e ) throws IOException {
        for ( int i = 0; i < schema.getNumAttributes(); i++ ) {
            switch ( schema.getAttribute(i).getType() ) {
            case BYTE:
                out.writeByte(e.get(i, Byte.class)); break;
            case SHORT:
                out.writeShort(e.get(i, Short.class)); break;
            case INTEGER:
                out.writeInt(e.get(i, Integer.class)); break;
            case LONG:
                out.writeLong(e.get(i, Long.class)); break;
            case FLOAT:
                out.writeFloat(e.get(i, Float.class)); break;
            case DOUBLE:
                out.writeDouble(e.get(i, Double.class)); break;
            case STRING:
                out.writeUTF(e.get(i, String.class)); break;
            default:
                throw new IllegalArgumentException("Unknown data-type: " + schema.getAttribute(i).getType() );
            }
        }
        out.writeLong(e.getT1());
        out.writeLong(e.getT2());
    }
    
    
    
    public void close() throws IOException {
        out.close();
    }
    

}
