/**
 * 
 */
package de.umr.ep.util.io;

import java.io.BufferedReader;
import java.io.Closeable;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardOpenOption;
import java.util.Iterator;

import de.umr.ep.event.Event;

/**
 * @author Michael Körber
 *
 */
public class CSVEventReader implements Iterator<Event>, Closeable {

	public static interface EventConverter {

		Event convert(String[] fields);
	}

	private final BufferedReader	reader;

	private final String			delim;

	private final EventConverter	converter;

	private Event					next	= null;

	public CSVEventReader(Path source, boolean skipFirst, EventConverter converter) throws IOException {
		this(source, skipFirst, converter, ',');
	}

	public CSVEventReader(Path source, boolean skipFirst, EventConverter converter, char delim) throws IOException {
		this.reader = new BufferedReader(new InputStreamReader(Files.newInputStream(source, StandardOpenOption.READ)));
		this.delim = "" + delim;
		this.converter = converter;
		if ( skipFirst )
			reader.readLine();
		this.next = readNext();
	}

	private Event readNext() {
		String line = null;
		try {
			line = reader.readLine();
			if ( line == null ) {
				closeSilently();
				return null;
			}

		}
		catch (IOException e) {
			closeSilently();
			return null;
		}

		String[] fields = line.split(delim);
		for (int i = 0; i < fields.length; i++)
			fields[i] = fields[i].trim();

		return converter.convert(fields);
	}

	private void closeSilently() {
		try {
			reader.close();
		}
		catch (IOException e1) {
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void close() throws IOException {
		closeSilently();
		next = null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean hasNext() {
		return next != null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Event next() {
		Event result = next;
		next = readNext();
		return result;
	}

}
