package de.umr.jepc.v2.addons.epa.topk;

import java.util.Arrays;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

public class Cell {

	private final Map<Integer, TMAQuery> influenceList;
	private final List<TopKEvent> eventList;
	private final int[] xyz;
	private final double[] xyzMax;
	private final int flatIdx;
	private final int[] neighbours;

	public Cell(int[] xyz, int flatIdx, double cellExtent) {
		influenceList = new Hashtable<>();
		eventList = new LinkedList<>();
		this.xyz = xyz.clone();
		this.xyzMax = new double[xyz.length];
		// Calculate max Coordinates of the cell
		for (int i = 0; i < xyzMax.length; i++) {
			xyzMax[i] = (this.xyz[i] + 1) * cellExtent;
		}
		this.flatIdx = flatIdx;
		// Set neighbours
		neighbours = new int[xyz.length];
		for (int i = 0; i < neighbours.length; i++) {
			if (i == 0 && flatIdx % 4 == 0) {
				neighbours[0] = -1;
				continue;
			}
			int cellToAdd = flatIdx - (int) Math.pow((int) Math.ceil(1.0 / cellExtent), i);
			neighbours[i] = cellToAdd;
		}
	}

	public void addEventToCell(TopKEvent event) {
		eventList.add(event);
	}

	public void removeEventFromCell(TopKEvent event) {
		eventList.remove(event);
	}

	public boolean addToInfluenceList(TMAQuery tk) {
		return influenceList.put(tk.getId(), tk) == null ? true : false;
	}

	public boolean removeFromInfluenceList(TMAQuery tk) {
		return influenceList.remove(tk) != null ? true : false;
	}

	public Map<Integer, TMAQuery> getInfluenceList() {
		return influenceList;
	}

	public List<TopKEvent> getEventList() {
		return eventList;
	}

	public int[] getXyz() {
		return xyz;
	}

	public double[] getXyzMax() {
		return xyzMax;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Cell other = (Cell) obj;
		if (!Arrays.equals(xyz, other.xyz))
			return false;
		return true;
	}

	public int getFlatIdx() {
		return flatIdx;
	}

	public int[] getNeighbours() {
		return neighbours;
	}
}
