package de.umr.jepc.v2.addons.epa.topk;

import java.util.Comparator;
import java.util.Iterator;
import java.util.TreeSet;

import de.umr.ep.common.IncompatibleTypeException;
import de.umr.ep.event.EventSchema;
import de.umr.jepc.v2.ql.TranslatorException;

public class SMAQuery extends TMAQuery {

	private Comparator<TopKEvent> skComp;

	public SMAQuery(GridMonitor gridMonitor, int k, TopKScoringFunction scoringFunction, EventSchema schema)
			throws TranslatorException, IncompatibleTypeException {
		super(gridMonitor, k, scoringFunction, schema);

		// Comparator for binary tree
		skComp = new Comparator<TopKEvent>() {
			@Override
			public int compare(TopKEvent arg0, TopKEvent arg1) {
				if (arg0.getEvent().getT2() > arg1.getEvent().getT2())
					return 1;
				return -1;
			}
		};
	}

	@Override
	public void processCycle() {
		processSMA();
		addEvents.clear();
		removeEvents.clear();
	}

	private void processSMA() {
		if (topList.size() < k) {
			computeSkyband();
			return;
		}
		boolean skybandChanged = false;
		for (TopKEvent toInsert : addEvents) {
			skybandChanged = addEventToSkyband(toInsert);
		}
		for (TopKEvent toRemoveEvent : removeEvents) {
			skybandChanged = removeEventFromSkyband(toRemoveEvent);
		}
		if (skybandChanged) {
			// Set topScore to the k-th Element
			if (topList.size() >= k) {
				int x = Math.min(k, topList.size() - k);
				if (x == k) {
					for (TopKEvent s : topList) {
						if (x == 0) {
							topScore = scoringFunction.eval(s.getEvent());
							break;
						}
						x--;
					}
				} else {
					for (TopKEvent s : topList.descendingSet()) {
						if (x == 0) {
							topScore = scoringFunction.eval(s.getEvent());
							break;
						}
					}
					x--;
				}
			} else {
				recomputations++;
				computeSkyband();
			}
		}
	}

	private boolean removeEventFromSkyband(TopKEvent toInsert) {
		boolean skybandChanged = false;
		for (TopKEvent toRemoveEvent : removeEvents) {
			if (scoringFunction.eval(toRemoveEvent.getEvent()) >= topScore) {
				boolean eventToRemoveFound = topList.remove(toRemoveEvent);
				if (eventToRemoveFound) {
					skybandChanged = true;
				}
			}
		}
		return skybandChanged;
	}

	private boolean addEventToSkyband(TopKEvent toInsert) {
		boolean skybandChanged = false;
		double score = scoringFunction.eval(toInsert.getEvent());
		if (score >= topScore) {
			Iterator<TopKEvent> iter = topList.descendingIterator();
			boolean dontInsert = false;
			while (iter.hasNext()) {
				TopKEvent sk = iter.next();
				if (score >= scoringFunction.eval(sk.getEvent())) {
					if (toInsert.getEvent().getT2() > sk.getEvent().getT2()) {
						if (sk.increaseDominanceCounter() >= k) {
							iter.remove();
							skybandChanged = true;
						}
					}
				}
				if (score <= scoringFunction.eval(sk.getEvent())) {
					if (toInsert.getEvent().getT2() < sk.getEvent().getT2()) {
						if (toInsert.increaseDominanceCounter() >= k) {
							dontInsert = true;
						}
					}
				}
			}
			if (!dontInsert) {
				topList.add(toInsert);
				skybandChanged = true;
			}
		}
		return skybandChanged;
	}

	private TreeSet<TopKEvent> computeSkyband() {
		computeTopKEvents();
		if (gridMonitor.valids.size() < k) {
			return topList;
		}
		SkybandTree<TopKEvent> bt = new SkybandTree<>(skComp);
		for (TopKEvent e : topList) {
			e.setDc(bt.add(e));
		}
		return topList;
	}
}
