package de.umr.jepc.v2.addons.epa.tpstream;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Holds a set of defined {@link Constraint temporal constraints}.
 * @author Michael Körber
 *
 */
public final class Constraints implements Iterable<Constraint> {
    
    /** The constraints */
    private final List<Constraint> constraints = new ArrayList<>();
    
    /** A mapping between a situation id and constraints it is involved in */
    private final Map<Character,List<Constraint>> constraintMap = new HashMap<>();

    /**
     * @param constraints the defined constraints
     */
    public Constraints(Constraint... constraints) {
        this.constraints.addAll(Arrays.asList(constraints));
        
        for ( Constraint c : constraints ) {
            constraintMap.put(c.idA, new ArrayList<>() );
            constraintMap.put(c.idB, new ArrayList<>() );
        }
        
        for ( Constraint c : constraints) {
            constraintMap.get(c.idA).add(c);
            constraintMap.get(c.idB).add(c);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Iterator<Constraint> iterator() {
        return constraints.iterator();
    }
    
    /**
     * @return the number of constraints
     */
    public int getNumberOfConstraints() {
        return constraints.size();
    }
    
    /**
     * @param id the situation stream id
     * @return all constraints situations of the given stream are involved in
     */
    public List<Constraint> getSituationConstraints( Character id ) {
        return constraintMap.get(id);
    }
    

    /**
     * {@inheritDoc}
     */
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((constraints == null) ? 0 : constraints.hashCode());
        return result;
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public String toString() {
        return constraints.toString();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        Constraints other = (Constraints) obj;
        if (constraints == null) {
            if (other.constraints != null)
                return false;
        } else if (!constraints.equals(other.constraints))
            return false;
        return true;
    }
}
