package de.umr.jepc.v2.addons.epa.tpstream;

import de.umr.jepc.v2.api.epa.pattern.symbol.Bindings;
import de.umr.jepc.v2.api.expression.BooleanExpression;

/**
 * Defines a situation with an unique identifier, the condition to be fulfilled by a sequence of
 * events and a list of aggregates to compute.
 * @author Michael Körber
 *
 */
public class SituationDefinition {
	
    /** The identifier */
	private final char identifier;
	
	/** The condition */
	private final BooleanExpression condition;
	
	/** The aggregates to compute */
	private final Bindings aggregates;

	/**
	 * @param identifier The identifier
	 * @param condition The condition
	 */
	public SituationDefinition(Character identifier, BooleanExpression condition) {
	    this(identifier,condition,new Bindings());
	}
	
	/**
	 * @param identifier The identifier
     * @param condition The condition
	 * @param aggregates The aggregates to compute
	 */
	public SituationDefinition(Character identifier, BooleanExpression condition, Bindings aggregates) {
		super();
		this.identifier = identifier;
		this.condition = condition;
	    this.aggregates = aggregates;
	}

	/**
	 * @return The identifier
	 */
	public char getId() {
		return identifier;
	}

	/**
	 * @return The condition
	 */
	public BooleanExpression getCondition() {
		return condition;
	}

	/**
	 * @return The aggregates to compute
	 */
	public Bindings getAggregates() {
		return aggregates;
	}
	

	/**
	 * {@inheritDoc}
	 */
	@Override
    public String toString() {
        return "[" + identifier + ":=" + condition + ", aggregates=" + aggregates + "]";
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((aggregates == null) ? 0 : aggregates.hashCode());
        result = prime * result + ((condition == null) ? 0 : condition.hashCode());
        result = prime * result + identifier;
        return result;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        SituationDefinition other = (SituationDefinition) obj;
        if (aggregates == null) {
            if (other.aggregates != null)
                return false;
        } else if (!aggregates.equals(other.aggregates))
            return false;
        if (condition == null) {
            if (other.condition != null)
                return false;
        } else if (!condition.equals(other.condition))
            return false;
        if (identifier != other.identifier)
            return false;
        return true;
    }
}
