/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.aggregates;

import de.umr.ep.event.Event;
import de.umr.ep.event.impl.SimpleEvent;
import de.umr.jepc.v2.addons.epa.tpstream.Situation;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinitions;
import de.umr.jepc.v2.api.epa.pattern.Output;
import de.umr.jepc.v2.bindings.NoSuchBindingException;

/**
 * Helper to assemble output events from a set of situations.
 * The values to retrieve as well as their positions inside the 
 * output-event are precomputed in order to efficiently build
 * those outputs.
 * @author Michael Körber
 *
 */
public final class OutputMapping {
    
    /** 
     * Remembers for each output-attribute the situation index
     * where the aggregate is stored 
     */
    private int[] outputToSituationIdx;
    
    /**
     * Remembers for each attribute the index of the 
     * corresponding aggregate
     */
    private int[] outputToBindingIdx;
    
    public OutputMapping( SituationDefinitions definitions, Output output ) {
        outputToSituationIdx = new int[output.getSize()];
        outputToBindingIdx = new int[output.getSize()];
        
        int idx = 0;
        for ( String var : output ) {
            for ( int sidx = 0; sidx < definitions.getNumberOfDefinitions(); sidx++) {
                int bIdx = definitions.getDefinition(sidx).getAggregates().indexOf(var);
                if ( bIdx >= 0 ) {
                    outputToSituationIdx[idx] = sidx;
                    outputToBindingIdx[idx] = bIdx;
                    break;
                }
            }
            idx++;
        }
    }
    
    /**
     * Constructs an output event from the given set of situations
     * @param workingset the temporal configuration to create an output event for
     * @param currentTime the current stream time
     * @return the event to output
     * @throws NoSuchBindingException
     */
    public final Event create( Situation[] workingset, long currentTime ) throws NoSuchBindingException {
        Object[] payload = new Object[outputToSituationIdx.length];
        for ( int i = 0; i < outputToSituationIdx.length; i++ ) {
            payload[i] = workingset[outputToSituationIdx[i]].getAggregates().get(outputToBindingIdx[i]);
        }
        return new SimpleEvent(payload, currentTime, currentTime+1);
    }

}
