package de.umr.jepc.v2.addons.epa.tpstream.buffer;

import de.umr.jepc.v2.api.util.Pair;

/**
 * Describes a continuous range of indexes inside
 * a {@link SituationBuffer}. The lower bound is inclusive,
 * the upper one exclusive.
 * @author Michael Körber
 *
 */
public class IndexRange extends Pair<Integer, Integer> {
	
    /** The unique value for an empty range */
	public static final IndexRange EMPTY = new IndexRange();
	

	/** 
	 * @param _1 the lower bound of the range (inclusive)
	 * @param _2 the upper bound of the range (exclusive)
	 */
	public IndexRange(Integer _1, Integer _2) {
		super(_1, _2);
		if ( _2 < 1 || _1 >= _2 )
			throw new IllegalArgumentException("Not a valid range: [" + _1 + "," + _2 + ")"); 
	}
	
	/**
	 * Empty range
	 */
	private IndexRange() {
		super(-1,-1);
	}
	
	/**
	 * @return the number of elements covered by this range
	 */
	public int getSize() {
		return _2 - _1;
	}
	
	/**
	 * Checks if this range intersects the given one.
	 * @param other the range to check for intersections
	 * @return <code>true</code> if this range intersects with the given one, <code>false</code> otherwise
	 */
	public boolean intersects( IndexRange other ) {
		return !(_2 <= other._1 || _1 >= other._2);
	}
	
	/**
	 * @param other the range to intersect this range with
	 * @return the range representing the intersection of this range with the given one
	 */
	public IndexRange intersect( IndexRange other ) {
		return new IndexRange( Math.max(_1, other._1), Math.min(_2, other._2));
	}
	
	/**
	 * @param other the range to unify this range with
	 * @return the range representing the union of this range with the given one
	 */
	public IndexRange union( IndexRange other ) {
		return new IndexRange( Math.min(_1, other._1), Math.max(_2, other._2));
	}
}
