/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.buffer;

import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinitions;

/**
 * Holds a {@link SituationBuffer} for each defined situation.
 * @author Michael Körber
 *
 */
public class SituationBuffers {
    
    /** The actual buffers */
    private final SituationBuffer[] buffers;
    
    /**
     * @param definitions the definitions
     */
    public SituationBuffers(SituationDefinitions definitions) {
        this.buffers  = new SituationBuffer[definitions.getNumberOfDefinitions()];
        for ( int i = 0; i < this.buffers.length; i++ ) {
        	buffers[i] = new SituationBuffer();
        }
    }
    
    /**
     * @return the number of available buffers
     */
    public int getNumberOfBuffers() {
        return buffers.length;
    }
    
    /**
     * Removes expired situations from all buffers. Those are
     * situations starting before the given point in time.
     * @param upTo the point in time up to which situations are purged.
     */
    public void cleanup( long upTo ) {
        for ( SituationBuffer b : buffers ) {
            b.cleanUp(upTo);
        }
    }
    
    /**
     * @param idx the index of the symbol to retrieve the buffer for
     * @return the interval-buffer for the given symbol
     */
    public SituationBuffer getBuffer( Integer idx ) {
        return buffers[idx];
    }
}
