/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.buffer;

import java.util.Comparator;

import de.umr.jepc.v2.api.util.RingBuffer;

/**
 * A simple ring-buffer assuming the elements are inserted in a specific order.
 * Thus this buffer provides methods for efficient searching of elements.
 * @author Michael Körber
 *
 */
public class SortedRingBuffer<T> extends RingBuffer<T> {

    /**
     * Searches for the given key using the given comparator
     * @param key the element to search for
     * @param comparator the comparator to use
     * @return the index of the searched element if it exists, -(ip+1) otherwise (where ip is the index where an element with the given key would be placed 
     */
    public int binarySearch( T key, Comparator<T> comparator ) {
        return binarySearch0(0, size(), key, comparator);
    }
    
    /**
     * Binary search implementation. Searches for the given key within the specified
     * range.
     * @param fromIndex the lower bound of the search range
     * @param toIndex the upper bound of the search range
     * @param key the key to search for
     * @param comparator the comparator to use 
     * @return the index of the searched element if it exists, -(ip+1) otherwise (where ip is the index where an element with the given key would be placed
     */
    @SuppressWarnings("unchecked")
    private int binarySearch0(int fromIndex, int toIndex, T key, Comparator<? super T> comparator) {
        int low = fromIndex;
        int high = toIndex - 1;

        while (low <= high) {
            int mid = (low + high) >>> 1;
            T midVal = (T) elements[(head + mid) & mask];
            int cmp = comparator.compare(midVal, key);
            if (cmp < 0)
                low = mid + 1;
            else if (cmp > 0)
                high = mid - 1;
            else
                return mid; // key found
        }
        return -(low + 1); // key not found.
    }
}
