/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.matcher;

import de.umr.jepc.v2.addons.epa.TPStream.Mode;
import de.umr.jepc.v2.addons.epa.tpstream.Situation;

/**
 * A matcher performs the actual pattern matching among streams
 * of {@link Situation situations}. It is responsible for managing
 * the corresponding buffers and to keep track of the time-window
 * constraint. 
 * @author Michael Körber
 *
 */
public interface Matcher {
    
	/**
	 * Callback to publish valid temporal configurations. That
	 * is a set of situations, one per stream, fulfilling all defined
	 * constraints
	 * 
	 * @author Michael Körber
	 */
    public static interface MatcherCallback {
        /**
         * Invoked whenever a temporal configuration fulfills all
         * defined temporal constraints
         * @param temporalConfiguration the temporal configuration
         * @param currentTime the current point in time (time of detection).
         */
        void onResult( Situation[] temporalConfiguration, long currentTime );
    }
    
    /**
     * Invoked whenever an event changes the state of one
     * or more situation streams. This is whenever a situation
     * is finished or it is started (in {@link Mode#LOW_LATENCY low-latency} mode.  
     * @param updates All updated situations. This array contains <code>null</null> if for this stream no change of interest occured.
     * @param currentTime the current time
     */
    void update(Situation[] updates, long currentTime);
    
}
