/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.matcher.eval;

import java.util.Iterator;

import de.umr.jepc.v2.addons.epa.tpstream.Situation;
import de.umr.jepc.v2.addons.epa.tpstream.optimizer.stats.SelectivityEstimate;

/**
 * A single step during evaluation. The idea is, that in each step, on situation from
 * a defined stream is added to the partial temporal configuration so that all applicable
 * temporal constraints are fulfilled. 
 * A step must provide two functions: First, it must be able to check the temporal constraints applicable in this
 * step on a given temporal configuration and second: It must retrieve all situations
 * not violating the applicable constraints of this step from the corresponding
 * situation buffer.
 * @author Michael Körber
 */
public interface EvaluationStep {
	
    /** The index of the situation stream considered in this step */
	int getSituationIndex();
	
	/** 
	 * Checks if the partial temporal configuration is valid according to this step's
	 * applicable temporal constraints.
	 * @param workingSet the temporal configuraion to check
	 * @return <code>true</code> if the given configuration is valid, <code>false</code> otherwise.
	 */
	boolean isSatisfied( Situation[] workingSet );
	
	/**
	 * Queries the assigned situation buffer for situations that when added to the given
	 * partial temporal configuration do not violate this step's temporal constraints.
	 * @param workingSet the partial temporal configuration without a conrete situation for this step
	 * @param estimate the selectivity estimate to update during search
	 * @return all situations applicable for the given temporal configuration
	 */
	Iterator<Situation> query( Situation[] workingSet, SelectivityEstimate estimate );
	
	
	Iterator<Situation> iterateBuffer();
}
