/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.matcher.eval;

import de.umr.jepc.v2.addons.epa.tpstream.Situation;
import de.umr.jepc.v2.addons.epa.tpstream.optimizer.stats.RunningStats;
import de.umr.jepc.v2.addons.epa.tpstream.optimizer.stats.TPStats;

/**
 * An evaluator consumes a working-set (a partial temporal configuration)
 * and completes it to a valid temporal configuration by search the corresponding
 * situation buffers.
 * @author Michael Körber
 *
 */
public interface Evaluator {
	
	/**
	 * @return The statistics this evaluator is based on
	 */
	TPStats getStatistics();
	
	/**
	 * Consumes the given partial temporal configuration and computes all matches resulting
	 * from this configuration considering the situation buffers.
	 * @param workingSet the partial temporal configuration to be completed.
	 * @param currentTime the current time (event-time)
	 * @param statistics the statistics to update with actual selectivities
	 */
	void evaluate( Situation[] workingSet, long currentTime, RunningStats statistics );
	
	/**
     * Consumes the given partial temporal configuration and computes all matches resulting
     * from this configuration considering the situation buffers.
     * @param workingSet the partial temporal configuration to be completed.
     * @param currentTime the current time (event-time)
     */
    void evaluate( Situation[] workingSet, long currentTime );
	
	

}
