/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.matcher.eval;

import java.util.ArrayList;
import java.util.List;

import de.umr.jepc.v2.addons.epa.tpstream.Constraint;
import de.umr.jepc.v2.addons.epa.tpstream.Constraints;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinitions;
import de.umr.jepc.v2.addons.epa.tpstream.buffer.SituationBuffers;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.Matcher.MatcherCallback;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.eval.impl.EVStepImpl;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.eval.impl.RecursiveEvaluator;
import de.umr.jepc.v2.addons.epa.tpstream.optimizer.stats.TPStats;

/**
 * Utility class for creating evaluators
 * @author Michael Körber
 *
 */
public class EvaluatorUtil {
	
	/**
	 * Creates a new evaluator instance
	 * @param statistics the statistics used to compute the evaluation order
	 * @param definitions the situation definitions
	 * @param constraints the constraints
	 * @param order the actual evaluation order
	 * @param buffers the situation buffers
	 * @param callback the result callback
	 * @return an evaluator instance based on the given parameters
	 */
	public static Evaluator create( TPStats statistics, SituationDefinitions definitions, Constraints constraints, List<Character> order, SituationBuffers buffers, MatcherCallback callback) {
		List<EvaluationStep> steps = new ArrayList<>();
		
		for ( int i = 0; i < order.size(); i++ ) {
			Character current = order.get(i);
			List<Constraint> applicable = new ArrayList<>();
			// Get applicable constraints
			for (Constraint c : constraints.getSituationConstraints(current)) {
				if (   c.idA == current && order.indexOf(c.idB) < i
					|| c.idB == current && order.indexOf(c.idA) < i ) {
					applicable.add(c);
				}
			}
			steps.add( new EVStepImpl(current, buffers.getBuffer(definitions.getDefinitionIndex(current)), definitions, applicable));
		}
		return new RecursiveEvaluator(statistics, constraints, steps, callback);
	}
	
	
	public static Evaluator createISEQ( TPStats statistics, SituationDefinitions definitions, Constraints constraints, List<Character> order, SituationBuffers buffers, MatcherCallback callback) {
        List<EvaluationStep> steps = new ArrayList<>();
        
        for ( int i = 0; i < order.size(); i++ ) {
            Character current = order.get(i);
            List<Constraint> applicable = new ArrayList<>();
            // Get applicable constraints
            for (Constraint c : constraints.getSituationConstraints(current)) {
                if (   c.idA == current && order.indexOf(c.idB) < i
                    || c.idB == current && order.indexOf(c.idA) < i ) {
                    applicable.add(c);
                }
            }
            steps.add( new EVStepImpl(current, buffers.getBuffer(definitions.getDefinitionIndex(current)), definitions, applicable));
        }
        return new RecursiveEvaluator(statistics, constraints, steps, callback);
    }

} 