package de.umr.jepc.v2.addons.epa.tpstream.optimizer.stats;

/**
 * Statistics wrapper to account for trigger situations.
 * The idea is, that because a trigger situation is fixed in
 * possible results, the scan of the corresponding situation buffer
 * is omitted. Thus the cardinality for this type of situation is 1.
 * 
 * @author Michael Körber
 */
public class TriggerWrapper extends TPStats {

    /** The id the trigger situation */
	private final Character triggerIdx;
	
	/**
	 * @param trigger the id of the trigger situation
	 * @param stats the actual statistics to wrap
	 */
	public TriggerWrapper( Character trigger, TPStats stats ) {
		super(stats);
		this.triggerIdx = trigger;
	}
	
	/**
	 * Retrieves the cardinality for the situation buffer belonging to the given id.
	 * If the requested cardinality is for the wrapped trigger situation 1 is returned.
	 * @return the cardinality
	 */
	@Override
	public double getCardinality( Character situationStreamId ) {
		if ( situationStreamId == triggerIdx )
			return 1;
		return super.getCardinality(situationStreamId);
	}
}
