/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.partition;

import de.umr.ep.common.IncompatibleTypeException;
import de.umr.ep.event.EventSchema;
import de.umr.jepc.v2.addons.epa.tpstream.Constraints;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinitions;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.AdaptiveFixedDFSMatcher;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.Matcher;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.Matcher.MatcherCallback;
import de.umr.jepc.v2.addons.epa.tpstream.optimizer.Optimizer;
import de.umr.jepc.v2.addons.epa.tpstream.partition.deriver.AbstractDeriver;
import de.umr.jepc.v2.addons.epa.tpstream.partition.deriver.ClassicDeriver;
import de.umr.jepc.v2.bindings.EVBindings;
import de.umr.jepc.v2.expressions.bool.EVBooleanExpression;
import de.umr.jepc.v2.ql.TranslatorException;

/**
 * Creates partitions for classic temporal pattern matching with closed interval semantics. 
 * @author Michael Körber
 *
 */
public final class ClassicAdaptivePartitionFactory extends AbstractPartitionFactory {

    private final double reoptimizeThreshold;
    
    /**
     * @param schema the schema of incoming events
     * @param definitions the situation stream definitions
     * @param constraints the defined constraints
     * @param window the defined time-window
     * @param callback the callback to invoke on new results
     * @throws TranslatorException if conditions could not be translated
     * @throws IncompatibleTypeException if conditions could not be translated
     */
    public ClassicAdaptivePartitionFactory(EventSchema schema, SituationDefinitions definitions, Constraints constraints, long window, MatcherCallback callback, double reoptimizeThreshold)
            throws TranslatorException, IncompatibleTypeException {
        super(schema, definitions, constraints, window, callback);
        this.reoptimizeThreshold = reoptimizeThreshold;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected AbstractDeriver createDeriver(EVBooleanExpression condition, EVBindings bindings) {
        return new ClassicDeriver(condition, bindings);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected Matcher createMatcher(SituationDefinitions definitions, Constraints constraints, long window, Optimizer optimizer, MatcherCallback callback) {
        return new AdaptiveFixedDFSMatcher(definitions, constraints, window, optimizer, callback,reoptimizeThreshold);
    }

}
