/*
 * Copyright (c) 2012, 2013
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.jepc.v2.addons.epa.tpstream.partition;

import de.umr.ep.event.Event;

/**
 * Used to partition events by a subset of their attributes
 * @author Michael Körber
 */
public final class EventPartitioner {

    /**
     * Key used if no attributes were given
     */
    private static final Object EMPTY_KEY = new Object();

    /** The separator used in building the key */
    private static final String SEPARATOR = ":";

    /** The attribute indexes used for the key */
    private final int[]         idxs;

    /**
     * Constructs a new EventPartitioner instance
     * @param keyIdxs the indexes of the attributes to use for the key
     */
    public EventPartitioner(int... keyIdxs) {
        this.idxs = keyIdxs;
    }

    /**
     * Builds the key for the given event by using the defined attributes
     * 
     * @param event the event to build the key for
     * @return the key for the given event
     */
    public Object buildKey(Event event) {
        if (idxs.length == 1)
            return event.get(idxs[0]);
        else if (idxs.length == 0)
            return EMPTY_KEY;
        else {
            StringBuilder res = new StringBuilder();
            for (int i : idxs) {
                res.append(event.get(i).toString()).append(SEPARATOR);
            }
            return res.toString();
        }
    }

}
