/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.partition;

import de.umr.ep.common.IncompatibleTypeException;
import de.umr.ep.event.EventSchema;
import de.umr.jepc.v2.addons.epa.tpstream.Constraints;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinitions;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.FloatingDFSMatcher;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.Matcher;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.Matcher.MatcherCallback;
import de.umr.jepc.v2.addons.epa.tpstream.optimizer.Optimizer;
import de.umr.jepc.v2.addons.epa.tpstream.partition.deriver.AbstractDeriver;
import de.umr.jepc.v2.addons.epa.tpstream.partition.deriver.LowLatencyDeriver;
import de.umr.jepc.v2.bindings.EVBindings;
import de.umr.jepc.v2.expressions.bool.EVBooleanExpression;
import de.umr.jepc.v2.ql.TranslatorException;

/**
 * Creates partitions for low-latency temporal pattern matching.
 * @author Michael Körber
 *
 */
public class LowLatencyPartitionFactory extends AbstractPartitionFactory {

    /**
     * @param schema the schema of incoming events
     * @param definitions the situation stream definitions
     * @param constraints the defined constraints
     * @param window the defined time-window
     * @param callback the callback to invoke on new results
     * @throws TranslatorException if conditions could not be translated
     * @throws IncompatibleTypeException if conditions could not be translated
     */
    public LowLatencyPartitionFactory(EventSchema schema, SituationDefinitions definitions, Constraints constraints, long window, MatcherCallback callback)
            throws TranslatorException, IncompatibleTypeException {
        super(schema, definitions, constraints, window, callback);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected AbstractDeriver createDeriver(EVBooleanExpression condition, EVBindings bindings) {
        return new LowLatencyDeriver(condition, bindings);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected Matcher createMatcher(SituationDefinitions definitions, Constraints constraints, long window, Optimizer optimizer, MatcherCallback callback) {
        return new FloatingDFSMatcher(definitions, constraints, window, optimizer, callback);
    }

}
