/*
 * Copyright (c) 2012, 2013
 * Database Research Group, University of Marburg.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.jepc.v2.addons.epa.tpstream.partition;

import de.umr.ep.event.Event;
import de.umr.jepc.v2.addons.epa.tpstream.Situation;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.Matcher;
import de.umr.jepc.v2.addons.epa.tpstream.partition.deriver.AbstractDeriver;

/**
 * Implementation of a single Partition.
 * @author Michael Körber
 */
public final class TPSPartition {

    /** The derivers for each situation-stream */
    private final AbstractDeriver[] derivers;

    /** The latest set of situations produced by the derivers */
    private final Situation[]       latest;

    /** The matching component */
    private final Matcher           matcher;

    /**
     * @param derivers the derivers to use
     * @param matcher the matcher to use
     */
    public TPSPartition(AbstractDeriver[] derivers, Matcher matcher) {
        this.derivers = derivers;
        this.matcher = matcher;
        this.latest = new Situation[derivers.length];
    }

    /**
     * Passes the given event to all configured derivers and
     * updates the matcher-state according to their results.
     * @param event the event to update this partition with
     */
    public final void update(Event event) {
        boolean update = false;
        for (int i = 0; i < derivers.length; i++) {
            this.latest[i] = derivers[i].update(event);
            update |= this.latest[i] != null;
        }
        if (update)
            matcher.update(latest, event.getT1());
    }
}
