package de.umr.jepc.v2.addons.epa.tpstream.partition.deriver;

import de.umr.ep.event.Event;
import de.umr.jepc.v2.addons.epa.TPStream.Mode;
import de.umr.jepc.v2.addons.epa.tpstream.Situation;
import de.umr.jepc.v2.bindings.EVBindings;
import de.umr.jepc.v2.expressions.bool.EVBooleanExpression;

/**
 * Deriver for {@link Mode#LOW_LATENCY low-latency} processing.
 * Situations are published, if the current event either
 * starts <b>OR</b> finishes a {@link Situation}.
 * 
 * @author Michael Körber
 */
public final class LowLatencyDeriver extends AbstractDeriver {
    
    /** The situation currently constructed */
    private Situation current;
    
    /**
     * @param condition the condition to evaluate on incoming events
     * @param aggregates the aggregates to compute for each situation
     */
    public LowLatencyDeriver(EVBooleanExpression condition, EVBindings aggregates) {
        super(condition, aggregates);
    }
    
    /**
     * @{inheritDoc}
     */
    public Situation update(Event event) {
        Situation result = null;

        // Expression is OK
        if (condition.eval(event)) {
            // Start a new situation
            if (current == null) {
                current = new Situation(event.getT1(), event.getT2(), aggregates.getNumberOfBindings());
                updateAggregates(event, current, true);
                result = current;
            }
            // Update the current situation
            else {
                current.update(event.getT2());
                updateAggregates(event, current, false);
            }
        }
        // If we had a current situation -> finish it
        else if (current != null) {
            current.finish(event.getT1());
            result = current;
            current = null;
        }
        return result;
    }
}
