package de.umr.jepc.v2.addons.epa.tpstream.processor;

import java.util.HashMap;
import java.util.Map;

import de.umr.ep.event.Event;
import de.umr.jepc.v2.addons.epa.tpstream.partition.EventPartitioner;
import de.umr.jepc.v2.addons.epa.tpstream.partition.PartitionFactory;
import de.umr.jepc.v2.addons.epa.tpstream.partition.TPSPartition;

/**
 * Processor used, to manage multiple partitions. Each event is examined
 * and passed to the partition according to its attribute values. 
 * @author Michael Körber
 */
public final class MultiPartitionProcessor implements Processor {

    /** The map of partition-keys to partitions */
	private final Map<Object, TPSPartition> partitions = new HashMap<>();

	/** The partitioner computes the partition-key for each event */
    private final EventPartitioner                ep;

    /** The factory used to create new partitions */
    private final PartitionFactory                partitionFactory;

    
	/**
	 * @param partitioner the partitioner to use
	 * @param partitionFactory the factory to create new partitions with
	 */
	public MultiPartitionProcessor(EventPartitioner partitioner, PartitionFactory partitionFactory) {
		this.ep = partitioner;
		this.partitionFactory = partitionFactory;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public final void process(Event event) {
		getPartition(ep.buildKey(event)).update(event);
	}
	
	/**
	 * Retrieves the partition for the given identifier or creates a new partition if required.
     * @param identifier the id to get the partition for
     * @return the partition for the given id
     */
    private TPSPartition getPartition(Object identifier) {
        TPSPartition result = partitions.get(identifier);
        if (result == null) {
            result = partitionFactory.create();
            partitions.put(identifier, result);
        }
        return result;
    }
}
