package de.umr.jepc.v2.addons.epa.tpstream.processor;

import de.umr.ep.event.Event;
import de.umr.jepc.v2.addons.epa.tpstream.partition.PartitionFactory;
import de.umr.jepc.v2.addons.epa.tpstream.partition.TPSPartition;

/**
 * Processor used, if no partition is specified. This implementation
 * is optimizied, since it does not need to lookup the target partition
 * for each incoming event.
 * @author Michael Körber
 */
public final class SinglePartitionProcessor implements Processor {
	
	/** The single partition to maintain */
	private final TPSPartition partition;
	
	/**
	 * @param factory the factory to create a new partition
	 */
	public SinglePartitionProcessor(PartitionFactory factory) {
		this.partition = factory.create();
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public final void process(Event event) {
		partition.update(event);
	}

}
