/*
 * Copyright (c) 2012, 2013, 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.jepc.v2.addons.epa;

import org.junit.Assert;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.umr.ep.common.Attribute;
import de.umr.ep.common.Attribute.DataType;
import de.umr.ep.common.EPException;
import de.umr.ep.event.EventSchema;
import de.umr.jepc.v2.addons.epa.TPStream.Mode;
import de.umr.jepc.v2.addons.epa.tpstream.Constraint;
import de.umr.jepc.v2.addons.epa.tpstream.Constraint.TemporalRelation;
import de.umr.jepc.v2.addons.epa.tpstream.Constraints;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinition;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinitions;
import de.umr.jepc.v2.api.EPProvider;
import de.umr.jepc.v2.api.epa.Stream;
import de.umr.jepc.v2.api.epa.pattern.Output;
import de.umr.jepc.v2.api.epa.pattern.symbol.Binding;
import de.umr.jepc.v2.api.epa.pattern.symbol.Bindings;
import de.umr.jepc.v2.api.epa.window.TimeWindow;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.DoubleConstant;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.Variable;
import de.umr.jepc.v2.api.expression.logical.And;
import de.umr.jepc.v2.api.expression.predicate.GreaterEq;
import de.umr.jepc.v2.api.expression.predicate.Less;
import de.umr.jepc.v2.api.provider.AlreadyRegisteredException;
import de.umr.jepc.v2.api.provider.NoSuchStreamException;
import de.umr.jepc.v2.api.provider.QueryHandle;
import de.umr.jepc.v2.api.provider.StreamHandle;
import de.umr.jepc.v2.api.util.CollectingSink;
import de.umr.jepc.v2.bridges.nat.NativeBridge;
import de.umr.jepc.v2.provider.def.DefaultEPProvider;

/**
 * @author Michael Körber
 *
 */
public class TPStreamTest {
  
  private static final Logger log = LoggerFactory.getLogger(TPStreamTest.class);
  
  private static final String STREAM = "stream";
  
  @Test
  public void simpleTest() throws AlreadyRegisteredException, NoSuchStreamException, EPException {
    
    
    Variable field = new Variable("value");
    TimeWindow src = new TimeWindow( new Stream(STREAM), 1);
    TPStream def = new TPStream(src,Mode.CLASSIC,15,
        new SituationDefinitions(
            new SituationDefinition('a', new And( new GreaterEq(field, new DoubleConstant(0.0)), new Less(field, new DoubleConstant(5.0))), new Bindings(new Binding("n_id",new Variable("id")))),
            new SituationDefinition('b', new And( new GreaterEq(field, new DoubleConstant(5.0)), new Less(field, new DoubleConstant(10.0))))
        ),
        new Constraints(new Constraint('a','b', TemporalRelation.AA_BB)),
        new Output("n_id")
    );
    
    final CollectingSink sink = new CollectingSink();
    
    
    EPProvider p = new DefaultEPProvider( new NativeBridge() );
    StreamHandle sh = p.registerStream(STREAM, new EventSchema(
            new Attribute("id", DataType.LONG),
            new Attribute("value", DataType.DOUBLE)
    ));
    QueryHandle qh = p.createQuery("query", def);
    p.registerSink(qh, sink);
    
    for ( long ts = 0; ts < 11; ts++ ) {
      sh.pushEvent(new Object[] { 1L, (double) ts }, ts);
    }
    
    log.debug("Results: ");
    sink.getEvents().forEach( event -> log.debug(event.toString()) );
    
    Assert.assertEquals(1, sink.getEvents().size() );
    
  }
  
  @Test
  public void spatialTest() {
    
//    GeometryFactory gf = new GeometryFactory();
//    
//    Attribute[] schema = { 
//        new Attribute("id", DataType.LONG),
//        new Attribute("geom", DataType.GEOMETRY),
//        new Attribute(Constants.T_START, Attribute.DataType.LONG), 
//        new Attribute(Constants.T_END, Attribute.DataType.LONG )
//    };
//    
//    Attribute[] schemaOut = { 
//        new Attribute("id", DataType.LONG),
//        new Attribute(Constants.T_START, Attribute.DataType.LONG), 
//        new Attribute(Constants.T_END, Attribute.DataType.LONG )
//    };
//    
//    String field = "geom";
//    Geometry p1 = gf.createPoint(new Coordinate(0, 0));
//    Geometry p2 = gf.createPoint(new Coordinate(5, 5));
//    
//    Stream src = new Stream("dummy");
//    
//    TPStream def = new TPStream("SpatialTPStream",
//        src,
//        new String[] {"id"},
//        new Symbol[] {
//          new Symbol('a', new IsWithinDistance(2,field, p1), new Variable("id","id")),
//          new Symbol('b', new IsWithinDistance(2,field, p2)) },
//        new Constraint[] { new Constraint('a','b', IntervalRelation.A_AB_B) }, 
//        new String[] {"id"},
//        15, Mode.FIXED
//    );
//    
//    CollectingOP op = new CollectingOP();
//    
//    DefaultTPStream tps = new DefaultTPStream(def, schema, op); 
//    
//    Assert.assertEquals( tps.getOutputSchema(), schemaOut );
//    
//    long timestamp = 1L;
//    tps.process(src,  new Object[] { 1L, gf.createPoint( new Coordinate( 1, 1 )),     timestamp++, timestamp } );
//    tps.process(src, new Object[] { 1L, gf.createPoint( new Coordinate( 1.5, 1.5 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 1L, gf.createPoint( new Coordinate( 2, 2 )),     timestamp++, timestamp } );
//    tps.process(src, new Object[] { 1L, gf.createPoint( new Coordinate( 2.5, 2.5 )), timestamp++, timestamp } );
//    
//    tps.process(src, new Object[] { 2L, gf.createPoint( new Coordinate( 1, 1 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 2L, gf.createPoint( new Coordinate( 1.5, 1.5 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 2L, gf.createPoint( new Coordinate( 2, 2 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 2L, gf.createPoint( new Coordinate( 2.5, 2.5 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 2L, gf.createPoint( new Coordinate( 3, 3 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 2L, gf.createPoint( new Coordinate( 3.5, 3.5 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 2L, gf.createPoint( new Coordinate( 4, 4 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 2L, gf.createPoint( new Coordinate( 12, 12 )), timestamp++, timestamp } );
//    
//    tps.process(src, new Object[] { 1L, gf.createPoint( new Coordinate( 3, 3 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 1L, gf.createPoint( new Cnew oordinate( 3.5, 3.5 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 1L, gf.createPoint( new Coordinate( 4, 4 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 1L, gf.createPoint( new Coordinate( 12, 12 )), timestamp++, timestamp } );
//    
//    tps.process(src, new Object[] { 1L, gf.createPoint( new Coordinate( 1, 1 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 1L, gf.createPoint( new Coordinate( 1.5, 1.5 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 1L, gf.createPoint( new Coordinate( 2, 2 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 1L, gf.createPoint( new Coordinate( 2.5, 2.5 )), timestamp++, timestamp } );
//    
//    tps.process(src, new Object[] { 2L, gf.createPoint( new Coordinate( 1, 1 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 2L, gf.createPoint( new Coordinate( 1.5, 1.5 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 2L, gf.createPoint( new Coordinate( 2, 2 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 2L, gf.createPoint( new Coordinate( 2.5, 2.5 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 2L, gf.createPoint( new Coordinate( 3, 3 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 2L, gf.createPoint( new Coordinate( 3.5, 3.5 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 2L, gf.createPoint( new Coordinate( 4, 4 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 2L, gf.createPoint( new Coordinate( 12, 12 )), timestamp++, timestamp } );
//    
//    tps.process(src, new Object[] { 1L, gf.createPoint( new Coordinate( 3, 3 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 1L, gf.createPoint( new Coordinate( 3.5, 3.5 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 1L, gf.createPoint( new Coordinate( 4, 4 )), timestamp++, timestamp } );
//    tps.process(src, new Object[] { 1L, gf.createPoint( new Coordinate( 12, 12 )), timestamp++, timestamp } );
//    
//    log.debug("Results: ");
//    op.getResults("SpatialTPStream").forEach( event -> log.debug(Arrays.toString(event)));
//    Assert.assertEquals(op.getResults("SpatialTPStream").size(), 4);
//    Assert.assertEquals(op.getResults("SpatialTPStream").get(0), new Object[] { 2L, 12L, 13L } );
//    Assert.assertEquals(op.getResults("SpatialTPStream").get(1), new Object[] { 1L, 16L, 17L } );
//    Assert.assertEquals(op.getResults("SpatialTPStream").get(2), new Object[] { 2L, 28L, 29L } );
//    Assert.assertEquals(op.getResults("SpatialTPStream").get(3), new Object[] { 1L, 32L, 33L } );
  }

}
