/*
 * Copyright (c) 2012, 2013, 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.jepc.v2.addons.epa;

import java.util.Arrays;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;

import de.umr.ep.common.Attribute;
import de.umr.ep.common.Attribute.DataType;
import de.umr.ep.common.EPException;
import de.umr.ep.event.Event;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.impl.SimpleEvent;
import de.umr.jepc.v2.addons.util.UserDefinedEPATest;
import de.umr.jepc.v2.api.epa.Stream;
import de.umr.jepc.v2.api.epa.window.TimeWindow;
import de.umr.jepc.v2.api.provider.AlreadyRegisteredException;
import de.umr.jepc.v2.api.provider.NoSuchStreamException;
import de.umr.jepc.v2.api.provider.QueryHandle;
import de.umr.jepc.v2.api.provider.StreamHandle;
import de.umr.jepc.v2.api.util.CollectingSink;

/**
 * @author Michael Körber
 *
 */
public class UnionAllTest extends UserDefinedEPATest {

//	private static final Logger log = LoggerFactory.getLogger(FilterTest.class);
    
    public static final String QUERY  = "union";
    
    @Test
    public void testUnionAll() throws AlreadyRegisteredException, NoSuchStreamException, EPException {
        EventSchema schema = new EventSchema(
            new Attribute("stream", DataType.STRING ),
            new Attribute("value", DataType.LONG)
        );
        
        UnionAll ua = new UnionAll(new TimeWindow(new Stream("A"),1),new TimeWindow(new Stream("B"),1),new TimeWindow(new Stream("C"),1));
        
        CollectingSink sink = new CollectingSink();
        
        StreamHandle sh1 = provider.registerStream("A", schema);
        StreamHandle sh2 = provider.registerStream("C", schema);
        StreamHandle sh3 = provider.registerStream("B", schema);
      
        QueryHandle qh = provider.createQuery(QUERY, ua);
        
        provider.registerSink(qh, sink);
        List<Event> results = sink.getEvents();

        List<Event> expected = Arrays.asList(
            new SimpleEvent(new Object[] { "A", 1L }, 1L, 2L),
            new SimpleEvent(new Object[] { "B", 1L }, 1L, 2L),
            new SimpleEvent(new Object[] { "C", 1L }, 1L, 2L),
            new SimpleEvent(new Object[] { "A", 2L }, 2L, 3L),
            new SimpleEvent(new Object[] { "B", 2L }, 2L, 3L),
            new SimpleEvent(new Object[] { "C", 2L }, 2L, 3L),
            new SimpleEvent(new Object[] { "A", 3L }, 3L, 4L),
            new SimpleEvent(new Object[] { "B", 3L }, 3L, 4L),
            new SimpleEvent(new Object[] { "C", 3L }, 3L, 4L)
        );
        
        for ( long i = 1; i <= 3; i++ ) {
            sh1.pushEvent(new Object[] { "A", i }, i);
            Assert.assertEquals((i-1)*3, results.size());
            sh2.pushEvent(new Object[] { "B", i }, i);
            Assert.assertEquals((i-1)*3, results.size());
            sh3.pushEvent(new Object[] { "C", i }, i);
            Assert.assertEquals(i*3, results.size());
        }
        
        compareResults(expected, results);
    }

}
