/*
 * Copyright (c) 2012, 2013
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.jepc.v2.addons.epa.tpstream;

import java.util.Arrays;
import java.util.List;

import org.junit.Test;

import de.umr.ep.common.Attribute;
import de.umr.ep.common.Attribute.DataType;
import de.umr.ep.common.EPException;
import de.umr.ep.event.Event;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.impl.SimpleEvent;
import de.umr.jepc.v2.addons.epa.TPStream;
import de.umr.jepc.v2.addons.epa.TPStream.Mode;
import de.umr.jepc.v2.addons.epa.tpstream.Constraint.TemporalRelation;
import de.umr.jepc.v2.addons.util.UserDefinedEPATest;
import de.umr.jepc.v2.api.epa.Stream;
import de.umr.jepc.v2.api.epa.pattern.Output;
import de.umr.jepc.v2.api.epa.pattern.Partition;
import de.umr.jepc.v2.api.epa.pattern.symbol.Binding;
import de.umr.jepc.v2.api.epa.pattern.symbol.Binding.BindingTime;
import de.umr.jepc.v2.api.epa.pattern.symbol.Bindings;
import de.umr.jepc.v2.api.epa.window.TimeWindow;
import de.umr.jepc.v2.api.expression.BooleanExpression;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.IntConstant;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.Variable;
import de.umr.jepc.v2.api.expression.predicate.Equal;
import de.umr.jepc.v2.api.provider.AlreadyRegisteredException;
import de.umr.jepc.v2.api.provider.NoSuchStreamException;
import de.umr.jepc.v2.api.provider.QueryHandle;
import de.umr.jepc.v2.api.provider.StreamHandle;
import de.umr.jepc.v2.api.util.CollectingSink;

/**
 * The ClosedIntervalSemantisTest
 * @author Michael Körber
 */
public class CommonSemanticsTest extends UserDefinedEPATest {

    private static final String STREAM = "test";

    private static final String QUERY  = "tpstream";

    @Test
    public void testSymbolOrderFixed() throws AlreadyRegisteredException, NoSuchStreamException, EPException {
        execute(Mode.CLASSIC);
    }

    @Test
    public void testSymbolOrderFloating() throws AlreadyRegisteredException, NoSuchStreamException, EPException {
        execute(Mode.LOW_LATENCY);
    }

    private void execute(Mode mode) throws AlreadyRegisteredException, EPException {
        EventSchema schema = new EventSchema(new Attribute("id", DataType.LONG), new Attribute("value", DataType.INTEGER));

        Variable field = new Variable("value");

        IntConstant ic = new IntConstant(0);

        BooleanExpression condition = new Equal(field, ic);

        SituationDefinitions symbols = new SituationDefinitions(
                new SituationDefinition('a', condition, new Bindings(new Binding("n_id", new Variable("id"), BindingTime.INITIAL))),
                new SituationDefinition('b', condition));

        Constraints constraints = new Constraints(new Constraint('a', 'b', TemporalRelation.A_BA_B));

        long window = 50;
        Output output = new Output("n_id");
        
        Partition partitionBy = new Partition( "id" );

        TPStream def = new TPStream(new TimeWindow(new Stream(STREAM), 1L), mode, window, symbols, constraints, output, partitionBy);

        CollectingSink sink = new CollectingSink();

        StreamHandle sh = provider.registerStream(STREAM, schema);
        QueryHandle qh = provider.createQuery(QUERY, def);
        provider.registerSink(qh, sink);

        List<Event> results = sink.getEvents();

        List<Event> expected = Arrays.asList(
                new SimpleEvent(new Object[] { 2L }, 6L, 7L),
                new SimpleEvent(new Object[] { 1L }, 8L, 9L)
                 
                
        );

        sh.pushEvent(new Object[] { 1L, 0 }, 1L);
        sh.pushEvent(new Object[] { 2L, 0 }, 1L);
        sh.pushEvent(new Object[] { 1L, 0 }, 2L);
        sh.pushEvent(new Object[] { 2L, 0 }, 4L);
        sh.pushEvent(new Object[] { 2L, 1 }, 6L);
        sh.pushEvent(new Object[] { 1L, 1 }, 8L);

        compareResults(expected, results);
    }
}
