package de.umr.jepc.v2.addons.epa.tpstream.optimizer;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.junit.Assert;
import org.junit.Test;

import de.umr.jepc.v2.addons.epa.tpstream.Constraint;
import de.umr.jepc.v2.addons.epa.tpstream.Constraints;
import de.umr.jepc.v2.addons.epa.tpstream.Constraint.TemporalRelation;
import de.umr.jepc.v2.addons.epa.tpstream.optimizer.constraints.ConstraintGraph;

public class ConstraintGraphTest {

	@Test
	public void testGetAllPaths() {
		Constraints cs = new Constraints(
			new Constraint('a', 'b', TemporalRelation.AABB ),
			new Constraint('b', 'c', TemporalRelation.AABB )
		);
		
		ConstraintGraph cg = new ConstraintGraph(cs);
		
		@SuppressWarnings("unchecked")
        List<List<Character>> expected = Arrays.asList( new List[] {
		        Arrays.asList( new Character[] {'a','b','c'} ),
		        Arrays.asList( new Character[] {'b','a','c'} ),
		        Arrays.asList( new Character[] {'b','c','a'} ),
		        Arrays.asList( new Character[] {'c','b','a'} )
		});
		
		List<List<Character>> actual = cg.getValidEvaluationOrders();
		
		Assert.assertTrue("Not all combinations yield", actual.containsAll(expected));
		Assert.assertTrue("Too many combinations yield", expected.containsAll(actual));
	}
	
	@Test
	public void testGetTriggerSymbolsSeq() {
	    Constraints cs = new Constraints(
            new Constraint('a', 'b', TemporalRelation.AABB ),
            new Constraint('b', 'c', TemporalRelation.AABB )
        );
	    
	    ConstraintGraph cg = new ConstraintGraph(cs);
	    
	    Set<Character> result = cg.getTriggerSymbols();
	    
	    Set<Character> exp = new HashSet<>( Arrays.asList(new Character[] {'c'}));
	    
	    Assert.assertEquals(exp, result);
	}
	
	@Test
	public void testGetTriggerSymbolsSimult() {
	    Constraints cs = new Constraints(
	            new Constraint('a', 'b', TemporalRelation.ABA_B)
        );
	    
	    ConstraintGraph cg = new ConstraintGraph(cs);
        
        Set<Character> result = cg.getTriggerSymbols();
        
        Set<Character> exp = new HashSet<>( Arrays.asList(new Character[] {'a','b'}));
        
        Assert.assertEquals(exp, result);
	}
	
	@Test
    public void testGetTriggerSymbolsAmb() {
        Constraints cs = new Constraints(
            new Constraint('a', 'b', TemporalRelation.AABB, TemporalRelation.BB_AA ),
            new Constraint('b', 'c', TemporalRelation.AABB )
        );
        
        ConstraintGraph cg = new ConstraintGraph(cs);
        
        Set<Character> result = cg.getTriggerSymbols();
        
        Set<Character> exp = new HashSet<>( Arrays.asList(new Character[] {'c','a'}));
        
        Assert.assertEquals(exp, result);
    }
	
	@Test
    public void testGetTriggerSymbolsEq() {
        Constraints cs = new Constraints(
                new Constraint('a', 'b', TemporalRelation.A_BA_B),
                new Constraint('b', 'c', TemporalRelation.A_BA_B),
                new Constraint('c', 'd', TemporalRelation.A_BA_B),
                new Constraint('d', 'e', TemporalRelation.A_BA_B),
                new Constraint('e', 'f', TemporalRelation.A_BA_B),
                new Constraint('f', 'g', TemporalRelation.A_BA_B),
                new Constraint('g', 'h', TemporalRelation.A_BA_B),
                new Constraint('h', 'i', TemporalRelation.A_BA_B),
                new Constraint('i', 'j', TemporalRelation.A_BA_B)
        );
        
        ConstraintGraph cg = new ConstraintGraph(cs);
        
        Set<Character> result = cg.getTriggerSymbols();
        
//        // FIXME: For experiment only
//        // Keep only one from simultaneous endings
//        Set<Character> exp = new HashSet<>( Arrays.asList(new Character[] {'j'}));
        
        Set<Character> exp = new HashSet<>( Arrays.asList(new Character[] {
        	'a','b','c','d','e','f','g','h','i','j'}));
        
        Assert.assertEquals(exp, result);
    }
	

}
