package de.umr.jepc.v2.api.epa;

import java.io.Serializable;

import de.umr.ep.common.IncompatibleTypeException;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.SchemaException;

/**
 * Abstract representation of event processing agents (EPA).
 * @author Michael Körber
 */
public interface EPA extends Serializable {
    
    /** A window has no input EPAs */
    public static final EPA[] EMPTY_EPA_ARRAY = {};

	/**
	 * @return all input EPAs of this EPA
	 */
	EPA[] getInputEPAs();
	
	/**
	 * Changes the idx-th input of this EPA 
	 * @param idx the input index to change
	 * @param input the new input EPA
	 */
	void setInput(int idx, EPA input);
	
	default int getDepth() {
		int childDepth = 0;
		for ( EPA c : getInputEPAs() )
			childDepth = Math.max(childDepth, c.getDepth());
		return 1 + childDepth;
	}
	
	
	/**
	 * @param inputSchemas the schemas of the input event streams
	 * @return the output schema of this epa
	 */
	EventSchema computeOutputSchema( EventSchema... inputSchemas ) throws SchemaException, IncompatibleTypeException;

}