/**
 * 
 */
package de.umr.jepc.v2.api.epa;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import de.umr.ep.common.Attribute;
import de.umr.ep.common.IncompatibleTypeException;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.SchemaException;
import de.umr.jepc.v2.api.epa.pattern.symbol.Bindings;
import de.umr.jepc.v2.api.expression.ArithmeticExpression;

/**
 * @author Michael Körber
 *
 */
public class Projection implements EPA {
    
    public static class AttributeProjection {
        private final String attributeOut;
        private final ArithmeticExpression expression;
        /**
         * @param attributeOut
         * @param expression
         */
        public AttributeProjection(String attributeOut, ArithmeticExpression expression) {
            this.attributeOut = attributeOut;
            this.expression = expression;
        }
        
        public String getAttributeOut() {
            return attributeOut;
        }
        
        public ArithmeticExpression getExpression() {
            return expression;
        }

        @Override
        public int hashCode() {
            final int prime = 31;
            int result = 1;
            result = prime * result + ((attributeOut == null) ? 0 : attributeOut.hashCode());
            result = prime * result + ((expression == null) ? 0 : expression.hashCode());
            return result;
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj)
                return true;
            if (obj == null)
                return false;
            if (getClass() != obj.getClass())
                return false;
            AttributeProjection other = (AttributeProjection) obj;
            if (attributeOut == null) {
                if (other.attributeOut != null)
                    return false;
            } else if (!attributeOut.equals(other.attributeOut))
                return false;
            if (expression == null) {
                if (other.expression != null)
                    return false;
            } else if (!expression.equals(other.expression))
                return false;
            return true;
        }

        @Override
        public String toString() {
            return attributeOut + "<-" + expression;
        }
    }

    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    
    /** The input-EPA of this filter */
    private EPA input;
    
    private final AttributeProjection[] projections;
    
    /**
     * @param input
     * @param projections
     */
    public Projection(EPA input, AttributeProjection... projections) {
        this.input = input;
        this.projections = projections;
    }
    
    public AttributeProjection[] getProjections() {
        return projections;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public EPA[] getInputEPAs() {
        return new EPA[] { input };
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setInput(int idx, EPA input) {
        if ( idx == 0 )
            this.input = input;
        else
            throw new IllegalArgumentException("Invalid input index: " + idx);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public EventSchema computeOutputSchema(EventSchema... inputSchemas) throws SchemaException {
        List<Attribute> result = new ArrayList<>();
        
        for ( AttributeProjection ap : projections ) {
            try {
                result.add( new Attribute(ap.getAttributeOut(), ap.expression.getDataType(inputSchemas[0], new Bindings())) );
            } catch (IncompatibleTypeException e) {
                throw new SchemaException("Illegal projection", e);
            }
        }
        return new EventSchema(result.toArray(new Attribute[result.size()]));
    }
    
    

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((input == null) ? 0 : input.hashCode());
        result = prime * result + Arrays.hashCode(projections);
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        Projection other = (Projection) obj;
        if (input == null) {
            if (other.input != null)
                return false;
        } else if (!input.equals(other.input))
            return false;
        if (!Arrays.equals(projections, other.projections))
            return false;
        return true;
    }

    @Override
    public String toString() {
        return "Projection [input=" + input + ", projections=" + Arrays.toString(projections) + "]";
    }
    
    

}
