package de.umr.jepc.v2.api.expression;

import de.umr.ep.common.Attribute.DataType;
import de.umr.ep.common.IncompatibleTypeException;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.SchemaException;
import de.umr.jepc.v2.api.epa.pattern.symbol.Bindings;

/**
 * Interface for any arithmetic expression.
 * Arithmetic expression in this case means, that it is evaluated
 * to a known {@link DataType}.
 * @author Michael Körber
 *
 */
public interface ArithmeticExpression extends Expression {
	
	/**
	 * {@inheritDoc}
	 */
	ArithmeticExpression getInput(int index);
	
	/**
	 * The JEPC data type of this expression
	 * @param schema the schema of incoming events
	 * @param bindings the active variable bindings
	 * @return the type of this expression
	 * @throws IncompatibleTypeException if input-types do not match this expression's specification
	 * @throws SchemaException if the referenced attribute is not part of the given schema
	 */
	DataType getDataType(EventSchema schema, Bindings bindings) throws IncompatibleTypeException, SchemaException;

}
