package de.umr.jepc.v2.api.expression.arithmetic.atomic;

import de.umr.ep.common.Attribute.DataType;
import de.umr.ep.event.EventSchema;
import de.umr.jepc.v2.api.epa.pattern.symbol.Bindings;
import de.umr.jepc.v2.api.expression.ArithmeticExpression;

/**
 * Implementation of a constant value
 * @author Michael Körber
 *
 * @param <T> the type of the value
 */
public abstract class Constant<T> implements ArithmeticExpression {
	
	/** The value */
	protected final T value;
	
	/** The JEPC data type */
	private final DataType type;
	
	/**
	 * @param value The value
	 * @param type The JEPC data type
	 */
	public Constant(T value, DataType type) {
		this.value = value;
		this.type = type;
	}
	
	/**
	 * @return The value
	 */
	public T getValue() {
		return value;
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
    public int getArity() {
        return 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public ArithmeticExpression getInput(int index) {
        throw new IndexOutOfBoundsException("Constant values have no inputs");
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public DataType getDataType(EventSchema schema, Bindings bindings) {
        return type;
    }

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return value == null ? "null" : value.toString();
	}

}
