package de.umr.jepc.v2.api.expression.arithmetic.atomic;

import de.umr.ep.common.Attribute.DataType;
import de.umr.ep.common.IncompatibleTypeException;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.SchemaException;
import de.umr.jepc.v2.api.epa.pattern.symbol.Binding;
import de.umr.jepc.v2.api.epa.pattern.symbol.Bindings;
import de.umr.jepc.v2.api.epa.pattern.symbol.NoSuchVariableException;
import de.umr.jepc.v2.api.expression.ArithmeticExpression;

/**
 * A variable referencing one of:
 * <ul>
 * <li>An event attribute</li>
 * <li>A variable bound while processing</li>
 * </ul>
 * @author Michael Körber
 *
 */
public class Variable implements ArithmeticExpression {
	
    /** The variable name */
	private final String name;

    /**
     * @param name The variable name
     */
	public Variable(String name) {
		this.name = name;
	}
	
	/**
	 * @return the name of this variable
	 */
	public String getName() {
		return name;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int getArity() {
		return 0;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public ArithmeticExpression getInput(int index) {
		throw new IndexOutOfBoundsException("Variables values have no inputs");
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public DataType getDataType(EventSchema schema, Bindings bindings) throws IncompatibleTypeException, SchemaException {
	    try {
	        return schema.byName(name).getType();
	    } catch ( NoSuchVariableException nsv ) {
	        Binding b = bindings.byName(name);
	        return b.getValue().getDataType(schema, bindings.without(b));
	    }
	}
	
	/**
     * {@inheritDoc}
     */
    public String toString() {
        return name;
    }

	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;

		Variable variable = (Variable) o;

		return name != null ? name.equals(variable.name) : variable.name == null;
	}

	@Override
	public int hashCode() {
		return name != null ? name.hashCode() : 0;
	}
}
