package de.umr.jepc.v2.test.bridge;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ServiceLoader;
import java.util.stream.Stream;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;

import de.umr.ep.event.Event;
import de.umr.jepc.v2.api.EPProvider;
import de.umr.jepc.v2.api.bridge.EPBridge;
import de.umr.jepc.v2.api.bridge.EPBridgeFactory;
import de.umr.jepc.v2.api.util.EventUtil;
import de.umr.jepc.v2.provider.def.DefaultEPProvider;

@RunWith(Parameterized.class)
public abstract class AbstractSemanticTest {

	@Parameters(name="{0}")
	public static Collection<EPBridgeFactory> data() {
		List<EPBridgeFactory> result = new LinkedList<>();

		ServiceLoader<EPBridgeFactory> bridgeService = ServiceLoader.load(EPBridgeFactory.class);

		for (EPBridgeFactory f : bridgeService) {
			result.add(f);
		}
		return result;
	}
	
	private EPBridgeFactory factory;
	
	protected EPProvider provider;
	
	public AbstractSemanticTest( EPBridgeFactory bf ) {
		this.factory = bf;
	}
	

	@Before
	public void setUp() throws Exception {
		EPBridge bridge = factory.createBridge();
		provider = new DefaultEPProvider(bridge);
	}

	@After
	public void tearDown() throws Exception {
		provider.destroy();
	}

	public final void compareResults(List<Event> expected, List<Event> actual) {
		Assert.assertEquals("Number of received events incorrect", expected.size(), actual.size());
		
		Stream<Long> ts = actual.stream().map( e -> e.getT1() ).distinct().sorted();
		
		for ( Iterator<Long> ti = ts.iterator(); ti.hasNext(); ) {
			long t = ti.next();
			List<Event> actualSnapshot = new ArrayList<>();
			List<Event> expectedSnapshot = new ArrayList<>();
			
			actual.stream().filter( e -> e.getT1() == t ).forEach(actualSnapshot::add);
			expected.stream().filter( e -> e.getT1() == t ).forEach(expectedSnapshot::add);
			
			Assert.assertEquals("Snapshots sizes do not match!", expectedSnapshot.size(), actualSnapshot.size()); 
			
			int i = 1;
			for ( Event exp : expectedSnapshot ) {
				boolean found = false;
				for ( Event act : actualSnapshot ) {
					found |= EventUtil.compareEvents(exp, act, 1e-10);
				}
				Assert.assertTrue("Expected event (" + i + ") not found in actual results", found);
				i++;
			}
		}
		
		
		
	}

}
