/*
 * Copyright (c) 2012, 2013, 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.jepc.v2.test.bridge;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.junit.Test;

import de.umr.ep.common.Attribute;
import de.umr.ep.common.EPException;
import de.umr.ep.event.Event;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.impl.SimpleEvent;
import de.umr.jepc.v2.api.bridge.EPBridgeFactory;
import de.umr.jepc.v2.api.epa.PatternMatcher;
import de.umr.jepc.v2.api.epa.Stream;
import de.umr.jepc.v2.api.epa.pattern.Output;
import de.umr.jepc.v2.api.epa.pattern.regex.KleenePlus;
import de.umr.jepc.v2.api.epa.pattern.regex.KleeneStar;
import de.umr.jepc.v2.api.epa.pattern.regex.Occurence;
import de.umr.jepc.v2.api.epa.pattern.regex.Regex;
import de.umr.jepc.v2.api.epa.pattern.symbol.Binding;
import de.umr.jepc.v2.api.epa.pattern.symbol.Bindings;
import de.umr.jepc.v2.api.epa.pattern.symbol.Symbol;
import de.umr.jepc.v2.api.epa.pattern.symbol.Symbols;
import de.umr.jepc.v2.api.epa.window.TimeWindow;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.IntConstant;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.Variable;
import de.umr.jepc.v2.api.expression.arithmetic.compund.numeric.Add;
import de.umr.jepc.v2.api.expression.logical.True;
import de.umr.jepc.v2.api.expression.predicate.Equal;
import de.umr.jepc.v2.api.expression.predicate.Greater;
import de.umr.jepc.v2.api.expression.predicate.Less;
import de.umr.jepc.v2.api.provider.AlreadyRegisteredException;
import de.umr.jepc.v2.api.provider.NoSuchStreamException;
import de.umr.jepc.v2.api.provider.QueryHandle;
import de.umr.jepc.v2.api.provider.StreamHandle;
import de.umr.jepc.v2.api.util.CollectingSink;

/**
 * @author Michael Körber
 *
 */
public class PatternMatcherTest extends AbstractSemanticTest {

    //	private static final Logger log = LoggerFactory.getLogger(PatternMatcherTest.class);

    private static final Stream STREAM = new Stream("test");
    private static final String QUERY  = "pattern";

    public PatternMatcherTest(EPBridgeFactory bf) {
        super(bf);
    }

    @Test
    public void pm1() throws AlreadyRegisteredException, NoSuchStreamException, EPException { // Multiple matches at a time
        PatternMatcher pm = new PatternMatcher(new TimeWindow(STREAM, 1), 100L,
            new Symbols(
                    new Symbol('a', new Equal(new Variable("p"), new IntConstant(42)), new Bindings(new Binding("p1", new Variable("p")))),
                    new Symbol('b', new Less(new Variable("p"), new IntConstant(40)), new Bindings(new Binding("p2", new Variable("p")))),
                    new Symbol('c', new Less(new Variable("p"), new IntConstant(20)), new Bindings(new Binding("p3", new Variable("p")))),
                    new Symbol('d', new Equal(new Variable("p"), new IntConstant(42)), new Bindings(new Binding("p4", new Variable("p"))))),

            new Regex(
                    new Occurence('a'), 
                    new KleeneStar('b'), 
                    new KleenePlus('c'), 
                    new Occurence('d')), 
            new Output("p1", "p2", "p3", "p4"));

        CollectingSink sink = new CollectingSink();

        StreamHandle sh = provider.registerStream(STREAM.getName(), new EventSchema(new Attribute("p", Attribute.DataType.INTEGER)));

        QueryHandle qh = provider.createQuery(QUERY, pm);

        provider.registerSink(qh, sink);
//        List<Event> results = sink.getEvents();
//
//        List<Event> expected = Arrays.asList(
//            new SimpleEvent(new Object[] { 42, 19, 18, 42 }, 600011L, 600012L),
//            new SimpleEvent(new Object[] { 42, 38, 18, 42 }, 600011L, 600012L)
//        );

        sh.pushEvent(new Object[] { 42 }, 600001L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 39 }, 600003L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 38 }, 600005L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 19 }, 600007L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 18 }, 600009L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 42 }, 600011L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 42 }, 600013L);
//        Assert.assertEquals(2, results.size());
//        sh.pushEvent(new Object[] { 42 }, 600015L);
//        Assert.assertEquals(2, results.size());
//        
//        compareResults(expected, results);
    }
    
    @Test
    public void pm2() throws AlreadyRegisteredException, NoSuchStreamException, EPException { // test for parallel universes
        PatternMatcher pm = new PatternMatcher(
            new TimeWindow(STREAM,1), 10L,
            new Symbols(
                new Symbol('a',new True(),new Bindings(new Binding("p1", new Variable("p")))),
                new Symbol('b',new Greater(new Variable("p"),new Variable("p1")), new Bindings(new Binding("p2",new Variable("p"))))
            ),
            new Regex( new Occurence('a'), new Occurence('b') ),
            new Output("p1","p2")
        );
        
        CollectingSink sink = new CollectingSink();

        StreamHandle sh = provider.registerStream(STREAM.getName(), new EventSchema(new Attribute("p", Attribute.DataType.INTEGER)));

        QueryHandle qh = provider.createQuery(QUERY, pm);
        provider.registerSink(qh, sink);
//        List<Event> results = sink.getEvents();
        
        List<Event> tmp = Arrays.asList(
                new SimpleEvent( new Object[] {1, 2}, 600010L, 600011L ),
                new SimpleEvent( new Object[] {2, 3}, 600015L, 600016L ),
                new SimpleEvent( new Object[] {3, 4}, 600020L, 600021L ),
                new SimpleEvent( new Object[] {4, 5}, 600025L, 600026L ),
                new SimpleEvent( new Object[] {5, 6}, 600030L, 600031L ),
                new SimpleEvent( new Object[] {6, 7}, 600035L, 600036L ),
                new SimpleEvent( new Object[] {7, 8}, 600040L, 600041L ),
                new SimpleEvent( new Object[] {8, 9}, 600045L, 600046L ),
                new SimpleEvent( new Object[] {9, 10}, 600050L, 600051L ),
                new SimpleEvent( new Object[] {10, 11}, 600055L, 600056L ),
                new SimpleEvent( new Object[] {11, 12}, 600060L, 600061L ),
                new SimpleEvent( new Object[] {12, 13}, 600065L, 600066L ),
                new SimpleEvent( new Object[] {13, 14}, 600070L, 600071L ),
                new SimpleEvent( new Object[] {14, 15}, 600075L, 600076L ),
                new SimpleEvent( new Object[] {15, 16}, 600080L, 600081L ),
                new SimpleEvent( new Object[] {16, 17}, 600085L, 600086L ),
                new SimpleEvent( new Object[] {17, 18}, 600090L, 600091L ),
                new SimpleEvent( new Object[] {18, 19}, 600095L, 600096L ),
                new SimpleEvent( new Object[] {19, 20}, 600100L, 600101L ),
                new SimpleEvent( new Object[] {20, 21}, 600105L, 600106L ),
                new SimpleEvent( new Object[] {21, 22}, 600110L, 600111L ),
                new SimpleEvent( new Object[] {22, 23}, 600115L, 600116L ),
                new SimpleEvent( new Object[] {23, 24}, 600120L, 600121L ),
                new SimpleEvent( new Object[] {24, 25}, 600125L, 600126L ),
                new SimpleEvent( new Object[] {25, 26}, 600130L, 600131L ),
                new SimpleEvent( new Object[] {26, 27}, 600135L, 600136L ),
                new SimpleEvent( new Object[] {27, 28}, 600140L, 600141L )
        );
        
        List<Event> expected = new ArrayList<>();
        expected.add(new SimpleEvent( new Object[] {0, 1}, 600005L, 600006L));
        expected.add(new SimpleEvent( new Object[] {0, 1}, 600005L, 600006L));
        
        for ( Event e : tmp ) {
            for ( int i = 0; i < 4; i++ )
                expected.add(e);
        }
        
        sh.pushEvent(new Object[] { 0 }, 600000L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 1 }, 600005L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 1 }, 600005L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 2 }, 600010L);
//        Assert.assertEquals(2,results.size());
//        sh.pushEvent(new Object[] { 2 }, 600010L);
//        Assert.assertEquals(2,results.size());
//        sh.pushEvent(new Object[] { 3 }, 600015L);
//        Assert.assertEquals(6,results.size());
//        sh.pushEvent(new Object[] { 3 }, 600015L);
//        Assert.assertEquals(6,results.size());
//        sh.pushEvent(new Object[] { 4 }, 600020L);
//        Assert.assertEquals(10,results.size());
//        sh.pushEvent(new Object[] { 4 }, 600020L);
//        Assert.assertEquals(10,results.size());
//        sh.pushEvent(new Object[] { 5 }, 600025L);
//        Assert.assertEquals(14,results.size());
//        sh.pushEvent(new Object[] { 5 }, 600025L);
//        Assert.assertEquals(14,results.size());
//        sh.pushEvent(new Object[] { 6 }, 600030L);
//        Assert.assertEquals(18,results.size());
//        sh.pushEvent(new Object[] { 6 }, 600030L);
//        Assert.assertEquals(18,results.size());
//        sh.pushEvent(new Object[] { 7 }, 600035L);
//        Assert.assertEquals(22,results.size());
//        sh.pushEvent(new Object[] { 7 }, 600035L);
//        Assert.assertEquals(22,results.size());
//        sh.pushEvent(new Object[] { 8 }, 600040L);
//        Assert.assertEquals(26,results.size());
//        sh.pushEvent(new Object[] { 8 }, 600040L);
//        Assert.assertEquals(26,results.size());
//        sh.pushEvent(new Object[] { 9 }, 600045L);
//        Assert.assertEquals(30,results.size());
//        sh.pushEvent(new Object[] { 9 }, 600045L);
//        Assert.assertEquals(30,results.size());
//        sh.pushEvent(new Object[] { 10 }, 600050L);
//        Assert.assertEquals(34,results.size());
//        sh.pushEvent(new Object[] { 10 }, 600050L);
//        Assert.assertEquals(34,results.size());
//        sh.pushEvent(new Object[] { 11 }, 600055L);
//        Assert.assertEquals(38,results.size());
//        sh.pushEvent(new Object[] { 11 }, 600055L);
//        Assert.assertEquals(38,results.size());
//        sh.pushEvent(new Object[] { 12 }, 600060L);
//        Assert.assertEquals(42,results.size());
//        sh.pushEvent(new Object[] { 12 }, 600060L);
//        Assert.assertEquals(42,results.size());
//        sh.pushEvent(new Object[] { 13 }, 600065L);
//        Assert.assertEquals(46,results.size());
//        sh.pushEvent(new Object[] { 13 }, 600065L);
//        Assert.assertEquals(46,results.size());
//        sh.pushEvent(new Object[] { 14 }, 600070L);
//        Assert.assertEquals(50,results.size());
//        sh.pushEvent(new Object[] { 14 }, 600070L);
//        Assert.assertEquals(50,results.size());
//        sh.pushEvent(new Object[] { 15 }, 600075L);
//        Assert.assertEquals(54,results.size());
//        sh.pushEvent(new Object[] { 15 }, 600075L);
//        Assert.assertEquals(54,results.size());
//        sh.pushEvent(new Object[] { 16 }, 600080L);
//        Assert.assertEquals(58,results.size());
//        sh.pushEvent(new Object[] { 16 }, 600080L);
//        Assert.assertEquals(58,results.size());
//        sh.pushEvent(new Object[] { 17 }, 600085L);
//        Assert.assertEquals(62,results.size());
//        sh.pushEvent(new Object[] { 17 }, 600085L);
//        Assert.assertEquals(62,results.size());
//        sh.pushEvent(new Object[] { 18 }, 600090L);
//        Assert.assertEquals(66,results.size());
//        sh.pushEvent(new Object[] { 18 }, 600090L);
//        Assert.assertEquals(66,results.size());
//        sh.pushEvent(new Object[] { 19 }, 600095L);
//        Assert.assertEquals(70,results.size());
//        sh.pushEvent(new Object[] { 19 }, 600095L);
//        Assert.assertEquals(70,results.size());
//        sh.pushEvent(new Object[] { 20 }, 600100L);
//        Assert.assertEquals(74,results.size());
//        sh.pushEvent(new Object[] { 20 }, 600100L);
//        Assert.assertEquals(74,results.size());
//        sh.pushEvent(new Object[] { 21 }, 600105L);
//        Assert.assertEquals(78,results.size());
//        sh.pushEvent(new Object[] { 21 }, 600105L);
//        Assert.assertEquals(78,results.size());
//        sh.pushEvent(new Object[] { 22 }, 600110L);
//        Assert.assertEquals(82,results.size());
//        sh.pushEvent(new Object[] { 22 }, 600110L);
//        Assert.assertEquals(82,results.size());
//        sh.pushEvent(new Object[] { 23 }, 600115L);
//        Assert.assertEquals(86,results.size());
//        sh.pushEvent(new Object[] { 23 }, 600115L);
//        Assert.assertEquals(86,results.size());
//        sh.pushEvent(new Object[] { 24 }, 600120L);
//        Assert.assertEquals(90,results.size());
//        sh.pushEvent(new Object[] { 24 }, 600120L);
//        Assert.assertEquals(90,results.size());
//        sh.pushEvent(new Object[] { 25 }, 600125L);
//        Assert.assertEquals(94,results.size());
//        sh.pushEvent(new Object[] { 25 }, 600125L);
//        Assert.assertEquals(94,results.size());
//        sh.pushEvent(new Object[] { 26 }, 600130L);
//        Assert.assertEquals(98,results.size());
//        sh.pushEvent(new Object[] { 26 }, 600130L);
//        Assert.assertEquals(98,results.size());
//        sh.pushEvent(new Object[] { 27 }, 600135L);
//        Assert.assertEquals(102,results.size());
//        sh.pushEvent(new Object[] { 27 }, 600135L);
//        Assert.assertEquals(102,results.size());
//        sh.pushEvent(new Object[] { 28 }, 600140L);
//        Assert.assertEquals(106,results.size());
//        sh.pushEvent(new Object[] { 28 }, 600140L);
//        Assert.assertEquals(106,results.size());
//        sh.pushEvent(new Object[] { 29 }, 600145L);
//        Assert.assertEquals(110,results.size());
//        sh.pushEvent(new Object[] { 29 }, 600145L);
//        Assert.assertEquals(110,results.size());
//        
//        compareResults(expected, results);
    }
    
    
    @Test
    public void pm3() throws AlreadyRegisteredException, NoSuchStreamException, EPException { // test for 3 (not even!) parallel universes
        PatternMatcher pm = new PatternMatcher(new TimeWindow(STREAM, 1), 10L,
            new Symbols(
                new Symbol('a', new Equal(new Variable("p"),new IntConstant(42)), new Bindings(new Binding("p1",new Variable("p")))),
                new Symbol('b', new True(), new Bindings(new Binding("p2", new Variable("p")))),
                new Symbol('c', new Equal(new Variable("p"),new IntConstant(50)), new Bindings(new Binding("p3", new Variable("p"))))
            ),
            new Regex(
                new Occurence('a'),
                new KleeneStar('b'),
                new Occurence('c')
            ),
            new Output("p1","p2","p3")
        );
        
        CollectingSink sink = new CollectingSink();

        StreamHandle sh = provider.registerStream(STREAM.getName(), new EventSchema(new Attribute("p", Attribute.DataType.INTEGER)));

        QueryHandle qh = provider.createQuery(QUERY, pm);

        provider.registerSink(qh, sink);
//        List<Event> results = sink.getEvents();
//
//        List<Event> expected = Arrays.asList(
//            new SimpleEvent(new Object[] { 42, 32, 50 }, 600004L, 600005L),
//            new SimpleEvent(new Object[] { 42, 32, 50 }, 600004L, 600005L),
//            new SimpleEvent(new Object[] { 42, 32, 50 }, 600004L, 600005L)
//        );
        
        
        sh.pushEvent(new Object[] { 42 }, 600000L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 30 }, 600001L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 31 }, 600001L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 32 }, 600001L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 32 }, 600002L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 32 }, 600003L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 50 }, 600004L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 33 }, 600005L);
//        Assert.assertEquals(3, results.size());
//
//        compareResults(expected, results);
    }
    
    
    @Test
    public void pm7() throws AlreadyRegisteredException, NoSuchStreamException, EPException { // homepage -> Quick Start -> PM
        PatternMatcher pm = new PatternMatcher(
            new TimeWindow(STREAM, 1), 100L,
            new Symbols(
                new Symbol('a', new Equal(new Variable("smoke"), new IntConstant(0) ), new Bindings(new Binding("a_temp", new Variable("temperature")))),
                new Symbol('b', new Equal(new Variable("smoke"), new IntConstant(1) ), new Bindings(new Binding("location", new Variable("sensorID")))),
                new Symbol('c', new Greater(new Variable("temperature"), new Add(new Variable("a_temp"), new IntConstant(1))))
            ),
            new Regex(
                new Occurence('a'),
                new Occurence('b'),
                new Occurence('c')
            ),
            new Output("a_temp","location")
        );
        
        CollectingSink sink = new CollectingSink();

        StreamHandle sh = provider.registerStream(STREAM.getName(), new EventSchema(
                new Attribute("sensorID",    Attribute.DataType.INTEGER),
                new Attribute("smoke",       Attribute.DataType.BYTE),
                new Attribute("temperature", Attribute.DataType.FLOAT)));

        QueryHandle qh = provider.createQuery(QUERY, pm);

        provider.registerSink(qh, sink);
//        List<Event> results = sink.getEvents();
//
//        List<Event> expected = Arrays.asList(
//            new SimpleEvent(new Object[] { 22.5, 7 }, 1430L, 1431L),
//            new SimpleEvent(new Object[] { 22.5, 4 }, 1430L, 1431L)
//        );
        
        sh.pushEvent(new Object[] { 20, 0, 22.4f }, 1400L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 20, 0, 22.3f }, 1410L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 20, 0, 22.5f }, 1420L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 4, 1, 0.0f }, 1423L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 7, 1, 0.0f }, 1423L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 20, 0, 23.8f }, 1430L);
//        Assert.assertTrue(results.isEmpty());
//        sh.pushEvent(new Object[] { 20, 0, 24.2f }, 1440L);
//        Assert.assertEquals(2, results.size());
//        sh.pushEvent(new Object[] { 20, 0, 24.6f }, 1450L);
//        Assert.assertEquals(2, results.size());
//        
//        compareResults(expected, results);
    }
    
}
