/*
 * Copyright (c) 2012, 2013, 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.jepc.v2.test.bridge;

import java.util.Arrays;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;

import de.umr.ep.common.Attribute;
import de.umr.ep.common.Attribute.DataType;
import de.umr.ep.common.EPException;
import de.umr.ep.event.Event;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.impl.SimpleEvent;
import de.umr.jepc.v2.api.bridge.EPBridgeFactory;
import de.umr.jepc.v2.api.epa.Projection;
import de.umr.jepc.v2.api.epa.Projection.AttributeProjection;
import de.umr.jepc.v2.api.epa.Stream;
import de.umr.jepc.v2.api.epa.window.TimeWindow;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.DoubleConstant;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.Variable;
import de.umr.jepc.v2.api.expression.arithmetic.compund.numeric.Add;
import de.umr.jepc.v2.api.expression.arithmetic.compund.numeric.Pow;
import de.umr.jepc.v2.api.expression.arithmetic.compund.numeric.Sqrt;
import de.umr.jepc.v2.api.provider.AlreadyRegisteredException;
import de.umr.jepc.v2.api.provider.QueryHandle;
import de.umr.jepc.v2.api.provider.StreamHandle;
import de.umr.jepc.v2.api.util.CollectingSink;

/**
 * @author Michael Körber
 *
 */
public class ProjectionTest extends AbstractSemanticTest {

//	private static final Logger log = LoggerFactory.getLogger(FilterTest.class);
    
    private static final Stream STREAM = new Stream("test");
    private static final String QUERY  = "project";

	public ProjectionTest(EPBridgeFactory bf) {
		super(bf);
	}

	/**
	 * Tests >= condition
	 * @throws AlreadyRegisteredException
	 * @throws EPException
	 */
	@Test
	public void projection1() throws EPException {

		Projection p = new Projection(
		        new TimeWindow(STREAM, 1),
		        new AttributeProjection("DISTORIGIN",
        		        new Sqrt(new Add(new Pow(new Variable("X"), new DoubleConstant(2.0)),
                                 new Add(new Pow(new Variable("Y"), new DoubleConstant(2.0)),
                                         new Pow(new Variable("Z"), new DoubleConstant(2.0))))))
		        
		);

		CollectingSink sink = new CollectingSink();

		StreamHandle sh = provider.registerStream(STREAM.getName(), new EventSchema(
		        new Attribute("X", DataType.DOUBLE),
		        new Attribute("Y", DataType.DOUBLE),
		        new Attribute("Z", DataType.DOUBLE)));
		
		QueryHandle qh = provider.createQuery(QUERY, p);
		
		provider.registerSink(qh, sink);
		List<Event> results = sink.getEvents();

		List<Event> expected = Arrays.asList(
	        new SimpleEvent(new Object[] { Math.sqrt(3.0) }, 1L, 2L), 
	        new SimpleEvent(new Object[] { Math.sqrt(12.0) }, 2L, 3L)
        );

		sh.pushEvent(new Object[] { 1.0, 1.0, 1.0 }, 1);
		Assert.assertEquals(1, results.size());
		sh.pushEvent(new Object[] { 2.0, 2.0, 2.0 }, 2);
        Assert.assertEquals(2, results.size());
        compareResults(expected, results);
	}
}
