package de.umr.jepc.v2.test.bridge;

import java.util.Arrays;
import java.util.List;

import org.junit.Test;

import de.umr.ep.common.Attribute;
import de.umr.ep.common.EPException;
import de.umr.ep.event.Event;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.impl.SimpleEvent;
import de.umr.jepc.v2.api.bridge.EPBridgeFactory;
import de.umr.jepc.v2.api.epa.Stream;
import de.umr.jepc.v2.api.epa.window.CountWindow;
import de.umr.jepc.v2.api.epa.window.TimeWindow;
import de.umr.jepc.v2.api.provider.QueryHandle;
import de.umr.jepc.v2.api.provider.StreamHandle;
import de.umr.jepc.v2.api.util.CollectingSink;

/**
 * Created by Christian on 24.01.2017.
 */
public class WindowTest  extends AbstractSemanticTest {

    private static final Stream STREAM = new Stream("test");
    private static final String QUERY  = "window";

    public WindowTest(EPBridgeFactory bf) {
        super(bf);
    }

    /**
     * Tests a timeWindow with size of 1 milliseconds (jump 1)
     * @throws EPException
     */
    @Test
    public void windowTest1() throws EPException {
        TimeWindow window = new TimeWindow(STREAM, 1);
        CollectingSink sink = new CollectingSink();

        StreamHandle sh = provider.registerStream(STREAM.getName(), new EventSchema(
                new Attribute("id", Attribute.DataType.LONG)));

        QueryHandle qh = provider.createQuery(QUERY, window);
        provider.registerSink(qh, sink);
        //push Events in sh und test output from qh against expected events
        for (long i = 0; i <= 20; i++) {
            sh.pushEvent(new Object[] { i }, i );
        }
        List<Event> expected = Arrays.asList(
                new SimpleEvent(new Object[] { 0L }, 0L, 1L),                
                new SimpleEvent(new Object[] { 1L }, 1L, 2L),
                new SimpleEvent(new Object[] { 2L }, 2L, 3L),
                new SimpleEvent(new Object[] { 3L }, 3L, 4L),
                new SimpleEvent(new Object[] { 4L }, 4L, 5L),
                new SimpleEvent(new Object[] { 5L }, 5L, 6L),
                new SimpleEvent(new Object[] { 6L }, 6L, 7L),
                new SimpleEvent(new Object[] { 7L }, 7L, 8L),
                new SimpleEvent(new Object[] { 8L }, 8L, 9L),
                new SimpleEvent(new Object[] { 9L }, 9L, 10L),
                new SimpleEvent(new Object[] { 10L }, 10L, 11L),
                new SimpleEvent(new Object[] { 11L }, 11L, 12L),
                new SimpleEvent(new Object[] { 12L }, 12L, 13L),
                new SimpleEvent(new Object[] { 13L }, 13L, 14L),
                new SimpleEvent(new Object[] { 14L }, 14L, 15L),
                new SimpleEvent(new Object[] { 15L }, 15L, 16L),
                new SimpleEvent(new Object[] { 16L }, 16L, 17L),
                new SimpleEvent(new Object[] { 17L }, 17L, 18L),
                new SimpleEvent(new Object[] { 18L }, 18L, 19L),
                new SimpleEvent(new Object[] { 19L }, 19L, 20L),
                new SimpleEvent(new Object[] { 20L }, 20L, 21L)
        );
        List<Event> results = sink.getEvents();
        compareResults(expected, results);
    }

    /**
     * Tests a timeWindow with size of 2 milliseconds (jump 1)
     * @throws EPException
     */
    @Test
    public void windowTest2() throws EPException {
        TimeWindow window = new TimeWindow(STREAM, 2);
        CollectingSink sink = new CollectingSink();

        StreamHandle sh = provider.registerStream(STREAM.getName(), new EventSchema(
                new Attribute("id", Attribute.DataType.LONG)));

        QueryHandle qh = provider.createQuery(QUERY, window);
        provider.registerSink(qh, sink);
        //push Events in sh und test output from qh against expected events
        for (long i = 0; i <= 20; i++) {
            sh.pushEvent(new Object[] { i }, i );
        }
        List<Event> expected = Arrays.asList(
                new SimpleEvent(new Object[] { 0L }, 0L, 2L),
                new SimpleEvent(new Object[] { 1L }, 1L, 3L),
                new SimpleEvent(new Object[] { 2L }, 2L, 4L),
                new SimpleEvent(new Object[] { 3L }, 3L, 5L),
                new SimpleEvent(new Object[] { 4L }, 4L, 6L),
                new SimpleEvent(new Object[] { 5L }, 5L, 7L),
                new SimpleEvent(new Object[] { 6L }, 6L, 8L),
                new SimpleEvent(new Object[] { 7L }, 7L, 9L),
                new SimpleEvent(new Object[] { 8L }, 8L, 10L),
                new SimpleEvent(new Object[] { 9L }, 9L, 11L),
                new SimpleEvent(new Object[] { 10L }, 10L, 12L),
                new SimpleEvent(new Object[] { 11L }, 11L, 13L),
                new SimpleEvent(new Object[] { 12L }, 12L, 14L),
                new SimpleEvent(new Object[] { 13L }, 13L, 15L),
                new SimpleEvent(new Object[] { 14L }, 14L, 16L),
                new SimpleEvent(new Object[] { 15L }, 15L, 17L),
                new SimpleEvent(new Object[] { 16L }, 16L, 18L),
                new SimpleEvent(new Object[] { 17L }, 17L, 19L),
                new SimpleEvent(new Object[] { 18L }, 18L, 20L),
                new SimpleEvent(new Object[] { 19L }, 19L, 21L),
                new SimpleEvent(new Object[] { 20L }, 20L, 22L)
        );
        List<Event> results = sink.getEvents();
        compareResults(expected, results);
    }

    /**
     * Tests a timeWindow with size of 3 milliseconds (jump 1)
     * @throws EPException
     */
    @Test
    public void windowTest3() throws EPException {
        TimeWindow window = new TimeWindow(STREAM, 3);
        CollectingSink sink = new CollectingSink();

        StreamHandle sh = provider.registerStream(STREAM.getName(), new EventSchema(
                new Attribute("id", Attribute.DataType.LONG)));

        QueryHandle qh = provider.createQuery(QUERY, window);
        provider.registerSink(qh, sink);
        //push Events in sh und test output from qh against expected events
        for (long i = 0; i <= 20; i++) {
            sh.pushEvent(new Object[] { i }, i );
        }
        List<Event> expected = Arrays.asList(
                new SimpleEvent(new Object[] { 0L }, 0L, 3L),
                new SimpleEvent(new Object[] { 1L }, 1L, 4L),
                new SimpleEvent(new Object[] { 2L }, 2L, 5L),
                new SimpleEvent(new Object[] { 3L }, 3L, 6L),
                new SimpleEvent(new Object[] { 4L }, 4L, 7L),
                new SimpleEvent(new Object[] { 5L }, 5L, 8L),
                new SimpleEvent(new Object[] { 6L }, 6L, 9L),
                new SimpleEvent(new Object[] { 7L }, 7L, 10L),
                new SimpleEvent(new Object[] { 8L }, 8L, 11L),
                new SimpleEvent(new Object[] { 9L }, 9L, 12L),
                new SimpleEvent(new Object[] { 10L }, 10L, 13L),
                new SimpleEvent(new Object[] { 11L }, 11L, 14L),
                new SimpleEvent(new Object[] { 12L }, 12L, 15L),
                new SimpleEvent(new Object[] { 13L }, 13L, 16L),
                new SimpleEvent(new Object[] { 14L }, 14L, 17L),
                new SimpleEvent(new Object[] { 15L }, 15L, 18L),
                new SimpleEvent(new Object[] { 16L }, 16L, 19L),
                new SimpleEvent(new Object[] { 17L }, 17L, 20L),
                new SimpleEvent(new Object[] { 18L }, 18L, 21L),
                new SimpleEvent(new Object[] { 19L }, 19L, 22L),
                new SimpleEvent(new Object[] { 20L }, 20L, 23L)
        );
        List<Event> results = sink.getEvents();
        compareResults(expected, results);
    }

    /**
     * Tests a timeWindow with size of 3 and jump of 3 milliseconds
     * @throws EPException
     */
    @Test
    public void windowTest4() throws EPException {
        TimeWindow window = new TimeWindow(STREAM, 3, 3);
        CollectingSink sink = new CollectingSink();

        StreamHandle sh = provider.registerStream(STREAM.getName(), new EventSchema(
                new Attribute("id", Attribute.DataType.LONG),
                new Attribute("value", Attribute.DataType.DOUBLE)));

        QueryHandle qh = provider.createQuery(QUERY, window);
        provider.registerSink(qh, sink);
        //push Events in sh und test output from qh against expected events
        for (long i = 0; i <= 20; i++) {
            sh.pushEvent(new Object[] { i }, i );
        }
        List<Event> expected = Arrays.asList(
                new SimpleEvent(new Object[] { 0L }, 0L, 3L),
                new SimpleEvent(new Object[] { 1L }, 0L, 3L),
                new SimpleEvent(new Object[] { 2L }, 0L, 3L),
                new SimpleEvent(new Object[] { 3L }, 3L, 6L),
                new SimpleEvent(new Object[] { 4L }, 3L, 6L),
                new SimpleEvent(new Object[] { 5L }, 3L, 6L),
                new SimpleEvent(new Object[] { 6L }, 6L, 9L),
                new SimpleEvent(new Object[] { 7L }, 6L, 9L),
                new SimpleEvent(new Object[] { 8L }, 6L, 9L),
                new SimpleEvent(new Object[] { 9L }, 9L, 12L),
                new SimpleEvent(new Object[] { 10L }, 9L, 12L),
                new SimpleEvent(new Object[] { 11L }, 9L, 12L),
                new SimpleEvent(new Object[] { 12L }, 12L, 15L),
                new SimpleEvent(new Object[] { 13L }, 12L, 15L),
                new SimpleEvent(new Object[] { 14L }, 12L, 15L),
                new SimpleEvent(new Object[] { 15L }, 15L, 18L),
                new SimpleEvent(new Object[] { 16L }, 15L, 18L),
                new SimpleEvent(new Object[] { 17L }, 15L, 18L),
                new SimpleEvent(new Object[] { 18L }, 18L, 21L),
                new SimpleEvent(new Object[] { 19L }, 18L, 21L),
                new SimpleEvent(new Object[] { 20L }, 18L, 21L)
        );
        List<Event> results = sink.getEvents();
        compareResults(expected, results);

    }

    /**
     * Tests a timeWindow with size of 3 and jump of 4 milliseconds
     * @throws EPException
     */
    @Test
    public void windowTest5() throws EPException {
        TimeWindow window = new TimeWindow(STREAM, 3, 4);
        CollectingSink sink = new CollectingSink();

        StreamHandle sh = provider.registerStream(STREAM.getName(), new EventSchema(
                new Attribute("id", Attribute.DataType.LONG),
                new Attribute("value", Attribute.DataType.DOUBLE)));

        QueryHandle qh = provider.createQuery(QUERY, window);
        provider.registerSink(qh, sink);
        //push Events in sh und test output from qh against expected events
        for (long i = 0; i <= 20; i++) {
            sh.pushEvent(new Object[] { i }, i );
        }
        List<Event> expected = Arrays.asList(
                new SimpleEvent(new Object[] { 1L }, 0L, 4L),
                new SimpleEvent(new Object[] { 2L }, 0L, 4L),
                new SimpleEvent(new Object[] { 3L }, 0L, 4L),
                new SimpleEvent(new Object[] { 5L }, 4L, 8L),
                new SimpleEvent(new Object[] { 6L }, 4L, 8L),
                new SimpleEvent(new Object[] { 7L }, 4L, 8L),
                new SimpleEvent(new Object[] { 9L }, 8L, 12L),
                new SimpleEvent(new Object[] { 10L }, 8L, 12L),
                new SimpleEvent(new Object[] { 11L }, 8L, 12L),
                new SimpleEvent(new Object[] { 13L }, 12L, 16L),
                new SimpleEvent(new Object[] { 14L }, 12L, 16L),
                new SimpleEvent(new Object[] { 15L }, 12L, 16L),
                new SimpleEvent(new Object[] { 17L }, 16L, 20L),
                new SimpleEvent(new Object[] { 18L }, 16L, 20L),
                new SimpleEvent(new Object[] { 19L }, 16L, 20L)
        );
        List<Event> results = sink.getEvents();
        compareResults(expected, results);
    }

    /**
     * Tests a timeWindow with size of 3 and jump of 5 milliseconds
     * @throws EPException
     */
    @Test
    public void windowTest6() throws EPException {
        TimeWindow window = new TimeWindow(STREAM, 3, 5);
        CollectingSink sink = new CollectingSink();

        StreamHandle sh = provider.registerStream(STREAM.getName(), new EventSchema(
                new Attribute("id", Attribute.DataType.LONG),
                new Attribute("value", Attribute.DataType.DOUBLE)));

        QueryHandle qh = provider.createQuery(QUERY, window);
        provider.registerSink(qh, sink);
        //push Events in sh und test output from qh against expected events
        for (long i = 0; i <= 20; i++) {
            sh.pushEvent(new Object[] { i }, i );
        }
        List<Event> expected = Arrays.asList(
                new SimpleEvent(new Object[] { 2L }, 0L, 5L),
                new SimpleEvent(new Object[] { 3L }, 0L, 5L),
                new SimpleEvent(new Object[] { 4L }, 0L, 5L),
                new SimpleEvent(new Object[] { 7L }, 5L, 10L),
                new SimpleEvent(new Object[] { 8L }, 5L, 10L),
                new SimpleEvent(new Object[] { 9L }, 5L, 10L),
                new SimpleEvent(new Object[] { 12L }, 10L, 15L),
                new SimpleEvent(new Object[] { 13L }, 10L, 15L),
                new SimpleEvent(new Object[] { 14L }, 10L, 15L),
                new SimpleEvent(new Object[] { 17L }, 15L, 20L),
                new SimpleEvent(new Object[] { 18L }, 15L, 20L),
                new SimpleEvent(new Object[] { 19L }, 15L, 20L)
        );
        List<Event> results = sink.getEvents();
        compareResults(expected, results);
    }

    /**
     * Tests a countWindow with size of 3 events (jump 1)
     * @throws EPException
     */
    @Test
    public void windowTest7() throws EPException {
        CountWindow window = new CountWindow(STREAM, 2);
        CollectingSink sink = new CollectingSink();

        StreamHandle sh = provider.registerStream(STREAM.getName(), new EventSchema(
                new Attribute("id", Attribute.DataType.LONG),
                new Attribute("value", Attribute.DataType.DOUBLE)));

        QueryHandle qh = provider.createQuery(QUERY, window);
        provider.registerSink(qh, sink);
        //push Events in sh und test output from qh against expected events
        for (int j = 1; j < 13; j++) {
            sh.pushEvent(new Object[] { j }, j );
        }
        List<Event> expected = Arrays.asList(
                new SimpleEvent(new Object[] { 1 } , 1L, 3L),
                new SimpleEvent(new Object[] { 2 } , 2L, 4L),
                new SimpleEvent(new Object[] { 3 } , 3L, 5L),
                new SimpleEvent(new Object[] { 4 } , 4L, 6L),
                new SimpleEvent(new Object[] { 5 } , 5L, 7L),
                new SimpleEvent(new Object[] { 6 } , 6L, 8L),
                new SimpleEvent(new Object[] { 7 } , 7L, 9L),
                new SimpleEvent(new Object[] { 8 } , 8L, 10L),
                new SimpleEvent(new Object[] { 9 } , 9L, 11L),
                new SimpleEvent(new Object[] { 10 } , 10L, 12L)
        );
        List<Event> results = sink.getEvents();
        compareResults(expected, results);
    }
    
    /**
     * Tests a countWindow with size of 3 events (jump 1)
     * @throws EPException
     */
    @Test
    public void windowTest8() throws EPException {
        CountWindow window = new CountWindow(STREAM, 2, 2);
        CollectingSink sink = new CollectingSink();

        StreamHandle sh = provider.registerStream(STREAM.getName(), new EventSchema(
                new Attribute("id", Attribute.DataType.LONG),
                new Attribute("value", Attribute.DataType.DOUBLE)));

        QueryHandle qh = provider.createQuery(QUERY, window);
        provider.registerSink(qh, sink);
        //push Events in sh und test output from qh against expected events
        for (int j = 1; j < 13; j++) {
            sh.pushEvent(new Object[] { j }, j );
        }
        List<Event> expected = Arrays.asList(
                new SimpleEvent(new Object[] { 1 } , 2L, 4L),
                new SimpleEvent(new Object[] { 2 } , 2L, 4L),
                new SimpleEvent(new Object[] { 3 } , 4L, 6L),
                new SimpleEvent(new Object[] { 4 } , 4L, 6L),
                new SimpleEvent(new Object[] { 5 } , 6L, 8L),
                new SimpleEvent(new Object[] { 6 } , 6L, 8L),
                new SimpleEvent(new Object[] { 7 } , 8L, 10L),
                new SimpleEvent(new Object[] { 8 } , 8L, 10L),
                new SimpleEvent(new Object[] { 9 } ,  10L, 12L),
                new SimpleEvent(new Object[] { 10 } , 10L, 12L)
        );
        List<Event> results = sink.getEvents();
        compareResults(expected, results);
    }
    
    /**
     * Tests a countWindow with size of 2 and jump of 3 events
     * @throws EPException
     */
    @Test
    public void windowTest9() throws EPException {
        CountWindow window = new CountWindow(STREAM, 4,3);
        CollectingSink sink = new CollectingSink();

        StreamHandle sh = provider.registerStream(STREAM.getName(), new EventSchema(
                new Attribute("id", Attribute.DataType.LONG),
                new Attribute("value", Attribute.DataType.DOUBLE)));

        QueryHandle qh = provider.createQuery(QUERY, window);
        provider.registerSink(qh, sink);
        //push Events in sh und test output from qh against expected events
        for (int j = 1; j < 13; j++) {
            sh.pushEvent(new Object[] { j }, j );
        }
        List<Event> expected = Arrays.asList(
                new SimpleEvent(new Object[] { 1 } , 3L, 6L),
                new SimpleEvent(new Object[] { 2 } , 3L, 6L),
                new SimpleEvent(new Object[] { 3 } , 3L, 9L),
                new SimpleEvent(new Object[] { 4 } , 6L, 9L),
                new SimpleEvent(new Object[] { 5 } , 6L, 9L),
                new SimpleEvent(new Object[] { 6 } , 6L, 12L),
                new SimpleEvent(new Object[] { 7 } , 9L, 12L),
                new SimpleEvent(new Object[] { 8 } , 9L, 12L)
        );
        List<Event> results = sink.getEvents();
        compareResults(expected, results);
    }

    /**
     * Tests a countWindow with size of 2 and jump of 3 events
     * @throws EPException
     */
    @Test
    public void windowTest10() throws EPException {
        CountWindow window = new CountWindow(STREAM, 2,3);
        CollectingSink sink = new CollectingSink();

        StreamHandle sh = provider.registerStream(STREAM.getName(), new EventSchema(
                new Attribute("id", Attribute.DataType.LONG),
                new Attribute("value", Attribute.DataType.DOUBLE)));

        QueryHandle qh = provider.createQuery(QUERY, window);
        provider.registerSink(qh, sink);
        //push Events in sh und test output from qh against expected events
        for (int j = 1; j < 13; j++) {
            sh.pushEvent(new Object[] { j }, j );
        }
        List<Event> expected = Arrays.asList(
                new SimpleEvent(new Object[] { 2 } , 3L, 6L),
                new SimpleEvent(new Object[] { 3 } , 3L, 6L),
                new SimpleEvent(new Object[] { 5 } , 6L, 9L),
                new SimpleEvent(new Object[] { 6 } , 6L, 9L),
                new SimpleEvent(new Object[] { 8 } , 9L, 12L),
                new SimpleEvent(new Object[] { 9 } , 9L, 12L)
        );
        List<Event> results = sink.getEvents();
        results.forEach(System.out::println);
        compareResults(expected, results);
    }

}
