package de.umr.jepc.v2.provider.def.handler;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.umr.ep.event.Event;
import de.umr.ep.event.EventSchema;
import de.umr.jepc.v2.api.bridge.EPACallback;
import de.umr.jepc.v2.api.bridge.EventChannel;
import de.umr.jepc.v2.api.epa.EPA;
import de.umr.jepc.v2.api.epa.UserDefinedEPA;
import de.umr.jepc.v2.api.provider.EventSink;

public class EPAHandler implements EPACallback {
	
	private static final Logger log = LoggerFactory.getLogger(EPAHandler.class);
    
    private EventChannel channel = null;
    
    private int refCount = 0;

	private final EPA def;
	
	private List<EventSink> sinks;
	
	private List<UserDefinedEPA> udos = new ArrayList<>();

	public EPAHandler(EPA def) {
		this.def = def;
	}
	
	public EPA getDefinition() {
		return def;
	}
	
	public void addUserDefinedOperator( UserDefinedEPA udo ) {
	    udos.add(udo);
	}
	
	public void removeUserDefinedOperator( UserDefinedEPA udo ) {
        udos.remove(udo);
    }
	
	@Override
	public void receive(Event event) {
		if ( log.isDebugEnabled() )
			log.debug("Received event on {}: {}", def, event );
		
		if ( sinks != null && !sinks.isEmpty() )
			for ( EventSink sink : sinks )
				sink.receiveEvent(event);
		
		for ( UserDefinedEPA udo : udos )
		    udo.process(getResultChannel(), event);
		
		getResultChannel().push( event );
	}
	
	public void setSinks( final List<EventSink> sinks ) {
		this.sinks = sinks;
	}
	
	public final EventChannel getResultChannel() {
        return channel;
    }
    
    public final void setResultChannel( EventChannel channel ) {
        this.channel = channel;
    }
    
    public EventSchema getSchema() {
        return this.channel.getSchema();
    }
    
    public void increaseRefCount() {
        refCount++;
    }
    
    public void decreaseRefCount() {
        refCount--;
    }
    
    public int getRefCount() {
        return refCount;
    }
}
