package de.umr.jepc.v2.provider.def.handler;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import de.umr.ep.event.EventSchema;
import de.umr.jepc.v2.api.epa.EPA;
import de.umr.jepc.v2.api.provider.EventSink;
import de.umr.jepc.v2.api.provider.QueryHandle;

public class Query implements QueryHandle, Iterable<EPAHandler> {
	
	private final String name;
	
	private final EPA def;
	
	private final LinkedList<EPAHandler> epas = new LinkedList<>();
	
	private final List<EventSink> sinks = new ArrayList<>();
	
	private EventSchema outputSchema;

	public Query(String name, EPA def) {
		this.name = name;
		this.def = def;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public EventSchema getOutputSchema() {
		return outputSchema;
	}
	
	public EPA getDefinition() {
		return def;
	}
	
	public void addEPA( EPAHandler epa ) {
		epa.increaseRefCount();
		// Rewire schema and sinks
		outputSchema = epa.getResultChannel().getSchema();
//		if ( !epas.isEmpty() ) 
//			epas.getLast().setSinks(null);
//		epa.setSinks(sinks);
		epas.add(epa);
	}
	
	public void addSink( EventSink sink ) {
		sinks.add(sink);
		if ( !epas.isEmpty() ) 
          epas.getLast().setSinks(sinks);
		
	}
	
	public void removeSink( EventSink sink ) {
		sinks.remove(sink);
		if ( !epas.isEmpty() ) 
	          epas.getLast().setSinks(sinks);
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public Iterator<EPAHandler> iterator() {
        return epas.descendingIterator();
    }
}
