package de.umr.jepc.v2.expressions.util;

import de.umr.ep.event.Event;
import de.umr.jepc.v2.bindings.BoundVariables;
import de.umr.jepc.v2.expressions.EvaluableExpression;

public abstract class EVAbstractUnaryExpression<TIN,TOUT> implements EvaluableExpression<TOUT> {
	
	protected static interface OperatorImpl<TIN,TOUT> {
		TOUT apply(TIN in);
	}
	
	private final EvaluableExpression<TIN> input;
	private final OperatorImpl<TIN,TOUT> impl;

	public EVAbstractUnaryExpression(EvaluableExpression<TIN> input, OperatorImpl<TIN,TOUT> impl) {
		this.input = input;
		this.impl = impl;
	}

	@Override
	public TOUT eval(Event event, BoundVariables bindings) {
		return impl.apply(input.eval(event, bindings)); 
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((impl == null) ? 0 : impl.hashCode());
		result = prime * result + ((input == null) ? 0 : input.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		EVAbstractUnaryExpression<?,?> other = (EVAbstractUnaryExpression<?,?>) obj;
		if (impl == null) {
			if (other.impl != null)
				return false;
		} else if (!impl.equals(other.impl))
			return false;
		if (input == null) {
			if (other.input != null)
				return false;
		} else if (!input.equals(other.input))
			return false;
		return true;
	}
}
