package de.umr.jepc.v2.ql.impl.atomic;

import org.kohsuke.MetaInfServices;

import de.umr.ep.common.Attribute;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.SchemaException;
import de.umr.jepc.v2.api.epa.pattern.symbol.Bindings;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.Variable;
import de.umr.jepc.v2.expressions.arithmetic.field.EVByteField;
import de.umr.jepc.v2.expressions.arithmetic.field.EVDoubleField;
import de.umr.jepc.v2.expressions.arithmetic.field.EVFloatField;
import de.umr.jepc.v2.expressions.arithmetic.field.EVIntField;
import de.umr.jepc.v2.expressions.arithmetic.field.EVLongField;
import de.umr.jepc.v2.expressions.arithmetic.field.EVNumericField;
import de.umr.jepc.v2.expressions.arithmetic.field.EVShortField;
import de.umr.jepc.v2.ql.ExpressionTranslator;
import de.umr.jepc.v2.ql.Translator;
import de.umr.jepc.v2.ql.TranslatorException;

@MetaInfServices
public class NumberVariableTranslator extends Translator<Variable, EVNumericField> {

	public NumberVariableTranslator() {
		super(Variable.class, EVNumericField.class);
	}

	@Override
	protected EVNumericField process(Variable in, ExpressionTranslator translator, EventSchema schema, Bindings bindings)
			throws TranslatorException, SchemaException {
		int index = schema.getAttributeIndex(in.getName());
		Attribute attr = schema.byName(in.getName());
		switch ( attr.getType() ) {
		case BYTE: return new EVByteField(index);
		case SHORT: return new EVShortField(index);
		case INTEGER: return new EVIntField(index);
		case LONG: return new EVLongField(index);
		case FLOAT: return new EVFloatField(index);
		case DOUBLE: return new EVDoubleField(index);
		default: throw new TranslatorException("Not a numeric attribute: " + in.getName() );
		}
	}
	
	

}
