package de.umr.jepc.v2.ql.impl.predicate;

import org.kohsuke.MetaInfServices;

import de.umr.ep.common.Attribute.DataType;
import de.umr.ep.common.IncompatibleTypeException;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.SchemaException;
import de.umr.jepc.v2.api.epa.pattern.symbol.Bindings;
import de.umr.jepc.v2.api.expression.ArithmeticExpression;
import de.umr.jepc.v2.api.expression.predicate.Between;
import de.umr.jepc.v2.expressions.arithmetic.EVNumericExpression;
import de.umr.jepc.v2.expressions.arithmetic.EVStringExpression;
import de.umr.jepc.v2.expressions.predicate.EVPredicate;
import de.umr.jepc.v2.expressions.predicate.numeric.EVNumericBetween;
import de.umr.jepc.v2.expressions.predicate.string.EVStringBetween;
import de.umr.jepc.v2.ql.ExpressionTranslator;
import de.umr.jepc.v2.ql.Translator;
import de.umr.jepc.v2.ql.TranslatorException;

@MetaInfServices
public class BetweenTranslator extends Translator<Between, EVPredicate> {

	public BetweenTranslator() {
		super(Between.class, EVPredicate.class);
	}

	@Override
	protected EVPredicate process(Between in, ExpressionTranslator translator, EventSchema schema, Bindings bindings)
			throws TranslatorException, IncompatibleTypeException, SchemaException {
		ArithmeticExpression v = in.getInput(0);
		ArithmeticExpression l = in.getInput(1);
		ArithmeticExpression u = in.getInput(2);
		
		DataType vt = l.getDataType(schema, bindings);
		DataType lt = l.getDataType(schema, bindings);
		DataType ut = u.getDataType(schema, bindings);

		DataType gct = DataType.getGCT(vt, DataType.getGCT(lt, ut));

		if (gct.isNumeric()) {
		    EVNumericExpression tv = translator.translateNumericExpression(v, schema, bindings);
			EVNumericExpression tl = translator.translateNumericExpression(l, schema, bindings);
			EVNumericExpression tu = translator.translateNumericExpression(u, schema, bindings);
			return new EVNumericBetween(gct, tv, tl, tu);
		} else if (gct == DataType.STRING) {
		    EVStringExpression tv = translator.translateStringExpression(v, schema, bindings);
			EVStringExpression tl = translator.translateStringExpression(l, schema, bindings);
			EVStringExpression tu = translator.translateStringExpression(u, schema, bindings);
			return new EVStringBetween(tv, tl, tu);
		} else
			throw new IncompatibleTypeException("No implementation for Between with input-types: " + vt + ", " + lt + ", " + ut);
	}
}
