package de.umr.jepc.v2.ql.impl.predicate;

import org.kohsuke.MetaInfServices;

import de.umr.ep.common.Attribute.DataType;
import de.umr.ep.common.IncompatibleTypeException;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.SchemaException;
import de.umr.jepc.v2.api.epa.pattern.symbol.Bindings;
import de.umr.jepc.v2.api.expression.ArithmeticExpression;
import de.umr.jepc.v2.api.expression.predicate.Equal;
import de.umr.jepc.v2.expressions.arithmetic.EVNumericExpression;
import de.umr.jepc.v2.expressions.arithmetic.EVStringExpression;
import de.umr.jepc.v2.expressions.predicate.EVPredicate;
import de.umr.jepc.v2.expressions.predicate.numeric.EVNumericEqual;
import de.umr.jepc.v2.expressions.predicate.string.EVStringEqual;
import de.umr.jepc.v2.ql.ExpressionTranslator;
import de.umr.jepc.v2.ql.Translator;
import de.umr.jepc.v2.ql.TranslatorException;

@MetaInfServices
public class EqualTranslator extends Translator<Equal, EVPredicate> {

	public EqualTranslator() {
		super(Equal.class, EVPredicate.class);
	}

	@Override
	protected EVPredicate process(Equal in, ExpressionTranslator translator, EventSchema schema, Bindings bindings)
			throws TranslatorException, IncompatibleTypeException, SchemaException {
		ArithmeticExpression l = in.getInput(0);
		ArithmeticExpression r = in.getInput(1);
		DataType lt = l.getDataType(schema, bindings);
		DataType rt = r.getDataType(schema, bindings);

		DataType gct = DataType.getGCT(lt, rt);

		if (gct.isNumeric()) {
			EVNumericExpression tl = translator.translateNumericExpression(l, schema, bindings);
			EVNumericExpression tr = translator.translateNumericExpression(r, schema, bindings);
			return new EVNumericEqual(gct, tl, tr);
		} else if (gct == DataType.STRING) {
			EVStringExpression tl = translator.translateStringExpression(l, schema, bindings);
			EVStringExpression tr = translator.translateStringExpression(r, schema, bindings);
			return new EVStringEqual(tl, tr);
		} else
			throw new IncompatibleTypeException("No implementation for Equal with input-types: " + lt + ", " + rt);
	}
}
