/**
 * 
 */
package de.umr.jepc.v2.federated.api.epa;

import de.umr.ep.event.EventSchema;
import de.umr.ep.event.SchemaException;
import de.umr.jepc.v2.api.epa.EPA;
import de.umr.jepc.v2.federated.api.provider.Resource;
import de.umr.jepc.v2.federated.api.provider.SplitStrategy;

/**
 * Signals that all inputs of this EPA are to be run 
 * on the given resource.
 * This EPA is inserted into the query-graph by implementations
 * of {@link SplitStrategy} to distribute a query among available
 * {@link Resource resources}
 * 
 * @author Michael Körber
 */
public class ResourceBoundary implements EPA {
    
    private static final long serialVersionUID = 1L;

    /** The id of the resource to deploy the sub-query on */
    private final String resourceId;
    
    /** The sub-query to distribute */
    private final EPA subQuery;
    
    
    /**
     * @param resourceId the id of the resource to deploy the sub-query on
     * @param subQuery the sub-query to distribute
     */
    public ResourceBoundary(String resourceId, EPA subQuery) {
        this.resourceId = resourceId;
        this.subQuery = subQuery;
    }
    
    /**
	 * @return The id of the resource to deploy the sub-query on
	 */
	public String getResourceId() {
		return resourceId;
	}

	/**
     * {@inheritDoc}
     */
    @Override
    public EPA[] getInputEPAs() {
        return new EPA[] {subQuery};
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString() {
        return "ResourceBoundary [resourceId=" + resourceId + ", input=" + subQuery + "]";
    }


	/**
	 * {@inheritDoc}
	 */
	@Override
	public EventSchema computeOutputSchema(EventSchema... inputSchemas) throws SchemaException {
		return inputSchemas[0];
	}


	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setInput(int idx, EPA input) {
		throw new UnsupportedOperationException("Rebase of ResourceBoundary not permitted not permitted");
	}

    @Override
    public int getDepth() {
        return subQuery.getDepth();
    }
	
	
}
