/**
 * 
 */
package de.umr.jepc.v2.federated.api.provider;

import java.util.List;

import de.umr.jepc.v2.api.provider.NoSuchStreamException;
import de.umr.jepc.v2.api.schema.StreamInfo;

/**
 * A resource is an abstract description of a processing unit
 * inside a federated EP system. It may host an arbitrary number of streams 
 * and is capable of executing queries. Depending on the nature of the system
 * (centralized, distributed), this interface may be extended with required information
 * for query distribution.
 * 
 * @author Michael Körber
 */
public interface Resource {

    /**
     * @return the unique identifier of this resource
     */
    String getIdentifier();
    
    /**
     * @return A list of streams provided by this resource
     */
    List<StreamInfo> getProvidedStreams();
    
    /**
     * @return the stream info for the given name
     * @throws NoSuchStreamException if no stream with the given name is available on this resource 
     */
    default StreamInfo getStreamByName( String name ) throws NoSuchStreamException {
    	for ( StreamInfo si : getProvidedStreams() )
    		if ( si.getName().equals(name) )
    			return si;
    	throw new NoSuchStreamException("Stream \"" + name + "\" is not provided by this resource");
    }
}
