/**
 * 
 */
package de.umr.jepc.v2.federated.api.provider;

import de.umr.jepc.v2.api.epa.EPA;
import de.umr.jepc.v2.federated.api.provider.query.FederatedSink;
import de.umr.jepc.v2.federated.api.util.ResourceException;

/**
 * A resource manager is responsible for deploying and undeploying (sub-) queries on the registered resources.
 * @author Michael Körber
 */
public interface ResourceManager<T extends Resource> {
    
    /**
     * Deploys a sink on the master-resource, so that query-results may be consumed
     * @param masterResource the master-resource
     * @param name the name of the query to deploy the sink for
     * @param input the description of result events
     * @param sink the sink to deploy
     * @throws ResourceException on any error deploying the sink
     */
    void deploySink( T masterResource, String name, EPA epn, FederatedSink sink ) throws ResourceException;

    /**
     * Deploys the given (sub-) query on the given {@link Resource resource} and installs a channel for sending results
     * to the successor {@link Resource resource}.
     * @param resource the resource to deploy the (sub-) query on
     * @param toResource the resource to send results to
     * @param name the name of the query
     * @param epn the operator-graph of the (sub-) query
     * @throws ResourceException on any error deploying the query
     */
    void deployQuery( T resource, T toResource, String name, EPA epn ) throws ResourceException;
    
    /**
     * Removes the query with the given name from the given resource. All channels
     * publishing results must be terminared, too.
     * @param resource the resource to remove the (sub-) query from
     * @param name the name of the (sub-) query to remove.
     * @throws ResourceException on any error removing the query
     */
    void removeQuery( T resource, String name ) throws ResourceException;

}
