/**
 * 
 */
package de.umr.jepc.v2.federated.api.provider;

import de.umr.jepc.v2.api.epa.EPA;
import de.umr.jepc.v2.federated.api.epa.ResourceBoundary;
import de.umr.jepc.v2.federated.api.util.ResourceException;

/**
 * A SplitStrategy is responsible for dissecting a query into multiple
 * parts and assign a computation resource to each of the parts.
 * Therefore, it receives an operator-graph as input and inserts
 * {@link ResourceBoundary boundaries} between them to signal
 * a resource change. The contract is, that each subsequent
 * operator should be executed on the resource specified by a boundary.
 * @author Michael Körber
 *
 */
public interface SplitStrategy<T extends Resource> {

	/**
	 * Inserts {@link ResourceBoundary boundaries} into the given operator-graph
	 * to tell how this query should be distributed among avialable resources.
	 * Two contracts must hold for the returned operator graph:
	 * <ul>
	 *     <li>The root operator is a {@link ResourceBoundary boundary}</li>
	 *     <li>The event sources are (not necessarily directly) followed by a {@link ResourceBoundary boundary} referencing the resource where they are created. 
	 * </ul>
	 * @param query the query to split up
	 * @param resources the available resources
	 * @return the query with inserted {@link ResourceBoundary boundaries}
	 * @throws ResourceException if a required resource is not registered
	 */
    EPA split( EPA query, Resources<? extends T> resources ) throws ResourceException;

}
