/**
 * 
 */
package de.umr.jepc.v2.federated.api.provider.query;

import java.util.ArrayDeque;
import java.util.Deque;
import java.util.Iterator;

import de.umr.ep.event.EventSchema;
import de.umr.jepc.v2.api.provider.QueryHandle;

/**
 * Holds all information about a federated query. This includes
 * the sub-queries and the resources they are executed on as well as the sink consuming 
 * the query's results.
 * 
 * @author Michael Körber
 *
 */
public class FederatedQuery implements Iterable<SubQuery>, QueryHandle {
	
    /** The name of the query */
	private final String name;
	
	/** The sub-queries */
	private final Deque<SubQuery> parts = new ArrayDeque<>();
	
	/** The results sink */
	private final FederatedSink sink = new FederatedSink();

	/**
	 * @param name the name of the query
	 */
	public FederatedQuery(String name) {
		this.name = name;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public String getName() {
        return name;
    }
	
	 /**
     * {@inheritDoc}
     */
    @Override
    public EventSchema getOutputSchema() {
        return parts.peek().getOutputSchema();
    }

    /**
     * Adds a new sub-query
     * @param subQuery the sub-query to add
     */
    public void addSubQuery( SubQuery subQuery ) {
		this.parts.push(subQuery);
	}
    
    /**
     * @return the sink receiving results of this query.
     */
	public FederatedSink getSink() {
        return sink;
    }

    /**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("FederatedQuery [").append(name).append('\n');
		for ( SubQuery qp : parts ) {
			sb.append("  ").append(qp).append('\n');
		}
		sb.append(']');
		
		return sb.toString();
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public Iterator<SubQuery> iterator() {
        return parts.iterator();
    }
    
    /**
     * @return the sub-queries in reversed order (from sources to root)
     */
    public Iterator<SubQuery> reverseIterator() {
        return parts.descendingIterator();
    }
}
