/**
 * 
 */
package de.umr.jepc.v2.federated.api.provider.query;

import java.util.HashSet;
import java.util.Set;

import de.umr.ep.event.Event;
import de.umr.jepc.v2.api.provider.EventSink;

/**
 * This sink is deployed at the end of each federeated query 
 * on the master resource and receives all results of query it is assigned to.
 * The results are then distributed to all {@link EventSink sinks} registered by the
 * user.
 * @author Michael Körber
 *
 */
public class FederatedSink {
    
    /** The user defined sinks to serve */
    private final Set<EventSink> sinks = new HashSet<>();

    public FederatedSink() {
    }

    /**
     * Callback invoked on each result-event of the associated query.
     * Invokes all user-defined event sinks.
     * @param event the result event
     */
    public void push( Event event ) {
        for ( EventSink s : sinks )
            s.receiveEvent(event);
    }
    
    /**
     * Adds a user-defined event sink
     * @param sink the sink to add
     */
    public void addSink( EventSink sink ) {
        sinks.add(sink);
    }
    
    /**
     * Removes a user-defined event sink
     * @param sink the sink to remove
     */
    public void removeSink( EventSink sink ) {
        sinks.remove(sink);
    }

}
