/**
 * 
 */
package de.umr.jepc.v2.federated.api.provider.query;

import de.umr.ep.event.EventSchema;
import de.umr.jepc.v2.api.epa.EPA;

/**
 * Description of a sub-query running on a dedicated resource inside a federeated
 * EP system.
 * @author Michael Körber
 *
 */
public class SubQuery {
	
	/** The id of the resource this sub-query runs on */
	private final String resourceId;
	
	/** The id of the resource to which results are published */
	private final String toResourceId;
	
	/** The operator-graph of this sub-query */
	private final EPA epn;
	
	/** The schema of result events */
	private final EventSchema outputSchema;

	/**
	 * @param resourceId The id of the resource this sub-query runs on
	 * @param toResourceId The id of the resource to which results are published
	 * @param queryPart The operator-graph of this sub-query
	 * @param outputSchema The schema of result events
	 */
	public SubQuery(String resourceId, String toResourceId, EPA queryPart, EventSchema outputSchema) {
		this.resourceId = resourceId;
		this.toResourceId = toResourceId;
		this.epn = queryPart;
		this.outputSchema = outputSchema;
	}

	/**
	 * @return The id of the resource this sub-query runs on
	 */
	public String getResourceId() {
		return resourceId;
	}
	
	/**
	 * @return The id of the resource to which results are published
	 */
	public String getToResourceId() {
        return toResourceId;
    }

    /**
	 * @return The operator-graph of this sub-query
	 */
	public EPA getQueryPart() {
		return epn;
	}

	/**
	 * @return The schema of result events
	 */
	public EventSchema getOutputSchema() {
		return outputSchema;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "QueryPart [resourceId=" + resourceId + ", toResourceId=" + toResourceId + ", queryPart=" + epn + ", outputSchema=" + outputSchema
				+ "]";
	}
	
	
}
