/**
 * 
 */
package de.umr.jepc.v2.federated.api.resource.query;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import de.umr.jepc.v2.api.provider.EventSink;
import de.umr.jepc.v2.api.provider.ExternalStreamHandle;
import de.umr.jepc.v2.api.provider.QueryHandle;
import de.umr.jepc.v2.federated.api.epa.ForeignInput;
import de.umr.jepc.v2.federated.api.provider.Resource;

/**
 * Represents a (sub-) query deployed on a {@link Resource} of the federated system.
 * @author Michael Körber
 *
 */
public class LocalQuery {

    /** The name of the query */
    private String name;
    
    /** The handle returned by the procsessing engine */
    private QueryHandle localHandle;
    
    /** The foreign streams serving as input to the query */
    private Map<String,ExternalStreamHandle> foreignStreams = new HashMap<>();
    
    /** The sink for collecting results */
    private EventSink sink;

    /**
     * @param name the name of the query
     */
    public LocalQuery(String name) {
        this.name = name;
    }
    
    /**
     * Retrieves the handle to inject events received from the given resource into the processing engine
     * @param resourceId the id of the resource
     * @return the handle for the foreign stream 
     */
    public ExternalStreamHandle getForeignStream( String resourceId ) {
        return foreignStreams.get(resourceId);
    }
    
    /**
     * Adds the given handle as foreign input to this query
     * @param def the foreign input definition
     * @param handle the handle to inject events into the processing engine
     */
    public void addForeignStream( ForeignInput def, ExternalStreamHandle handle ) {
        this.foreignStreams.put(def.getOriginResourceId(), handle);
    }

    /**
     * @return the name of the query
     */
    public String getName() {
        return name;
    }

    /**
     * @return the query handle obtained from the processing engine
     */
    public QueryHandle getLocalHandle() {
        return localHandle;
    }

    /**
     * Sets the handle obtained when registering the (sub-) query with the processing engine.
     * @param localHandle the query handle obtained from the processing engine
     */
    public void setLocalHandle(QueryHandle localHandle) {
        this.localHandle = localHandle;
    }

    /**
     * @return all registered handles for foreing streams
     */
    public Collection<ExternalStreamHandle> getForeignStreams() {
        return foreignStreams.values();
    }

    /**
     * @return the event sink receiving all results of this (sub-) query.
     */
    public EventSink getSink() {
        return sink;
    }

    /**
     * Sets the event sink receiving all results of this (sub-) query.
     * @param sink the sink to set
     */
    public void setSink(EventSink sink) {
        this.sink = sink;
    }
}
