/**
 * 
 */
package de.umr.jepc.v2.federated.api.provider;

import java.util.ArrayList;
import java.util.List;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import de.umr.ep.common.Attribute;
import de.umr.ep.common.Attribute.DataType;
import de.umr.ep.common.EPException;
import de.umr.ep.event.EventSchema;
import de.umr.jepc.v2.api.epa.Correlator;
import de.umr.jepc.v2.api.epa.EPA;
import de.umr.jepc.v2.api.epa.Filter;
import de.umr.jepc.v2.api.epa.Stream;
import de.umr.jepc.v2.api.epa.window.TimeWindow;
import de.umr.jepc.v2.api.expression.logical.True;
import de.umr.jepc.v2.api.schema.StreamInfo;
import de.umr.jepc.v2.federated.api.FederatedEPProvider;
import de.umr.jepc.v2.federated.api.epa.QualifiedStream;
import de.umr.jepc.v2.federated.api.provider.query.FederatedQuery;
import de.umr.jepc.v2.federated.api.provider.query.SubQuery;
import de.umr.jepc.v2.federated.api.provider.strategy.SimpleSplitStrategy;

/**
 * @author Michael Körber
 *
 */
public class FederatedProviderTest {
    
    private SimpleSplitStrategy strategy = new SimpleSplitStrategy();
    
    private TestManager manager = new TestManager();
    
    @Before
    public void setup() {
        this.strategy = new SimpleSplitStrategy();
        this.manager = new TestManager();
    }
    

	/**
	 * Test method for {@link de.umr.jepc.v2.federated.api.FederatedEPProvider#createQuery(java.lang.String, de.umr.jepc.v2.api.epa.EPA)}.
	 * @throws JEPCException 
	 */
	@Test
	public void testCreateSingleResourceQuery() throws EPException {
	    TestResource root = new TestResource("root");
		TestResource tr = new TestResource("A", new StreamInfo("S", new EventSchema(new Attribute("A1",DataType.BYTE))));
		EPA query = new Filter( new TimeWindow(new QualifiedStream("A", new Stream("S")),1), new True() );
		
		FederatedEPProvider<TestResource> c = new FederatedEPProvider<>(strategy, root, manager);
		c.addResource(tr);
		FederatedQuery qh = (FederatedQuery) c.createQuery("TestQuery", query);
		
		List<SubQuery> parts = new ArrayList<>();
		for ( SubQuery qp : qh )
		    parts.add(qp);
		
		Assert.assertEquals(1, parts.size());
		Assert.assertEquals("A", parts.get(0).getResourceId());
		Assert.assertEquals("root", parts.get(0).getToResourceId());
		c.destroy();
	}
	
	@Test
	public void testCreateSingleJoinQuery() throws EPException {
	    TestResource root = new TestResource("root");
		TestResource tr1 = new TestResource("A", new StreamInfo("S", new EventSchema(new Attribute("A1",DataType.BYTE))));
		TestResource tr2 = new TestResource("B", new StreamInfo("S", new EventSchema(new Attribute("A1",DataType.BYTE))));
		
		EPA query = new Correlator(
				new Filter( new TimeWindow(new QualifiedStream("A", new Stream("S")),1), new True() ), "A",
				new Filter( new TimeWindow(new QualifiedStream("B", new Stream("S")),1), new True() ), "B",
				new True() );
		
		
		FederatedEPProvider<TestResource> c = new FederatedEPProvider<>(strategy, root, manager);
		c.addResource(tr1);
		c.addResource(tr2);
		
		FederatedQuery qh = (FederatedQuery) c.createQuery("TestQuery", query);
        List<SubQuery> parts = new ArrayList<>();
        for ( SubQuery qp : qh )
            parts.add(qp);
        
        Assert.assertEquals(2, parts.size());
        Assert.assertEquals("B", parts.get(1).getResourceId());
        Assert.assertEquals("A", parts.get(1).getToResourceId());
        Assert.assertEquals("A", parts.get(0).getResourceId());
        Assert.assertEquals("root", parts.get(0).getToResourceId());
        c.destroy();
	}
	
	@Test
	public void testCreateTripleJoinQuery() throws EPException {
	    TestResource root = new TestResource("root");
		TestResource tr1 = new TestResource("A", new StreamInfo("S", new EventSchema(new Attribute("A1",DataType.BYTE))));
		TestResource tr2 = new TestResource("B", new StreamInfo("S", new EventSchema(new Attribute("A1",DataType.BYTE))));
		TestResource tr3 = new TestResource("C", new StreamInfo("S", new EventSchema(new Attribute("A1",DataType.BYTE))));
		
		EPA query = new Correlator(
			new TimeWindow(new QualifiedStream("C", new Stream("S")),1), "C",
			new Correlator(
				new Filter( new TimeWindow(new QualifiedStream("A", new Stream("S")),1), new True() ), "A",
				new Filter( new TimeWindow(new QualifiedStream("B", new Stream("S")),1), new True() ), "B",
				new True() ), null,
			new True() );
		
		
		FederatedEPProvider<TestResource> c = new FederatedEPProvider<>(strategy, root, manager);
		c.addResource(tr1);
		c.addResource(tr2);
		c.addResource(tr3);
		
		FederatedQuery qh = (FederatedQuery) c.createQuery("TestQuery", query);
        List<SubQuery> parts = new ArrayList<>();
        for ( SubQuery qp : qh )
            parts.add(qp);
        
        Assert.assertEquals(3, parts.size());
        Assert.assertEquals("B", parts.get(2).getResourceId());
        Assert.assertEquals("A", parts.get(2).getToResourceId());
        Assert.assertEquals("A", parts.get(1).getResourceId());
        Assert.assertEquals("C", parts.get(1).getToResourceId());
        Assert.assertEquals("C", parts.get(0).getResourceId());
        Assert.assertEquals("root", parts.get(0).getToResourceId());
        c.destroy();
	}

}
