/**
 * 
 */
package de.umr.jepc.v2.federated.local.provider;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.umr.ep.common.EPException;
import de.umr.ep.event.Event;
import de.umr.jepc.v2.api.epa.EPA;
import de.umr.jepc.v2.api.provider.NoSuchQueryException;
import de.umr.jepc.v2.federated.api.provider.ResourceManager;
import de.umr.jepc.v2.federated.api.provider.query.FederatedSink;
import de.umr.jepc.v2.federated.api.resource.RemoteEventChannel;
import de.umr.jepc.v2.federated.api.resource.ResourceInstance;
import de.umr.jepc.v2.federated.api.util.ResourceException;
import de.umr.jepc.v2.federated.local.resource.LocalResource;

/**
 * Local implementation of the {@link ResourceManager}.
 * Local resources hold a reference to the corresponding {@link ResourceInstance} and pass events via method calls.
 * @author Michael Körber
 *
 */
public class LocalResourceManager implements ResourceManager<LocalResource> {
    
    private static final Logger log = LoggerFactory.getLogger(LocalResourceManager.class);

    /**
     * {@inheritDoc}
     */
    @Override
    public void deploySink(LocalResource masterResource, String name, EPA epn, FederatedSink sink ) throws ResourceException {
        try {
            masterResource.getInstance().deployQuery(name, epn, new RemoteEventChannel() {
                @Override
                public void pushEvent(Event event) {
                    sink.push(event);
                }
            });
        } catch (EPException e) {
            throw new ResourceException("Could not deploy sink on resource " + masterResource.getIdentifier(), e);
        }
        
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void deployQuery(LocalResource resource, LocalResource toResource, String name, EPA epn) throws ResourceException {
        try {
            resource.getInstance().deployQuery(name, epn, new RemoteEventChannel() {
                @Override
                public void pushEvent(Event event) {
                    try {
                        toResource.getInstance().pushForeignEvent(resource.getIdentifier(), name, event);
                    } catch (NoSuchQueryException e) {
                        log.error("Could not forward event to target-resource. From: " + resource.getIdentifier() + ", to: " + toResource.getIdentifier(),e);
                    }
                }
            });
        } catch (EPException e) {
            throw new ResourceException("Could not deploy query: " + name + " on resource " + resource.getIdentifier(), e);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void removeQuery(LocalResource resource, String name) throws ResourceException {
        try {
            resource.getInstance().removeQuery(name);
        } catch (EPException e) {
            throw new ResourceException("Could not remove query: " + name + " from resource " + resource.getIdentifier(), e);
        }
    }

}
