/**
 * 
 */
package de.umr.jepc.v2.federated.local.resource;

import java.util.List;

import de.umr.jepc.v2.api.schema.StreamInfo;
import de.umr.jepc.v2.federated.api.provider.Resource;
import de.umr.jepc.v2.federated.api.resource.ResourceInstance;

/**
 * Local implementation of the resource interface.
 * Holds a reference to a {@link ResourceInstance} and 
 * allows the manager to deploy queries directly via method calls.
 * @author Michael Körber
 *
 */
public class LocalResource implements Resource {
    
    /** The id of this resource */
    private final String id;
    
    /** Information about provided streams */
    private final List<StreamInfo> streams;
    
    /** The instance this resource refers to */
    private final ResourceInstance instance;
    
    /**
     * @param id The id of this resource
     * @param providedStreams Information about provided streams
     * @param instance The instance this resource refers to
     */
    public LocalResource(String id, List<StreamInfo> providedStreams, ResourceInstance instance) {
        this.id = id;
        this.streams = providedStreams;
        this.instance = instance;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getIdentifier() {
        return id;
    }

    /**
     * {@inheritDoc}
     */ 
    @Override
    public List<StreamInfo> getProvidedStreams() {
        return streams;
    }
    
    /**
     * @return The instance this resource refers to
     */
    public ResourceInstance getInstance() {
        return instance;
    }
}
