/**
 * 
 */
package de.umr.jepc.v2.federated.local;

import org.junit.Assert;
import org.junit.Test;

import de.umr.ep.common.Attribute;
import de.umr.ep.common.Attribute.DataType;
import de.umr.ep.common.EPException;
import de.umr.ep.event.Event;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.SchemaException;
import de.umr.jepc.v2.api.epa.Correlator;
import de.umr.jepc.v2.api.epa.EPA;
import de.umr.jepc.v2.api.epa.Filter;
import de.umr.jepc.v2.api.epa.Stream;
import de.umr.jepc.v2.api.epa.window.TimeWindow;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.IntConstant;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.Variable;
import de.umr.jepc.v2.api.expression.predicate.Equal;
import de.umr.jepc.v2.api.expression.predicate.GreaterEq;
import de.umr.jepc.v2.api.provider.AlreadyRegisteredException;
import de.umr.jepc.v2.api.provider.QueryHandle;
import de.umr.jepc.v2.api.schema.StreamInfo;
import de.umr.jepc.v2.api.util.CollectingSink;
import de.umr.jepc.v2.federated.api.FederatedEPProvider;
import de.umr.jepc.v2.federated.api.epa.QualifiedStream;
import de.umr.jepc.v2.federated.api.provider.strategy.SimpleSplitStrategy;
import de.umr.jepc.v2.federated.local.provider.LocalResourceManager;
import de.umr.jepc.v2.federated.local.resource.LocalResource;

/**
 * @author Michael Körber
 *
 */
public class LocalFederationTest {
    
    /** Each resource provides the same stream */
    private static final StreamInfo STREAM;
    
    static {
        try {
            STREAM = new StreamInfo("S", new EventSchema( new Attribute("A1", DataType.INTEGER)) );
        } catch (SchemaException e) {
            throw new RuntimeException("Bad!");
        }
    }

    
    /**
     * @throws EPException 
     * @throws AlreadyRegisteredException 
     * 
     */
    @Test
    public void testQuery1() throws AlreadyRegisteredException, EPException {
        // Create resources
        TestInstance master = new TestInstance("root");
        TestInstance nodeA = new TestInstance("A", STREAM);
        TestInstance nodeB = new TestInstance("B", STREAM);
        
        // Created federated engine
        FederatedEPProvider<LocalResource> controller = new FederatedEPProvider<LocalResource>(new SimpleSplitStrategy(), master.getDesc(), new LocalResourceManager());
        controller.addResource(nodeA.getDesc());
        controller.addResource(nodeB.getDesc());
        
        
        // Create query
        EPA query = new Correlator(
                new Filter(new TimeWindow(new QualifiedStream("A", new Stream("S")), 50), new GreaterEq(new Variable("A1"), new IntConstant(50))), "A",
                new Filter(new TimeWindow(new QualifiedStream("B", new Stream("S")), 50), new GreaterEq(new Variable("A1"), new IntConstant(50))), "B",
                new Equal(new Variable("A.A1"), new Variable("B.A1")));
        
        // Deploy query
        QueryHandle qh = controller.createQuery("TestQuery", query);

        // Register sink
        CollectingSink cs = new CollectingSink();
        controller.registerSink(qh, cs );
        
        // Inject events at loca resources
        nodeA.getLocalStream("S").pushEvent(new Object[] {10}, 10L);
        nodeB.getLocalStream("S").pushEvent(new Object[] {50}, 10L);
        nodeA.getLocalStream("S").pushEvent(new Object[] {50}, 20L);
        nodeB.getLocalStream("S").pushEvent(new Object[] {74}, 25L);
        
        Assert.assertEquals(1, cs.getEvents().size());
        Event r = cs.getEvents().get(0);
        Assert.assertEquals(Integer.valueOf(50), r.get(0, Integer.class));
        Assert.assertEquals(Integer.valueOf(50), r.get(1, Integer.class));
        Assert.assertEquals(20L, r.getT1());
        Assert.assertEquals(60L, r.getT2());
        
        
        // Shutdown
        controller.destroy();
        nodeB.destroy();
        nodeA.destroy();
        master.destroy();
    }

}
