/**
 * 
 */
package de.umr.jepc.v2.federated.local;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import de.umr.ep.common.EPException;
import de.umr.jepc.v2.api.EPProvider;
import de.umr.jepc.v2.api.provider.AlreadyRegisteredException;
import de.umr.jepc.v2.api.provider.StreamHandle;
import de.umr.jepc.v2.api.schema.StreamInfo;
import de.umr.jepc.v2.bridges.nat.NativeBridge;
import de.umr.jepc.v2.federated.api.resource.ResourceInstance;
import de.umr.jepc.v2.federated.local.resource.LocalResource;
import de.umr.jepc.v2.provider.def.DefaultEPProvider;

/**
 * Sum up all parts of a processing resource for easy testing 
 * @author Michael Körber
 */
public class TestInstance {

    private final ResourceInstance instance;
    
    private final LocalResource desc;
    
    private final Map<String, StreamHandle> localStreams = new HashMap<>();
    
    public TestInstance( String id, StreamInfo... streams) throws AlreadyRegisteredException, EPException {
        EPProvider engine = new DefaultEPProvider(new NativeBridge());
        for ( StreamInfo si : streams ) {
            localStreams.put(si.getName(), engine.registerStream(si.getName(), si.getSchema()));
        }
        this.instance = new ResourceInstance(engine);
        this.desc = new LocalResource(id, Arrays.asList(streams), instance);
    }

    public ResourceInstance getInstance() {
        return instance;
    }

    public LocalResource getDesc() {
        return desc;
    }
    
    public StreamHandle getLocalStream( String name ) {
        return localStreams.get(name);
    }

    public void destroy() {
        instance.destroy();
    }
    
}
