/*
 * Copyright (c) 2012, 2013, 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.jepc.v2.bridges.nat.epa.impl.aggregator.aggregates;

import java.io.Serializable;

/**
 * Abstracts (partial) aggregates.
 * 
 * @author Michael Körber
 *
 */
public interface AggregateFunction<T_MERGE, T_EVAL, T_INPUT> extends Serializable {

  /**
   * @return the neutral value of this aggregate
   */
  T_MERGE fInit();

  /**
   * Creates a partial aggregate from the given input-value
   * 
   * @param initialValue the input-value
   * @return a partial-aggregate from the given input-value
   */
  T_MERGE fInit(T_INPUT initialValue);

  /**
   * Merges two partial aggregates
   * 
   * @param left the left partial aggregate
   * @param right the right partial aggregate
   * @return the merged partial aggregate
   */
  T_MERGE fMerge(T_MERGE left, T_MERGE right);

  /**
   * Evaluates the given partial aggregate
   * 
   * @param value the partial aggregate value
   * @param counter the number of values the given aggregate represents
   * @return the result-value for the given partial aggregate and counter
   */
  T_EVAL fEval(T_MERGE value, long counter);

}
