/*
 * Copyright (c) 2012, 2013, 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.jepc.v2.bridges.nat.epa.impl.aggregator.aggregates;

import de.umr.jepc.v2.api.util.Pair;

/**
 * Partial aggregate for standard-deviance computations.
 * 
 * @author Michael Körber
 *
 */
public class StdDevFunction implements AggregateFunction<Pair<Double, Double>, Double, Number> {

  private static final long serialVersionUID = 1L;
	
  /**
   * @{inheritDoc
   */
  @Override
  public Pair<Double, Double> fInit() {
    return new Pair<>(0.0, 0.0);
  }

  /**
   * @{inheritDoc
   */
  @Override
  public Pair<Double, Double> fInit(Number initialValue) {
    double v = initialValue.doubleValue();
    return new Pair<>(v, v * v);
  }

  /**
   * @{inheritDoc
   */
  @Override
  public Pair<Double, Double> fMerge(Pair<Double, Double> left, Pair<Double, Double> right) {
    return new Pair<>(left._1 + right._1, left._2 + right._2);
  }

  /**
   * @{inheritDoc
   */
  @Override
  public Double fEval(Pair<Double, Double> value, long counter) {
    if (counter < 2)
      return 0.0;
    return Math.sqrt(Math.max(0,
        (1.0 / (counter - 1)) * (value._2 - ((1.0 / counter) * (value._1 * value._1)))
        ));
  }
}
