/*
 * Copyright (c) 2012, 2013, 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.jepc.v2.bridges.nat.epa.impl.aggregator.aggregates.max;

import de.umr.jepc.v2.bridges.nat.epa.impl.aggregator.aggregates.AggregateFunction;

/**
 * Partial aggregate for maximum computations on double-values.
 * 
 * @author Michael Körber
 *
 */
public class DoubleMaxFunction implements AggregateFunction<Double, Double, Double> {

  private static final long serialVersionUID = 1L;

  /**
   * @{inheritDoc
   */
  @Override
  public Double fInit() {
    return Double.NEGATIVE_INFINITY;
  }

  /**
   * @{inheritDoc
   */
  @Override
  public Double fMerge(Double left, Double right) {
    return right > left ? right : left;
  }

  /**
   * @{inheritDoc
   */
  @Override
  public Double fEval(Double value, long counter) {
    return value;
  }

  /**
   * @{inheritDoc
   */
  @Override
  public Double fInit(Double initialValue) {
    return initialValue;
  }

}
