/*
 * Copyright (c) 2012, 2013, 2014
 * Database Research Group, University of Marburg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.umr.jepc.v2.bridges.nat.epa.impl.aggregator.aggregates.min;

import de.umr.jepc.v2.bridges.nat.epa.impl.aggregator.aggregates.AggregateFunction;

/**
 * Partial aggregate for minimum computations on int-values.
 * 
 * @author Michael Körber
 *
 */
public class IntMinFunction implements AggregateFunction<Integer, Integer, Integer> {

  private static final long serialVersionUID = 1L;
	
  /**
   * @{inheritDoc
   */
  @Override
  public Integer fInit() {
    return Integer.MAX_VALUE;
  }

  /**
   * @{inheritDoc
   */
  @Override
  public Integer fMerge(Integer left, Integer right) {
    return right < left ? right : left;
  }

  /**
   * @{inheritDoc
   */
  @Override
  public Integer fEval(Integer value, long counter) {
    return value;
  }

  /**
   * @{inheritDoc
   */
  @Override
  public Integer fInit(Integer initialValue) {
    return initialValue;
  }

}
