package de.umr.jepc.v2.bridges.nat.epa.impl.projection;

import de.umr.ep.common.IncompatibleTypeException;
import de.umr.ep.event.Event;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.SchemaException;
import de.umr.ep.event.impl.SimpleEvent;
import de.umr.jepc.v2.api.bridge.EPACallback;
import de.umr.jepc.v2.api.bridge.EventChannel;
import de.umr.jepc.v2.api.epa.Projection;
import de.umr.jepc.v2.api.epa.pattern.symbol.Bindings;
import de.umr.jepc.v2.bridges.nat.epa.NativeOperator;
import de.umr.jepc.v2.expressions.arithmetic.EVArithmeticExpression;
import de.umr.jepc.v2.ql.ExpressionTranslator;
import de.umr.jepc.v2.ql.TranslatorException;

public class NativeProjection implements NativeOperator {
	
	private final EventSchema schema;
	
	private final EVArithmeticExpression<?>[] expressions;
	
	private EPACallback callback;
	
	public NativeProjection(Projection def, EventChannel input) throws TranslatorException, IncompatibleTypeException, SchemaException {
		this.schema = def.computeOutputSchema(input.getSchema());
		this.expressions = new EVArithmeticExpression<?>[def.getProjections().length];
		ExpressionTranslator et = new ExpressionTranslator();
		Bindings b = new Bindings();
		for ( int i = 0; i < this.expressions.length; i++ ) {
		    switch ( schema.getAttribute(i).getType() ) {
		    case BYTE:
		    case SHORT:
		    case INTEGER:
		    case LONG:
		    case FLOAT:
		    case DOUBLE:
		        this.expressions[i] = et.translateNumericExpression(def.getProjections()[i].getExpression(), input.getSchema(), b);
		        break;
		    case STRING:
		        this.expressions[i] = et.translateStringExpression(def.getProjections()[i].getExpression(), input.getSchema(), b);
		        break;
	        default:
	            throw new IncompatibleTypeException("Don't know how to translate expression of type: " + schema.getAttribute(i).getType());
		    }
		}
	}

	@Override
	public void process(EventChannel input, Event event) {
	    Object[] payload = new Object[expressions.length];
	    for ( int i = 0; i < payload.length; i++ ) {
	        payload[i] = expressions[i].eval(event);
	    }
	    callback.receive( new SimpleEvent(payload, event.getT1(), event.getT2()) );
	}

	@Override
	public void setCallback(EPACallback callback) {
		this.callback = callback;
	}

	@Override
	public EventSchema getSchema() {
		return schema;
	}

}
