/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.evaluation.latency;

import static de.umr.jepc.v2.addons.epa.tpstream.Constraint.TemporalRelation.AABB;
import static de.umr.jepc.v2.addons.epa.tpstream.Constraint.TemporalRelation.ABAB;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Iterator;

import de.umr.ep.common.EPException;
import de.umr.jepc.v2.addons.epa.MeasuringTPStream;
import de.umr.jepc.v2.addons.epa.tpstream.Constraint;
import de.umr.jepc.v2.addons.epa.tpstream.Constraints;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinition;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinitions;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.CountingSink;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.EVUtil;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.EVEvent;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.EventGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.iseq.MeasuringSyntheticISEQ;
import de.umr.jepc.v2.api.EPProvider;
import de.umr.jepc.v2.api.epa.Stream;
import de.umr.jepc.v2.api.epa.pattern.Output;
import de.umr.jepc.v2.api.epa.pattern.symbol.Binding;
import de.umr.jepc.v2.api.epa.pattern.symbol.Bindings;
import de.umr.jepc.v2.api.expression.BooleanExpression;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.IntConstant;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.Variable;
import de.umr.jepc.v2.api.expression.predicate.Equal;
import de.umr.jepc.v2.api.provider.AlreadyRegisteredException;
import de.umr.jepc.v2.api.provider.ExternalStreamHandle;
import de.umr.jepc.v2.api.provider.QueryHandle;
import de.umr.jepc.v2.bridges.nat.NativeBridge;
import de.umr.jepc.v2.provider.def.DefaultEPProvider;

/**
 * @author Michael Körber
 *
 */

public class WallClockLatency1 {
    
    
    
    private static EventGenerator getData(final int limit) throws FileNotFoundException {
        return EVUtil.defaultGenerator(3, limit);
    }
    
    private static void testTPStream(int numEvents, long window) throws AlreadyRegisteredException, EPException, FileNotFoundException, IOException {
        EPProvider provider = new DefaultEPProvider(new NativeBridge());

        EventGenerator gen = getData(numEvents);
        
        ExternalStreamHandle sh = provider.registerExternalStream("linear", gen.getSchema());

        BooleanExpression a = new Equal(new Variable("A1"), new IntConstant(1));
        BooleanExpression b = new Equal(new Variable("A2"), new IntConstant(1));
        BooleanExpression c = new Equal(new Variable("A3"), new IntConstant(1));

        MeasuringTPStream def = new MeasuringTPStream(new Stream("linear"), window,
                new SituationDefinitions(
                        new SituationDefinition('a', a, new Bindings(new Binding("V1", new Variable("ID")))), 
                        new SituationDefinition('b', b),
                        new SituationDefinition('c', c)),
                new Constraints(
                        new Constraint('a', 'b', AABB),
                        new Constraint('b', 'c', ABAB) ),
                new Output("V1"));

        QueryHandle qh = provider.createQuery("aggressive", def);
        CountingSink cs = new CountingSink();

        provider.registerSink(qh, cs);

        long time = -System.nanoTime();
        while (gen.hasNext()) {
            sh.pushEvent(gen.next());
        }
        time += System.nanoTime();

        System.out.println("Results: " + def.getResultCount() + ", Computation time: " + (time / 1_000_000) + " ms");
        System.out.println("Input: " + def.getInputCount() + " events.");
        System.out.println("Average result latency: " + def.getAverageLatency() + " ns");
        System.out.println("Average event latency : " + def.getAverageEventLatency() + " ns");
        System.out.println("Average proc latency  : " + def.getAverageProcessingLatency() + " ns");

        provider.destroy();
    }
    
    private static void testISEQ(int numEvents, long window) throws AlreadyRegisteredException, EPException, FileNotFoundException, IOException {
        EPProvider provider = new DefaultEPProvider(new NativeBridge());

        EventGenerator gen = getData(numEvents);
        
        ExternalStreamHandle sh = provider.registerExternalStream("linear", gen.getSchema());

        BooleanExpression a = new Equal(new Variable("A1"), new IntConstant(1));
        BooleanExpression b = new Equal(new Variable("A2"), new IntConstant(1));
        BooleanExpression c = new Equal(new Variable("A3"), new IntConstant(1));

        MeasuringSyntheticISEQ def = new MeasuringSyntheticISEQ(new Stream("linear"), window,
                new SituationDefinitions(
                        new SituationDefinition('a', a, new Bindings(new Binding("V1", new Variable("ID")))), 
                        new SituationDefinition('b', b),
                        new SituationDefinition('c', c)),
                new Output("V1"));

        QueryHandle qh = provider.createQuery("aggressive", def);
        CountingSink cs = new CountingSink();

        provider.registerSink(qh, cs);

        long time = -System.nanoTime();
        while (gen.hasNext()) {
            sh.pushEvent(gen.next());
        }
        time += System.nanoTime();

        System.out.println("Results: " + def.getResultCount() + ", Computation time: " + (time / 1_000_000) + " ms");
        System.out.println("Input: " + def.getInputCount() + " events.");
        System.out.println("Average result latency: " + def.getAverageLatency() + " ns");
        System.out.println("Average event latency : " + def.getAverageEventLatency() + " ns");
        System.out.println("Average proc latency  : " + def.getAverageProcessingLatency() + " ns");
        
        provider.destroy();
    }
    
    
    private static void deserialize(int count) throws FileNotFoundException {
        Iterator<EVEvent> i = getData(count);
        long items = 0;
        long t = -System.nanoTime();
        while (i.hasNext()) {
            i.next();
            items++;
        }
        t += System.nanoTime();
        System.out.println("Results: " + items + ", deserialization time: " + (t / 1_000_000) + "ms");
    }
    
    /**
     * @param args
     * @throws JEPCException 
     * @throws AlreadyRegisteredException 
     * @throws IOException 
     * @throws FileNotFoundException 
     */
    public static void main(String[] args) throws Exception {
        if ( args.length < 3 ) {
            System.out.println("Usage: Synthetic engine count window");
            System.exit(1);
        }
        execute(args[0], Integer.valueOf(args[1]), Long.valueOf(args[2]));
    }
    
    private static void execute( String scenario, int count, long window ) throws Exception {
        System.out.println("Processing " + scenario + "/" + count + "/" + window);
        System.out.print("Performing warm-up: ");
        executeSingle(scenario, 100_000, 500);
        System.out.print("Measuring: ");
        executeSingle(scenario, count, window);
        System.out.println("=========================================");
    }
    
    private static void executeSingle( String scenario, int count, long window ) throws Exception {
        switch ( scenario.toLowerCase() ) {
            case "deserialize":
                deserialize(count); break;
            case "tps":
                testTPStream(count, window); break;
            case "iseq":
                testISEQ(count, window); break;
            default:
                throw new IllegalArgumentException("Unknown scenario: " + scenario);
        }
    }
}
