/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.evaluation.latency;

import static de.umr.jepc.v2.addons.epa.tpstream.Constraint.TemporalRelation.AABB;
import static de.umr.jepc.v2.addons.epa.tpstream.Constraint.TemporalRelation.ABAB;

import java.io.FileNotFoundException;
import java.io.IOException;

import de.umr.ep.common.EPException;
import de.umr.jepc.v2.addons.epa.MeasuringTPStream;
import de.umr.jepc.v2.addons.epa.tpstream.Constraint;
import de.umr.jepc.v2.addons.epa.tpstream.Constraints;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinition;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinitions;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.CountingSink;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.EVUtil;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.EventGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.ThrottledEVGen;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.iseq.MeasuringSyntheticISEQ;
import de.umr.jepc.v2.api.EPProvider;
import de.umr.jepc.v2.api.epa.Stream;
import de.umr.jepc.v2.api.epa.pattern.Output;
import de.umr.jepc.v2.api.epa.pattern.symbol.Binding;
import de.umr.jepc.v2.api.epa.pattern.symbol.Bindings;
import de.umr.jepc.v2.api.expression.BooleanExpression;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.IntConstant;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.Variable;
import de.umr.jepc.v2.api.expression.predicate.Equal;
import de.umr.jepc.v2.api.provider.AlreadyRegisteredException;
import de.umr.jepc.v2.api.provider.ExternalStreamHandle;
import de.umr.jepc.v2.api.provider.QueryHandle;
import de.umr.jepc.v2.bridges.nat.NativeBridge;
import de.umr.jepc.v2.provider.def.DefaultEPProvider;

/**
 * @author Michael Körber
 *
 */

public class WallClockLatency2 {
	
	private static class BreakThread extends Thread {

		private boolean exit = false;

		private long sleepTime;
		/**
		 * Creates a new SyntheticThrottledLatency.BreakThread instance
		 */
		public BreakThread(long millis) {
			this.sleepTime = millis;
		}
		
		/**
		 * @{inheritDoc}
		 */
		@Override
		public void run() {
			try {
				Thread.sleep(sleepTime);
			}
			catch (InterruptedException ex) {
			} 
			finally {
				exit = true;
			}
		}
		
		public void cancel() {
			this.interrupt();
			try {
				this.join();
			}
			catch (InterruptedException ex) {
			}
		}
		
	}
	
    
	private static EventGenerator getWarmUpData(final int limit) throws FileNotFoundException {
        return EVUtil.defaultGenerator(3, limit);
    }
    
    private static EventGenerator getData(final int limit, final long eventsPerSecond) throws FileNotFoundException {
        return new ThrottledEVGen(EVUtil.defaultGenerator(3, limit), eventsPerSecond);
    }
    
    private static void testTPStream(boolean warmUp, long breakAfter, int numEvents, long window, long eventsPerSecond) throws AlreadyRegisteredException, EPException, FileNotFoundException, IOException {
        EPProvider provider = new DefaultEPProvider(new NativeBridge());

        EventGenerator gen = warmUp ? getWarmUpData(numEvents) : getData(numEvents, eventsPerSecond);
        
        ExternalStreamHandle sh = provider.registerExternalStream("linear", gen.getSchema());

        BooleanExpression a = new Equal(new Variable("A1"), new IntConstant(1));
        BooleanExpression b = new Equal(new Variable("A2"), new IntConstant(1));
        BooleanExpression c = new Equal(new Variable("A3"), new IntConstant(1));

        MeasuringTPStream def = new MeasuringTPStream(new Stream("linear"), window,
                new SituationDefinitions(
                        new SituationDefinition('a', a, new Bindings(new Binding("V1", new Variable("ID")))), 
                        new SituationDefinition('b', b),
                        new SituationDefinition('c', c)),
                new Constraints(
                        new Constraint('a', 'b', AABB),
                        new Constraint('b', 'c', ABAB) ),
                new Output("V1"));

        QueryHandle qh = provider.createQuery("aggressive", def);
        CountingSink cs = new CountingSink();

        provider.registerSink(qh, cs);

        BreakThread bt = new BreakThread(breakAfter);
        
        bt.start();
        long time = -System.nanoTime();
        while (gen.hasNext() && !bt.exit) {
            sh.pushEvent(gen.next());
        }
        time += System.nanoTime();

        System.out.println("Results: " + def.getResultCount() + ", Computation time: " + (time / 1_000_000) + " ms");
        System.out.println("Input: " + def.getInputCount() + " events.");
        System.out.println("Average result latency: " + def.getAverageLatency() + " ns");
        System.out.println("Average event latency : " + def.getAverageEventLatency() + " ns");
        System.out.println("Average proc latency  : " + def.getAverageProcessingLatency() + " ns");

        bt.cancel();
        
        provider.destroy();
    }
    
    private static void testISEQ(boolean warmUp, long breakAfter, int numEvents, long window, long eventsPerSecond) throws AlreadyRegisteredException, EPException, FileNotFoundException, IOException {
        EPProvider provider = new DefaultEPProvider(new NativeBridge());

        EventGenerator gen = warmUp ? getWarmUpData(numEvents) : getData(numEvents, eventsPerSecond);
        
        ExternalStreamHandle sh = provider.registerExternalStream("linear", gen.getSchema());

        BooleanExpression a = new Equal(new Variable("A1"), new IntConstant(1));
        BooleanExpression b = new Equal(new Variable("A2"), new IntConstant(1));
        BooleanExpression c = new Equal(new Variable("A3"), new IntConstant(1));

        MeasuringSyntheticISEQ def = new MeasuringSyntheticISEQ(new Stream("linear"), window,
                new SituationDefinitions(
                        new SituationDefinition('a', a, new Bindings(new Binding("V1", new Variable("ID")))), 
                        new SituationDefinition('b', b),
                        new SituationDefinition('c', c)),
                new Output("V1"));

        QueryHandle qh = provider.createQuery("aggressive", def);
        CountingSink cs = new CountingSink();

        provider.registerSink(qh, cs);
        
        BreakThread bt = new BreakThread(breakAfter);
        
        bt.start();

        long time = -System.nanoTime();
        while (gen.hasNext() && !bt.exit) {
            sh.pushEvent(gen.next());
        }
        time += System.nanoTime();
        
        System.out.println("Results: " + def.getResultCount() + ", Computation time: " + (time / 1_000_000) + " ms");
        System.out.println("Input: " + def.getInputCount() + " events.");
        System.out.println("Average result latency: " + def.getAverageLatency() + " ns");
        System.out.println("Average event latency : " + def.getAverageEventLatency() + " ns");
        System.out.println("Average proc latency  : " + def.getAverageProcessingLatency() + " ns");
        
        bt.cancel();
        
        provider.destroy();
    }
    
    
    /**
     * @param args
     * @throws JEPCException 
     * @throws AlreadyRegisteredException 
     * @throws IOException 
     * @throws FileNotFoundException 
     */
    public static void main(String[] args) throws Exception {
        if ( args.length < 3 ) {
            System.out.println("Usage: Synthetic engine count window");
            System.exit(1);
        }
        execute(args[0], Integer.valueOf(args[1]), Long.valueOf(args[2]), Long.valueOf(args[3]));
    }
    
    private static void execute( String scenario, int count, long window, long eventsPerSecond ) throws Exception {
        System.out.println("Processing " + scenario + "/" + count + "/" + window + ", rate: " + eventsPerSecond);
        System.out.print("Performing warm-up: ");
        executeSingle(true, scenario, 100_000, 500, 10_000_000);
        System.out.print("Measuring: ");
        executeSingle(false, scenario, count, window, eventsPerSecond);
        System.out.println("=========================================");
    }
    
    private static void executeSingle( boolean warmUp, String scenario, int count, long window, long eventsPerSecond ) throws Exception {
        switch ( scenario.toLowerCase() ) {
            case "tps":
                testTPStream(warmUp, 1_800_000L, count, window,eventsPerSecond); break;
            case "iseq":
                testISEQ(warmUp, 1_800_000L, count, window, eventsPerSecond); break;
            default:
                throw new IllegalArgumentException("Unknown scenario: " + scenario);
        }
    }
}
