package de.umr.jepc.v2.addons.epa.tpstream.evaluation.optimizer;

import java.util.Arrays;
import java.util.List;

import de.umr.ep.event.SchemaException;
import de.umr.jepc.v2.addons.epa.tpstream.Constraint;
import de.umr.jepc.v2.addons.epa.tpstream.Constraint.TemporalRelation;
import de.umr.jepc.v2.addons.epa.tpstream.Constraints;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinition;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinitions;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.CountingSink;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.EVUtil;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.FixedOrderMatcher;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.generator.BaseEventGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.generator.RepeatableSituationGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.generator.WrappingSituationGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.situations.SituationDef;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.situations.duration.UniformDurationGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.AdaptiveFixedDFSMatcher;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.Matcher;
import de.umr.jepc.v2.addons.epa.tpstream.optimizer.Optimizer;
import de.umr.jepc.v2.api.expression.logical.True;

public class AdaptivityExperiment {
	
	private static final int NUM_RUNS = 100;

	private static final long window = 10000;
	
	private static final int NUM_EVENTS = 20_000_000;

	private static void run( Matcher matcher, int run, 
	        long[] tuni, long[] tskew1, long[] tskew2,
	        RepeatableSituationGenerator uni, RepeatableSituationGenerator skew1, RepeatableSituationGenerator skew2 ) {
		
		tuni[run] = -System.currentTimeMillis();
		EVUtil.feedMatcher(uni, matcher, window);
		tuni[run] += System.currentTimeMillis();
		
		tskew1[run] = -System.currentTimeMillis();
		EVUtil.feedMatcher(skew1, matcher, window);
		tskew1[run] += System.currentTimeMillis();
		
		tskew2[run] = -System.currentTimeMillis();
		EVUtil.feedMatcher(skew2, matcher, window);
		tskew2[run] += System.currentTimeMillis();
		
		uni.reset();
        skew1.reset();
        skew2.reset();
	}
	
	private static void runPerfect(SituationDefinitions ss, Constraints cs, List<Character> order1, List<Character> order2, List<Character> order3,
	        RepeatableSituationGenerator uni, RepeatableSituationGenerator skew1, RepeatableSituationGenerator skew2) {
		
		long[] tuni  = new long[NUM_RUNS+1];
        long[] tskew1 = new long[NUM_RUNS+1];
        long[] tskew2 = new long[NUM_RUNS+1];

        for ( int i = 0; i < tuni.length; i++ ) {
            FixedOrderMatcher matcher = new FixedOrderMatcher(ss, cs, window, new CountingSink(), order1);
        
    		tuni[i] = -System.currentTimeMillis();
    		EVUtil.feedMatcher(uni, matcher, window);
    		tuni[i] += System.currentTimeMillis();
    		
    		matcher.setOrder(order2);
    		
    		tskew1[i] = -System.currentTimeMillis();
    		EVUtil.feedMatcher(skew1, matcher, window);
    		tskew1[i] += System.currentTimeMillis();
    		
    		matcher.setOrder(order3);
    		
    		tskew2[i] = -System.currentTimeMillis();
    		EVUtil.feedMatcher(skew2, matcher, window);
    		tskew2[i] += System.currentTimeMillis();
    		
    		uni.reset();
            skew1.reset();
            skew2.reset();
        }
        outputResults("Perfect", tuni, tskew1, tskew2);
	}
	
	
	private static void runStatic(SituationDefinitions ss, Constraints cs, List<Character> order,
	        RepeatableSituationGenerator uni, RepeatableSituationGenerator skew1, RepeatableSituationGenerator skew2) {
		long[] timingsuni  = new long[NUM_RUNS+1];
        long[] timingskew1 = new long[NUM_RUNS+1];
        long[] timingskew2 = new long[NUM_RUNS+1];
        
        
        for ( int i = 0; i < timingsuni.length; i++ ) {
            FixedOrderMatcher fom = new FixedOrderMatcher(ss, cs, window, new CountingSink(), order);
            run(fom, i,
                    timingsuni, timingskew1, timingskew2,
                    uni, skew1, skew2 );
        }
        
        outputResults("Static " + order, timingsuni, timingskew1, timingskew2);
	}
	
	private static void runAdaptive(SituationDefinitions ss, Constraints cs,
	        RepeatableSituationGenerator uni, RepeatableSituationGenerator skew1, RepeatableSituationGenerator skew2) {
	    long[] timingsuni  = new long[NUM_RUNS+1];
	    long[] timingskew1 = new long[NUM_RUNS+1];
	    long[] timingskew2 = new long[NUM_RUNS+1];
	    
	    Optimizer opt = new Optimizer(cs);
	    
	    for ( int i = 0; i < timingsuni.length; i++ ) {
    		AdaptiveFixedDFSMatcher aom = new AdaptiveFixedDFSMatcher(ss, cs, window, opt, new CountingSink(), 0.2);
    		run(aom, i,
    		        timingsuni, timingskew1, timingskew2,
    		        uni, skew1, skew2 );
	    }
		outputResults("Adaptive", timingsuni, timingskew1, timingskew2);
	}
	
	private static void outputResults(String name, long[] timingsuni, long[] timingskew1, long[] timingskew2) {
	    long auni = 0;
        long ask1 = 0;
        long ask2 = 0;
        for ( int i = 1; i < timingsuni.length; i++ ) {
            auni += timingsuni[i];
            ask1 += timingskew1[i];
            ask2 += timingskew2[i];
        }
        
        auni /= (timingsuni.length-1);
        ask1 /= (timingsuni.length-1);
        ask2 /= (timingsuni.length-1);
        
        System.out.println(name + ": AVG[" + auni + "," + ask1 + "," + ask2 + "]=" + (auni+ask1+ask2) + ", throughput: [" + 
                EVUtil.tp(NUM_EVENTS, auni) + "," +
                EVUtil.tp(NUM_EVENTS, ask1) + "," +
                EVUtil.tp(NUM_EVENTS, ask2) + ",]= " + EVUtil.tp(3*NUM_EVENTS, auni + ask1 + ask2) + " events /s, details: " +
                Arrays.toString(timingsuni) + "," + Arrays.toString(timingskew1) + "," + Arrays.toString(timingskew2));
	}

	public static void main(String... args) throws SchemaException {
		System.out.println("****************************************");
		System.out.println("         Experiment: Adaptivity");

		BaseEventGenerator uniform = new BaseEventGenerator("Test", NUM_EVENTS, -1, 1,
				new SituationDef(new UniformDurationGenerator(50, 100), new UniformDurationGenerator(1, 100)),
				new SituationDef(new UniformDurationGenerator(50, 100), new UniformDurationGenerator(1, 100)),
				new SituationDef(new UniformDurationGenerator(50, 100), new UniformDurationGenerator(1, 100)));

		BaseEventGenerator skewed1 = new BaseEventGenerator("Test", NUM_EVENTS, NUM_EVENTS, 1,
				new SituationDef(new UniformDurationGenerator(50, 100), new UniformDurationGenerator(1, 5_000)),
				new SituationDef(new UniformDurationGenerator(50, 100), new UniformDurationGenerator(1, 100)),
				new SituationDef(new UniformDurationGenerator(50, 100), new UniformDurationGenerator(1, 100)));

		BaseEventGenerator skewed2 = new BaseEventGenerator("Test", NUM_EVENTS, 2*NUM_EVENTS+1, 1,
				new SituationDef(new UniformDurationGenerator(50, 100), new UniformDurationGenerator(1, 100)),
				new SituationDef(new UniformDurationGenerator(50, 100), new UniformDurationGenerator(1, 5_000)),
				new SituationDef(new UniformDurationGenerator(50, 100), new UniformDurationGenerator(1, 100)));

		RepeatableSituationGenerator ru = new RepeatableSituationGenerator(
				new WrappingSituationGenerator("TestUniform", uniform, "A1", "A2", "A3"));
		RepeatableSituationGenerator rs1 = new RepeatableSituationGenerator(
				new WrappingSituationGenerator("TestSkewed1", skewed1, "A1", "A2", "A3"));
		RepeatableSituationGenerator rs2 = new RepeatableSituationGenerator(
				new WrappingSituationGenerator("TestSkewed2", skewed2, "A1", "A2", "A3"));
		
		SituationDefinitions ss = new SituationDefinitions(new SituationDefinition('a', new True()), new SituationDefinition('b', new True()), new SituationDefinition('c', new True()));

		Constraints cs = new Constraints(new Constraint('a', 'b', TemporalRelation.AABB),
				new Constraint('a', 'c', TemporalRelation.AABB), new Constraint('b', 'c', TemporalRelation.AABB));

		runStatic (ss, cs, Arrays.asList( 'c','a','b' ) , ru, rs1, rs2);
		runStatic (ss, cs, Arrays.asList( 'c','b','a' ) , ru, rs1, rs2);
		runPerfect(ss, cs, Arrays.asList( 'c','b','a' ), Arrays.asList( 'c','a','b' ), Arrays.asList( 'c','b','a' ), ru, rs1, rs2);
		runAdaptive(ss, cs, ru, rs1, rs2);
		System.out.println("****************************************");
	}

}
