package de.umr.jepc.v2.addons.epa.tpstream.evaluation.optimizer;

import java.util.Arrays;
import java.util.List;

import de.umr.ep.event.SchemaException;
import de.umr.jepc.v2.addons.epa.tpstream.Constraint;
import de.umr.jepc.v2.addons.epa.tpstream.Constraint.TemporalRelation;
import de.umr.jepc.v2.addons.epa.tpstream.Constraints;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinition;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinitions;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.CountingSink;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.EVUtil;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.FixedOrderMatcher;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.EventGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.SituationGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.generator.WrappingSituationGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.optimizer.Optimizer;
import de.umr.jepc.v2.addons.epa.tpstream.optimizer.constraints.ConstraintGraph;
import de.umr.jepc.v2.addons.epa.tpstream.optimizer.stats.TPStats;
import de.umr.jepc.v2.api.expression.logical.True;

public class InitalOrderExperiment {

	private static final int NUM_RUNS = 100;
	
    private static final long window = 5000;
	
	
    private static void processOrder( SituationDefinitions ss, Constraints cs, List<Character> order ) throws SchemaException {
        long[] timings = new long[NUM_RUNS+1];
        long[] rates   = new long[NUM_RUNS+1];
        
        for ( int i = 0; i < timings.length; i++ ) {
            EventGenerator egen = EVUtil.defaultGenerator(ss.getNumberOfDefinitions());
            SituationGenerator sgen = new WrappingSituationGenerator(egen.getName(), egen, "A1", "A2", "A3");
            
            long t = -System.currentTimeMillis();
            FixedOrderMatcher fom = new FixedOrderMatcher(ss, cs, window, new CountingSink(), order);
            EVUtil.feedMatcher(sgen, fom, window);
            t += System.currentTimeMillis();
            timings[i] = t;
            rates[i] = EVUtil.tp(t);
        }
        
        double avg = 0;
        long avgr = 0;
        for ( int i = 1; i < timings.length; i++ ) {
            avg += timings[i];
            avgr += rates[i];
        }
        avg /= (timings.length-1);
        avgr /= (rates.length-1);
        
        System.out.println(order + ": Avg(" + avg + "ms, " + avgr + " events/s), details: " + Arrays.toString(timings));
    }
    
	private static void execute( SituationDefinitions ss, Constraints cs ) throws SchemaException {
		Optimizer opt = new Optimizer(cs);
		ConstraintGraph cg = new ConstraintGraph(cs);
		
		System.out.println("Evaluating orders for: ");
		System.out.println("  Symbols:     " + ss);
		System.out.println("  Constraints: " + cs);
		System.out.println("  Window:      " + window);
		
		List<Character> suggest = opt.calculateOrder('c', new TPStats(ss, cs));

		System.out.println("  Suggested:   " + suggest);
		
		for ( List<Character> order : cg.getValidEvaluationOrders() ) {
			processOrder(ss, cs, order);
		}
	}
	
	private static void exp1() throws SchemaException {
	    SituationDefinitions ss = new SituationDefinitions(
		        new SituationDefinition('a', new True()),
		        new SituationDefinition('b', new True()),
		        new SituationDefinition('c', new True()));
		
		Constraints cs = new Constraints( 
		        new Constraint('a', 'b', TemporalRelation.ABAB), 
		        new Constraint('a', 'c', TemporalRelation.ABAB),
		        new Constraint('b', 'c', TemporalRelation.A_BAB) );
		
		execute(ss, cs);
	}
	
	private static void exp2() throws SchemaException {
	    SituationDefinitions ss = new SituationDefinitions(
                new SituationDefinition('a', new True()),
                new SituationDefinition('b', new True()),
                new SituationDefinition('c', new True()));
        
        Constraints cs = new Constraints( 
                new Constraint('a', 'b', TemporalRelation.ABAB), 
                new Constraint('a', 'c', TemporalRelation.AABB),
                new Constraint('b', 'c', TemporalRelation.ABAB) );
        
        execute(ss, cs);
    }
	
	private static void exp3() throws SchemaException {
	    SituationDefinitions ss = new SituationDefinitions(
                new SituationDefinition('a', new True()),
                new SituationDefinition('b', new True()),
                new SituationDefinition('c', new True()));
        
        Constraints cs = new Constraints( 
                new Constraint('a', 'b', TemporalRelation.AABB), 
                new Constraint('a', 'c', TemporalRelation.AABB),
                new Constraint('b', 'c', TemporalRelation.AABB) );
        
        execute(ss, cs);
    }
	
	
	public static void main(String...args) throws SchemaException {
	    System.out.println("****************************************");
        System.out.println("         Experiment: Initial order");
		exp1();
		System.out.println("============================");
		exp2();
        System.out.println("============================");
        exp3();
        System.out.println("****************************************");
	}

}
