/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.evaluation.processingTime;

import static de.umr.jepc.v2.addons.epa.tpstream.Constraint.TemporalRelation.AABB;
import static de.umr.jepc.v2.addons.epa.tpstream.Constraint.TemporalRelation.AA_BB;
import static de.umr.jepc.v2.addons.epa.tpstream.Constraint.TemporalRelation.ABAB;
import static de.umr.jepc.v2.addons.epa.tpstream.Constraint.TemporalRelation.ABA_B;
import static de.umr.jepc.v2.addons.epa.tpstream.Constraint.TemporalRelation.ABBA;
import static de.umr.jepc.v2.addons.epa.tpstream.Constraint.TemporalRelation.A_BAB;
import static de.umr.jepc.v2.addons.epa.tpstream.Constraint.TemporalRelation.BAAB;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Iterator;

import de.umr.ep.common.Attribute;
import de.umr.ep.common.Attribute.DataType;
import de.umr.ep.common.EPException;
import de.umr.ep.event.Event;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.SchemaException;
import de.umr.ep.util.io.BinaryEventReader;
import de.umr.jepc.v2.addons.epa.TPStream;
import de.umr.jepc.v2.addons.epa.TPStream.Mode;
import de.umr.jepc.v2.addons.epa.tpstream.Constraint;
import de.umr.jepc.v2.addons.epa.tpstream.Constraints;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinition;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinitions;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.CountingSink;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.adapter.EsperEventAdapter;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.adapter.SaseEventAdapter;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.adapter.EsperEventAdapter.SituationQuery;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.iseq.AggressiveISEQ;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.iseq.AggressiveISEQLight;
import de.umr.jepc.v2.api.EPProvider;
import de.umr.jepc.v2.api.epa.Stream;
import de.umr.jepc.v2.api.epa.pattern.Output;
import de.umr.jepc.v2.api.epa.pattern.Partition;
import de.umr.jepc.v2.api.epa.pattern.symbol.Binding;
import de.umr.jepc.v2.api.epa.pattern.symbol.Binding.BindingTime;
import de.umr.jepc.v2.api.epa.pattern.symbol.Bindings;
import de.umr.jepc.v2.api.expression.BooleanExpression;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.DoubleConstant;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.IntConstant;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.Variable;
import de.umr.jepc.v2.api.expression.predicate.GreaterEq;
import de.umr.jepc.v2.api.expression.predicate.LessEq;
import de.umr.jepc.v2.api.provider.AlreadyRegisteredException;
import de.umr.jepc.v2.api.provider.ExternalStreamHandle;
import de.umr.jepc.v2.api.provider.QueryHandle;
import de.umr.jepc.v2.bridges.nat.NativeBridge;
import de.umr.jepc.v2.provider.def.DefaultEPProvider;

/**
 * @author Michael Körber
 *
 */
/*
 Cumulated: 
  Speed (avg): 31.64006414
  Speed (min): 0.0
  Speed (max): 75.0
  Accel (max): 0.015
  Accel (min): -0.065
  
  Reading time: 530224ms
 */

public class AggressiveDrivers {
	
	public static final EventSchema SCHEMA;
	
	static {
        try {
            SCHEMA = new EventSchema(
                // Vehicle ID
                new Attribute("VID", DataType.INTEGER),
                // Speed (0,100) mph
                new Attribute("SPEED", DataType.INTEGER),
                // Velocity (mp/h^2)
                new Attribute("ACCEL", DataType.DOUBLE),
                // Expressway number
                new Attribute("XWay", DataType.INTEGER),
                // Land [0,4], 0 enter, 4 exit
                new Attribute("Lane", DataType.BYTE),
                // {0 = Eastbound,1 = Westbound }
                new Attribute("Dir", DataType.BYTE),
                // [0,99] Road segment
                new Attribute("Seg", DataType.BYTE),
                // Horizontal position in ft from origin [0,527999]
                new Attribute("Pos", DataType.INTEGER)
            );
        } catch (SchemaException e) {
            throw new IllegalArgumentException("Invalid Schema",e);
        }
    }
    
    private static Iterator<Event> getData(String file, final long limit) throws IOException {
    	Path dataIn = Paths.get(file);
        BinaryEventReader ber = new BinaryEventReader(SCHEMA, dataIn);
        
        return new Iterator<Event>() {
            long counter = 0;

            @Override
            public boolean hasNext() {
                if ( !(ber.hasNext() && counter < limit) ) {
                    try {
                        ber.close();
                    } catch (IOException e) {
                    }
                    return false;
                }
                return true;   
            }

            @Override
            public Event next() {
                counter++;
                return ber.next();
            }
        };
        
    }
    
    private static void testTPStream(Mode mode, Iterator<Event> events) throws AlreadyRegisteredException, EPException, FileNotFoundException, IOException {
        EPProvider provider = new DefaultEPProvider(new NativeBridge());

        ExternalStreamHandle sh = provider.registerExternalStream("linear", SCHEMA);

        BooleanExpression speeding = new GreaterEq(new Variable("SPEED"), new IntConstant(63));
        BooleanExpression accel = new GreaterEq(new Variable("ACCEL"), new DoubleConstant(0.0050));
        BooleanExpression decel = new LessEq(new Variable("ACCEL"), new DoubleConstant(-0.00455));

        TPStream def = new TPStream(new Stream("linear"), mode, 300_000,
                new SituationDefinitions(
                        new SituationDefinition('a', accel), 
                        new SituationDefinition('b', speeding, new Bindings(new Binding("CAR_ID", new Variable("VID"), BindingTime.INITIAL))),
                        new SituationDefinition('c', decel)),
                new Constraints(
                        new Constraint('a', 'b', AA_BB, ABAB, A_BAB, BAAB),
                        new Constraint('b', 'c', ABBA, ABA_B, ABAB, AA_BB),
                        new Constraint('a', 'c', AABB, AA_BB)),
                new Output("CAR_ID"), new Partition("VID"));

        QueryHandle qh = provider.createQuery("aggressive", def);
        CountingSink cs = new CountingSink();

        provider.registerSink(qh, cs);

        long time = -System.nanoTime();
        while (events.hasNext()) {
            sh.pushEvent(events.next());
        }
        time += System.nanoTime();

        System.out.println("Results: " + cs.count + ", Computation time: " + (time / 1_000_000) + "ms");

        provider.destroy();
    }
    
    private static void testTPStreamSimple(Mode mode, Iterator<Event> events) throws AlreadyRegisteredException, EPException, FileNotFoundException, IOException {
        EPProvider provider = new DefaultEPProvider(new NativeBridge());

        ExternalStreamHandle sh = provider.registerExternalStream("linear", SCHEMA);

        BooleanExpression speeding = new GreaterEq(new Variable("SPEED"), new IntConstant(63));
        BooleanExpression accel = new GreaterEq(new Variable("ACCEL"), new DoubleConstant(0.0050));
        BooleanExpression decel = new LessEq(new Variable("ACCEL"), new DoubleConstant(-0.00455));

        TPStream def = new TPStream(new Stream("linear"), mode, 300_000,
                new SituationDefinitions(
                        new SituationDefinition('a', accel), 
                        new SituationDefinition('b', speeding, new Bindings(new Binding("CAR_ID", new Variable("VID"), BindingTime.INITIAL))),
                        new SituationDefinition('c', decel)),
                new Constraints(
                        new Constraint('a', 'b', AA_BB, ABAB),
                        new Constraint('b', 'c', ABAB, AA_BB)),
                new Output("CAR_ID"), new Partition("VID"));

        QueryHandle qh = provider.createQuery("aggressive", def);
        CountingSink cs = new CountingSink();
        provider.registerSink(qh, cs);

        long time = -System.nanoTime();
        while (events.hasNext()) {
            sh.pushEvent(events.next());
        }
        time += System.nanoTime();

        System.out.println("Results: " + cs.count + ", Computation time: " + (time / 1_000_000) + "ms");

        provider.destroy();
    }
    
    
    private static void testISEQ(Iterator<Event> events) throws AlreadyRegisteredException, EPException, FileNotFoundException, IOException {
        EPProvider provider = new DefaultEPProvider(new NativeBridge());

        ExternalStreamHandle sh = provider.registerExternalStream("linear", SCHEMA);

        BooleanExpression speeding = new GreaterEq(new Variable("SPEED"), new IntConstant(63));
        BooleanExpression accel = new GreaterEq(new Variable("ACCEL"), new DoubleConstant(0.0050));
        BooleanExpression decel = new LessEq(new Variable("ACCEL"), new DoubleConstant(-0.00455));

        AggressiveISEQ def = new AggressiveISEQ(new Stream("linear"), 300_000,
                new SituationDefinitions(
                        new SituationDefinition('a', accel), 
                        new SituationDefinition('b', speeding, new Bindings(new Binding("CAR_ID", new Variable("VID"), BindingTime.INITIAL))),
                        new SituationDefinition('c', decel)),
                new Output("CAR_ID"), new Partition("VID"));

        QueryHandle qh = provider.createQuery("aggressive", def);
        CountingSink cs = new CountingSink();
        provider.registerSink(qh, cs);

        long time = -System.nanoTime();
        while (events.hasNext()) {
            sh.pushEvent(events.next());
        }
        time += System.nanoTime();

        System.out.println("Results: " + cs.count + ", Computation time: " + (time / 1_000_000) + "ms");

        provider.destroy();
    }
    
    private static void testISEQSimple(Iterator<Event> events) throws AlreadyRegisteredException, EPException, FileNotFoundException, IOException {
        EPProvider provider = new DefaultEPProvider(new NativeBridge());

        ExternalStreamHandle sh = provider.registerExternalStream("linear", SCHEMA);

        BooleanExpression speeding = new GreaterEq(new Variable("SPEED"), new IntConstant(63));
        BooleanExpression accel = new GreaterEq(new Variable("ACCEL"), new DoubleConstant(0.0050));
        BooleanExpression decel = new LessEq(new Variable("ACCEL"), new DoubleConstant(-0.00455));

        AggressiveISEQLight def = new AggressiveISEQLight(new Stream("linear"), 300_000,
                new SituationDefinitions(
                        new SituationDefinition('a', accel), 
                        new SituationDefinition('b', speeding, new Bindings(new Binding("CAR_ID", new Variable("VID"), BindingTime.INITIAL))),
                        new SituationDefinition('c', decel)),
                new Output("CAR_ID"), new Partition("VID"));

        QueryHandle qh = provider.createQuery("aggressive", def);
        CountingSink cs = new CountingSink();
        provider.registerSink(qh, cs);

        long time = -System.nanoTime();
        while (events.hasNext()) {
            sh.pushEvent(events.next());
        }
        time += System.nanoTime();

        System.out.println("Results: " + cs.count + ", Computation time: " + (time / 1_000_000) + "ms");

        provider.destroy();
    }
    
    private static void testSaseLL(Iterator<Event> events) throws IOException {
        String query = "PATTERN SEQ(CARS a, CARS+ b, CARS c)\n" + 
                "WHERE partition-contiguity\n" +  
                "AND [VID]\n" +
                "AND a.SPEED < 63 \n" +
                "AND a.ACCEL >= 500 \n" +
                "AND b[0].SPEED > 62 \n" +
                "AND b[i].SPEED > 62 \n" +
                "AND c.SPEED < 63 \n" +
                "AND c.ACCEL <= -455 \n" +
                "WITHIN 300000";
        SaseEventAdapter sea = new SaseEventAdapter("CARS", SCHEMA, events, query);
        long t = -System.nanoTime();
        sea.run();
        t += System.nanoTime();
        System.out.println("Results: " + sea.getNumberOfResults() + ", Computation time: " + (t / 1_000_000) + "ms");
        sea.destroy();
    }
    
    private static void testEsper(Iterator<Event> events) throws SchemaException, IOException {
        String query1 = "insert into SPEEDING " +  
                    "select * from CARS.win:time(5 minutes)" + 
                    "match_recognize ( " + 
                    "partition by VID " + 
                    "measures FIRST(A.VID) as VID, FIRST(A.TSTART) as TSTART, B.TSTART as TEND " + 
                    "pattern (A+ B)" + 
                    "define " +  
                    "A as A.SPEED >= 63, " + 
                    "B as B.SPEED < 63)";
        
        String query2 = "insert into ACCEL " +  
                "select * from CARS.win:time(5 minutes)" + 
                "match_recognize ( " + 
                "partition by VID " + 
                "measures FIRST(A.VID) as VID, FIRST(A.TSTART) as TSTART, B.TSTART as TEND " + 
                "pattern (A+ B)" + 
                "define " +  
                "A as A.ACCEL >= 0.0050, " + 
                "B as B.ACCEL < 0.0050)";
        
        String query3 = "insert into DECEL " +  
                "select * from CARS.win:time(5 minutes)" + 
                "match_recognize ( " + 
                "partition by VID " + 
                "measures FIRST(A.VID) as VID, FIRST(A.TSTART) as TSTART, B.TSTART as TEND " + 
                "pattern (A+ B)" + 
                "define " +  
                "A as A.ACCEL <= -0.00455, " + 
                "B as B.ACCEL > -0.00455)";
        
        
        EventSchema sout = new EventSchema(new Attribute("VID", DataType.INTEGER));
        
        String fQuery = "select a.VID as VID, a.TSTART as TSA, a.TEND as TEA, b.TSTART as TSB, b.TEND as TEB, c.TSTART AS TSC, c.TEND as TEC, c.TEND as TSTART, c.TEND+1 as TEND from pattern[ " +
        "every a=ACCEL ->" +
        "((b=SPEEDING(VID=a.VID, TSTART <= a.TEND, TEND >= a.TEND) " +
        " -> c=DECEL(VID=a.VID, TSTART >= a.TEND, TSTART <= b.TEND, TEND - a.TSTART < 300000)) OR " +
        "(every c=DECEL(VID=a.VID, TSTART >= a.TEND)" +
        " -> b=SPEEDING(VID=a.VID, TSTART <= a.TEND, TEND > a.TEND, TEND > c.TEND, TEND - a.TSTART < 300000))) " +
        "where timer:within(5 minutes)]";
        
        EsperEventAdapter eea = new EsperEventAdapter("CARS", SCHEMA, events, fQuery,
                new SituationQuery("SPEEDING",sout,query1),
                new SituationQuery("ACCEL",sout,query2),
                new SituationQuery("DECEL",sout,query3)
        );
        
        
        long t = -System.nanoTime();
        eea.run();
        t += System.nanoTime();
        System.out.println("Results: " + eea.getNumberOfResults() + ", Computation time: " + (t / 1_000_000) + "ms");
        eea.destroy();
    }
    
    private static void testEsperSimple(Iterator<Event> events) throws SchemaException, IOException {
        String query1 = "insert into SPEEDING " +  
                    "select * from CARS.win:time(5 minutes)" + 
                    "match_recognize ( " + 
                    "partition by VID " + 
                    "measures FIRST(A.VID) as VID, FIRST(A.TSTART) as TSTART, B.TSTART as TEND " + 
                    "pattern (A+ B)" + 
                    "define " +  
                    "A as A.SPEED >= 63, " + 
                    "B as B.SPEED < 63)";
        
        String query2 = "insert into ACCEL " +  
                "select * from CARS.win:time(5 minutes)" + 
                "match_recognize ( " + 
                "partition by VID " + 
                "measures FIRST(A.VID) as VID, FIRST(A.TSTART) as TSTART, B.TSTART as TEND " + 
                "pattern (A+ B)" + 
                "define " +  
                "A as A.ACCEL >= 0.0050, " + 
                "B as B.ACCEL < 0.0050)";
        
        String query3 = "insert into DECEL " +  
                "select * from CARS.win:time(5 minutes)" + 
                "match_recognize ( " + 
                "partition by VID " + 
                "measures FIRST(A.VID) as VID, FIRST(A.TSTART) as TSTART, B.TSTART as TEND " + 
                "pattern (A+ B)" + 
                "define " +  
                "A as A.ACCEL <= -0.00455, " + 
                "B as B.ACCEL > -0.00455)";
        
        
        EventSchema sout = new EventSchema(new Attribute("VID", DataType.INTEGER));
        
        String fQuery = "select a.VID as VID, a.TSTART as TSA, a.TEND as TEA, b.TSTART as TSB, b.TEND as TEB, c.TSTART AS TSC, c.TEND as TEC, c.TEND as TSTART, c.TEND+1 as TEND from pattern[ " +
        "every a=ACCEL ->" +
        "(b=SPEEDING(VID=a.VID, TSTART > a.TSTART, TSTART <= a.TEND, TEND > a.TEND) " +
        " -> c=DECEL(VID=a.VID, TSTART >= a.TEND, TSTART <= b.TEND, TEND > b.TEND, TEND - a.TSTART < 300000)) " +
        "where timer:within(5 minutes)]";
        
        EsperEventAdapter eea = new EsperEventAdapter("CARS", SCHEMA, events, fQuery,
                new SituationQuery("SPEEDING",sout,query1),
                new SituationQuery("ACCEL",sout,query2),
                new SituationQuery("DECEL",sout,query3)
        );
        
        
        long t = -System.nanoTime();
        eea.run();
        t += System.nanoTime();
        System.out.println("Results: " + eea.getNumberOfResults() + ", Computation time: " + (t / 1_000_000) + "ms");
        eea.destroy();
    }
    
    private static void testEsperLL(Iterator<Event> events) throws SchemaException, IOException {
        String pattern = "(" +
                         "(A (S | SA | SD)+ D) | " +
                         "(N SA (S | SA | SD)+ D) | " +
                         "(S SA (S | SA | SD)+ D) | " +
                         
                         "(A (S | SA | SD)+ SD N) | " +
                         "(N SA (S | SA | SD)+ SD N) | " +
                         "((S | SD) SA (S | SA | SD)+ SD N) | " +
                         
                        "(A (S | SA | SD)+ SD S) | " +
                        "(N SA (S | SA | SD)+ SD S) | " +
                        "((S | SD) SA (S | SA | SD)+ SD S) " +
                         ") ";
        
        String query = "select * from CARS.win:time(10 minutes) " + 
                    "match_recognize ( " + 
                    "partition by VID " + 
                    "measures coalesce(D.VID, A.VID , FIRST(N.VID) , FIRST(S.VID)) as VID, current_timestamp() as TIMESTAMP " +
                    "all matches " +
                    "after match skip to next row " +
                    "pattern " + pattern +
                    "define " +  
                    "A as A.ACCEL >= 0.005 AND A.SPEED < 63, " +
                    "D as D.ACCEL <= -0.0045 AND D.SPEED < 63, " +
                    
                    "S AS S.SPEED >= 63 AND S.ACCEL > -0.0045 AND S.ACCEL < 0.005, " +
                    "SA AS SA.SPEED >= 63 AND SA.ACCEL >= 0.005, " +
                    "SD AS SD.SPEED >= 63 AND SD.ACCEL <= -0.0045, " +
                    
                    "N as N.SPEED < 63 AND N.ACCEL > -0.0045 AND N.ACCEL < 0.005" +
                     ")";
        
        EsperEventAdapter eea = new EsperEventAdapter("CARS", SCHEMA, events, query);
        long t = -System.nanoTime();
        eea.run();
        t += System.nanoTime();
        System.out.println("Results: " + eea.getNumberOfResults() + ", Computation time: " + (t / 1_000_000) + "ms");
        eea.destroy();
    }
    
    private static void testEsperLLSimple(Iterator<Event> events) throws SchemaException, IOException {
      
      
      String query = "select * from CARS.win:time(10 minutes) " + 
                  "match_recognize ( " + 
                  "partition by VID " + 
                  "measures A.VID as VID, current_timestamp() as TIMESTAMP " +
                  "all matches " +
                  "after match skip to next row " +
                  "pattern (A S+ D)" +
                  "define " +  
                  "A as A.ACCEL >= 0.005 AND A.SPEED < 63, " +
                  "D as D.ACCEL <= -0.0045 AND D.SPEED < 63, " +
                  
                  "S AS S.SPEED >= 63" +
                   ")";
      
      EsperEventAdapter eea = new EsperEventAdapter("CARS", SCHEMA, events, query);
      long t = -System.nanoTime();
      eea.run();
      t += System.nanoTime();
      System.out.println("Results: " + eea.getNumberOfResults() + ", Computation time: " + (t / 1_000_000) + "ms");
      eea.destroy();
  }
    
    private static void deserialize(Iterator<Event> events) throws IOException {
        long items = 0;
        long t = -System.nanoTime();
        while (events.hasNext()) {
            events.next();
            items++;
        }
        t += System.nanoTime();
        System.out.println("Results: " + items + ", deserialization time: " + (t / 1_000_000) + "ms");
    }
    
    /**
     * @param args
     * @throws EPException 
     * @throws AlreadyRegisteredException 
     * @throws IOException 
     * @throws FileNotFoundException 
     */
    public static void main(String[] args) throws Exception {
        if ( args.length < 3 ) {
            System.out.println("Usage: AggressiveDrivers scenario file count");
            System.exit(1);
        }
        execute(args[0], args[1], Integer.valueOf(args[2]));
    }
    
    private static void execute( String scenario, String file, int count ) throws Exception {
        System.out.println("Processing " + scenario + "/" + count);
        System.out.print("Performing warm-up: ");
        executeSingle(scenario, getData(file, 1_000_000) );
        System.out.print("Measuring: ");
        executeSingle(scenario, getData(file, count) );
        System.out.println("=========================================");
    }
    
    private static void executeSingle( String scenario, Iterator<Event> events ) throws Exception {
    	 
        switch ( scenario.toLowerCase() ) {
            case "deserialize":
                deserialize(events); break;
            case "tpsll":
                testTPStream(Mode.LOW_LATENCY, events); break;
            case "tpsll*":
                testTPStreamSimple(Mode.LOW_LATENCY, events); break;
            case "iseq":
                testISEQ(events); break;
            case "iseq*":
                testISEQSimple(events); break;
            case "esperclassic":
                testEsper(events); break;
            case "esperclassic*":
                testEsperSimple(events); break;
            case "esperll":
                testEsperLL(events); break;
            case "esperll*":
                testEsperLLSimple(events); break;
            case "sasell*":
                testSaseLL(events); break;
            default:
                throw new IllegalArgumentException("Unknown scenario: " + scenario);
        }
    }
}
