/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.evaluation.processingTime;

import static de.umr.jepc.v2.addons.epa.tpstream.Constraint.TemporalRelation.AABB;
import static de.umr.jepc.v2.addons.epa.tpstream.Constraint.TemporalRelation.ABAB;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Iterator;

import de.umr.ep.common.Attribute;
import de.umr.ep.common.Attribute.DataType;
import de.umr.ep.common.EPException;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.SchemaException;
import de.umr.jepc.v2.addons.epa.TPStream;
import de.umr.jepc.v2.addons.epa.TPStream.Mode;
import de.umr.jepc.v2.addons.epa.tpstream.Constraint;
import de.umr.jepc.v2.addons.epa.tpstream.Constraints;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinition;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinitions;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.CountingSink;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.EVUtil;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.EVEvent;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.EventGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.adapter.EsperEventAdapter;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.adapter.SaseEventAdapter;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.adapter.EsperEventAdapter.SituationQuery;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.iseq.SyntheticISEQ;
import de.umr.jepc.v2.api.EPProvider;
import de.umr.jepc.v2.api.epa.Stream;
import de.umr.jepc.v2.api.epa.pattern.Output;
import de.umr.jepc.v2.api.epa.pattern.symbol.Binding;
import de.umr.jepc.v2.api.epa.pattern.symbol.Bindings;
import de.umr.jepc.v2.api.expression.BooleanExpression;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.IntConstant;
import de.umr.jepc.v2.api.expression.arithmetic.atomic.Variable;
import de.umr.jepc.v2.api.expression.predicate.Equal;
import de.umr.jepc.v2.api.provider.AlreadyRegisteredException;
import de.umr.jepc.v2.api.provider.ExternalStreamHandle;
import de.umr.jepc.v2.api.provider.QueryHandle;
import de.umr.jepc.v2.bridges.nat.NativeBridge;
import de.umr.jepc.v2.provider.def.DefaultEPProvider;

/**
 * @author Michael Körber
 *
 */
/*
 Cumulated: 
  Speed (avg): 31.64006414
  Speed (min): 0.0
  Speed (max): 75.0
  Accel (max): 0.015
  Accel (min): -0.065
  
  Reading time: 530224ms
 */

public class DisconnectedPattern {
    
//    private static final Map<Integer, RepeatableEventGenerator> gens = new HashMap<>(); 
    
    private static class MemThread extends Thread {
        boolean run = true;

        double sum   = 0;
        long   count = 0;
        double max   = 0;
        
        long sleep = 100;
        
        /**
         * 
         */
        public MemThread() {
            super("Memory Tracker");
            setDaemon(true);
        }

        @Override
        public void run() {
            while (run) {
                try {
                    Thread.sleep(sleep);
                } catch (InterruptedException e) {
                }
                double current = (Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory()) / 1024.0 / 1024.0;
                sum += current;
                count++;
                max = Math.max(max, current);
            }
        }
    }
    
    
    
    private static EventGenerator getData(final int limit) throws FileNotFoundException {
        return EVUtil.defaultGenerator(3, limit);
    }
    
    private static void testTPStream(Mode mode, int numEvents, long window) throws AlreadyRegisteredException, EPException, FileNotFoundException, IOException {
        EPProvider provider = new DefaultEPProvider(new NativeBridge());

        EventGenerator gen = getData(numEvents);
        
        ExternalStreamHandle sh = provider.registerExternalStream("linear", gen.getSchema());

        BooleanExpression a = new Equal(new Variable("A1"), new IntConstant(1));
        BooleanExpression b = new Equal(new Variable("A2"), new IntConstant(1));
        BooleanExpression c = new Equal(new Variable("A3"), new IntConstant(1));

        TPStream def = new TPStream(new Stream("linear"), mode, window,
                new SituationDefinitions(
                        new SituationDefinition('a', a, new Bindings(new Binding("V1", new Variable("ID")))), 
                        new SituationDefinition('b', b),
                        new SituationDefinition('c', c)),
                new Constraints(
                        new Constraint('a', 'b', AABB),
                        new Constraint('b', 'c', ABAB) ),
                new Output("V1"));

        QueryHandle qh = provider.createQuery("aggressive", def);
        CountingSink cs = new CountingSink();

        provider.registerSink(qh, cs);

        long time = -System.nanoTime();
        while (gen.hasNext()) {
            sh.pushEvent(gen.next());
        }
        time += System.nanoTime();

        System.out.println("Results: " + cs.count + ", Computation time: " + (time / 1_000_000) + "ms");

        provider.destroy();
    }
    
    private static void testISEQ(int numEvents, long window) throws AlreadyRegisteredException, EPException, FileNotFoundException, IOException {
        EPProvider provider = new DefaultEPProvider(new NativeBridge());

        EventGenerator gen = getData(numEvents);
        
        ExternalStreamHandle sh = provider.registerExternalStream("linear", gen.getSchema());

        BooleanExpression a = new Equal(new Variable("A1"), new IntConstant(1));
        BooleanExpression b = new Equal(new Variable("A2"), new IntConstant(1));
        BooleanExpression c = new Equal(new Variable("A3"), new IntConstant(1));

        SyntheticISEQ def = new SyntheticISEQ(new Stream("linear"), window,
                new SituationDefinitions(
                        new SituationDefinition('a', a, new Bindings(new Binding("V1", new Variable("ID")))), 
                        new SituationDefinition('b', b),
                        new SituationDefinition('c', c)),
                new Output("V1"));

        QueryHandle qh = provider.createQuery("aggressive", def);
        CountingSink cs = new CountingSink();

        provider.registerSink(qh, cs);

        long time = -System.nanoTime();
        while (gen.hasNext()) {
            sh.pushEvent(gen.next());
        }
        time += System.nanoTime();

        System.out.println("Results: " + cs.count + ", Computation time: " + (time / 1_000_000) + "ms");

        provider.destroy();
    }
    
    private static void testSase(int numEvents, long window) throws FileNotFoundException {
        String query = "PATTERN SEQ(STREAM a, STREAM+ b, STREAM c, STREAM+ d, STREAM e, STREAM+ f, STREAM g)\n" + 
                "WHERE partition-contiguity\n" +
//                "WHERE skip-till-any-match\n" +
                "AND [ID]\n" +
                "AND a.A1 = 0 \n" +
                "AND b[0].A1 = 1 \n" +
                "AND b[i].A1 = 1 \n" +
                "AND c.A1 = 0 \n" +
                "AND c.A2 = 0 \n" +
                "AND e.A2 = 1 \n" + 
                "AND e.A3 = 0 \n" +
                "AND f[0].A2 = 1 \n" + 
                "AND f[0].A3 = 1 \n" +
                "AND f[i].A2 = 1 \n" + 
                "AND f[i].A3 = 1 \n" +
                "AND g.A2 = 0 \n" + 
                "AND g.A3 = 1 \n" +
                "WITHIN " + window;

        EventGenerator gen = getData(numEvents);
        SaseEventAdapter sea = new SaseEventAdapter("STREAM", gen.getSchema(), gen, query);
//        ConfigFlags.printResults = true;
        long t = -System.nanoTime();
        sea.run();
        t += System.nanoTime();
        System.out.println("Results: " + sea.getNumberOfResults() + ", Computation time: " + (t / 1_000_000) + "ms");
        sea.destroy();
    }
    
    private static void testEsper(int numEvents, long window) throws SchemaException, FileNotFoundException {
        String query1 = "insert into S1 " +  
                    "select * from STREAM.win:time(" + window + " milliseconds)" + 
                    "match_recognize ( " + 
                    "measures FIRST(B.ID) AS ID, FIRST(A.TSTART) as TSTART, LAST(B.TSTART) as TEND " + 
                    "pattern (B A+ B)" + 
                    "define " +  
                    "A as A.A1 = 1, " + 
                    "B as B.A1 = 0)";
        
        String query2 = "insert into S2 " +  
                "select * from STREAM.win:time(" + window + " milliseconds)" + 
                "match_recognize ( " + 
                "measures FIRST(B.ID) AS ID, FIRST(A.TSTART) as TSTART, LAST(B.TSTART) as TEND " + 
                "pattern (B A+ B)" + 
                "define " +  
                "A as A.A2 = 1, " + 
                "B as B.A2 = 0)";
        
        String query3 = "insert into S3 " +  
                "select * from STREAM.win:time(" + window + " milliseconds)" + 
                "match_recognize ( " + 
                "measures FIRST(B.ID) AS ID, FIRST(A.TSTART) as TSTART, LAST(B.TSTART) as TEND " + 
                "pattern (B A+ B)" + 
                "define " +  
                "A as A.A3 = 1, " + 
                "B as B.A3 = 0)";
        
        
        EventSchema sout = new EventSchema(new Attribute("ID", DataType.INTEGER));
        
        String fQuery = "select c.TEND as TIMESTAMP from pattern[ " +
        "every a=S1 " +
        " -> ( (every b=S2(TSTART > a.TEND)) " +
        " -> every c=S3(TSTART > b.TSTART, TSTART < b.TEND, TEND > b.TEND, TEND - a.TSTART < " + window + "))" +
        " where timer:within(" + window + " milliseconds)]";
        
        EventGenerator gen = getData(numEvents);
        
        EsperEventAdapter eea = new EsperEventAdapter("STREAM", gen.getSchema(), gen, fQuery,
                new SituationQuery("S1",sout,query1),
                new SituationQuery("S2",sout,query2),
                new SituationQuery("S3",sout,query3)
        );
        
        
        long t = -System.nanoTime();
        eea.run();
        t += System.nanoTime();
        System.out.println("Results: " + eea.getNumberOfResults() + ", Computation time: " + (t / 1_000_000) + "ms");
        eea.destroy();
    }
    
    private static void testEsperLL(int numEvents, long window) throws SchemaException, FileNotFoundException {
        String query = "select * from STREAM.win:time(" + (window+1) + " milliseconds)" +  
                    "match_recognize ( " + 
                    "measures "
//                    + "FIRST(A1.TSTART)  as ATS, LAST(A1.TEND) as ATE, "
//                    + "SA2.TSTART as BTS, LAST(A23.TEND) as BTE, "
//                    + "FIRST(A23.TSTART) as CTS, SA3.TEND as CTE, "
                    + "current_timestamp() as TIMESTAMP " +
                    "after match skip to current row " +
                    "pattern (NA1 A1+ ((NA12 F*) | (NA1A2 A2* NA2 F*))  SA2 A23+ SA3)" +
                    "define " +  
                    "A1 as A1.A1 = 1, " +
                    "A2 as A2.A2 = 1, " +
                    
                    "A23 as A23.A2 = 1 AND A23.A3 = 1, " +
                    "SA2 as SA2.A2 = 1 AND SA2.A3 = 0, " +
                    "SA3 as SA3.A3 = 1 AND SA3.A2 = 0, " +
					
					"NA1 as NA1.A1 = 0, " +
					"NA2 as NA2.A2 = 0, " +
					
					"NA1A2 as NA1A2.A1 = 0 AND NA1A2.A2 = 1, " +
					"NA12  as NA12.A1  = 0 AND NA12.A2 = 0 " +

                     ")";
        
        EventGenerator gen = getData(numEvents);
        EsperEventAdapter eea = new EsperEventAdapter("STREAM", gen.getSchema(), gen, query);
        long t = -System.nanoTime();
        eea.run();
        t += System.nanoTime();
        System.out.println("Results: " + eea.getNumberOfResults() + ", Computation time: " + (t / 1_000_000) + "ms");
        eea.destroy();
    }
    
    private static void deserialize(int count) throws FileNotFoundException {
        Iterator<EVEvent> i = getData(count);
        long items = 0;
        long t = -System.nanoTime();
        while (i.hasNext()) {
            i.next();
            items++;
        }
        t += System.nanoTime();
        System.out.println("Results: " + items + ", deserialization time: " + (t / 1_000_000) + "ms");
    }
    
    /**
     * @param args
     * @throws JEPCException 
     * @throws AlreadyRegisteredException 
     * @throws IOException 
     * @throws FileNotFoundException 
     */
    public static void main(String[] args) throws Exception {
        if ( args.length < 3 ) {
            System.out.println("Usage: Synthetic engine count window");
            System.exit(1);
        }
        execute(args[0], Integer.valueOf(args[1]), Long.valueOf(args[2]));
    }
    
    private static void execute( String scenario, int count, long window ) throws Exception {
        System.out.println("Processing " + scenario + "/" + count + "/" + window);
        System.out.print("Performing warm-up: ");
        executeSingle(scenario, 100_000, 500);
        System.out.print("Measuring: ");
        MemThread mt = new MemThread();
        mt.start();
        executeSingle(scenario, count, window);
        mt.run = false;
        mt.join();
        System.out.println("Memory Consumption: Avg: " + Math.round(mt.sum/mt.count) + ", Max: " + Math.round(mt.max));
        System.out.println("=========================================");
    }
    
    private static void executeSingle( String scenario, int count, long window ) throws Exception {
        switch ( scenario.toLowerCase() ) {
            case "deserialize":
                deserialize(count); break;
            case "tpsll":
                testTPStream(Mode.LOW_LATENCY, count, window); break;
            case "iseq":
                testISEQ(count, window); break;
            case "esperclassic":
                testEsper(count, window); break;
            case "esperll":
                testEsperLL(count, window); break;
            case "sasell*":
                testSase(count, window); break;
            default:
                throw new IllegalArgumentException("Unknown scenario: " + scenario);
        }
    }
}
