/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.evaluation.util;

import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.EventGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.SituationGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.generator.BaseEventGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.situations.Situation;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.situations.SituationDef;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.situations.duration.UniformDurationGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.Matcher;
import de.umr.jepc.v2.api.epa.Stream;

/**
 * @author Michael Körber
 *
 */
public final class EVUtil {
    
    public static final int NUM_EVENTS = 20_000_000;
    
    public static final String STREAM = "S";
    
    public static final Stream STREAM_EPA = new Stream(STREAM);

    private EVUtil() {
    }
    
    public static void feedMatcher(SituationGenerator sgen, Matcher m, long window) {
        de.umr.jepc.v2.addons.epa.tpstream.Situation[] work = new de.umr.jepc.v2.addons.epa.tpstream.Situation[sgen.getNumberOfSituations()];
        long last = Long.MIN_VALUE;
        if (sgen.hasNext()) {
            Situation s = sgen.next();
            work[s.idx] = s.interval;
            last = s.interval.getEnd();
        }

        while (sgen.hasNext()) {
            Situation s = sgen.next();
            if (s.interval.getEnd() > last) {
                m.update(work, last - window);
                for (int i = 0; i < work.length; i++)
                    work[i] = null;
            }
            work[s.idx] = s.interval;
            last = s.interval.getEnd();
        }
        for (int i = 0; i < work.length; i++) {
            if (work[i] != null) {
                m.update(work, last - window);
                break;
            }
        }
    }
    
    public static long tp( long duration ) {
        return tp( NUM_EVENTS, duration );
    }
    
    public static long tp( int numEvents, long duration ) {
        return (long) (numEvents / (duration/1000.0));
    }
    
    public static EventGenerator defaultGenerator( int numSymbols ) {
        return defaultGenerator(numSymbols, NUM_EVENTS);
    }
    
    public static EventGenerator defaultGenerator( int numSymbols, int numEvents ) {
        SituationDef[] defs = new SituationDef[numSymbols];
        for (int i = 0; i < defs.length; i++) {
            defs[i] = new SituationDef(new UniformDurationGenerator(10, 100), new UniformDurationGenerator(10, 50));
        }
        return new BaseEventGenerator(STREAM, numEvents, 0, 1, defs);
    }

}
