package de.umr.jepc.v2.addons.epa.tpstream.evaluation.util;

import java.util.Arrays;
import java.util.List;

import de.umr.jepc.v2.addons.epa.tpstream.Constraints;
import de.umr.jepc.v2.addons.epa.tpstream.Situation;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinitions;
import de.umr.jepc.v2.addons.epa.tpstream.buffer.SituationBuffers;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.Matcher;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.eval.Evaluator;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.eval.EvaluatorUtil;
import de.umr.jepc.v2.addons.epa.tpstream.optimizer.stats.TPStats;

public class FixedOrderMatcher implements Matcher {
	

	private final SituationBuffers buffers;
	
	private final SituationDefinitions definitions;
	
	private final Constraints constraints;
	
	private final long window;
	
	private final MatcherCallback callback;

	private Evaluator[] evaluators;

	public FixedOrderMatcher(SituationDefinitions definitions, Constraints constraints, long window, MatcherCallback callback, List<Character> order) {
        this.buffers = new SituationBuffers(definitions);
        this.evaluators = new Evaluator[definitions.getNumberOfDefinitions()];
        this.definitions = definitions;
        this.constraints = constraints;
        this.window = window;
        this.callback = callback;
        setOrder(order);
    }
	
	public void setOrder( List<Character> order ) {
		Evaluator ev = EvaluatorUtil.create(new TPStats(definitions, constraints), definitions, constraints, order, buffers, callback);
		Arrays.fill(evaluators, ev);
	}
	
	@Override
	public void update(Situation[] updates, long currentTime) {
		buffers.cleanup(currentTime - window);
		for (int idx = 0; idx < updates.length; idx++) {
			Situation interval = updates[idx];
			if ( interval == null || !interval.isFinished() )
		        continue;
			
			buffers.getBuffer(idx).add(interval);
	
			Evaluator ev = evaluators[idx];
			if ( ev != null ) {
			    Situation[] work = new Situation[evaluators.length];
			    work[idx] = interval;
				ev.evaluate(work,currentTime);
			}
		}
	}

}
