/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.evaluation.util;

import de.umr.ep.common.EPException;
import de.umr.ep.event.EventSchema;
import de.umr.jepc.v2.addons.epa.AbstractSituationMatcher;
import de.umr.jepc.v2.addons.epa.tpstream.Constraints;
import de.umr.jepc.v2.addons.epa.tpstream.Situation;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinitions;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.Matcher;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.Matcher.MatcherCallback;
import de.umr.jepc.v2.addons.epa.tpstream.optimizer.Optimizer;
import de.umr.jepc.v2.addons.epa.tpstream.partition.AbstractPartitionFactory;
import de.umr.jepc.v2.addons.epa.tpstream.partition.PartitionFactory;
import de.umr.jepc.v2.addons.epa.tpstream.partition.deriver.AbstractDeriver;
import de.umr.jepc.v2.addons.epa.tpstream.partition.deriver.ClassicDeriver;
import de.umr.jepc.v2.api.epa.EPA;
import de.umr.jepc.v2.api.epa.pattern.Output;
import de.umr.jepc.v2.api.epa.pattern.Partition;
import de.umr.jepc.v2.bindings.EVBindings;
import de.umr.jepc.v2.expressions.bool.EVBooleanExpression;


/**
 * @author mika
 *
 */
public class SituationCounter extends AbstractSituationMatcher {
	
	/** The serialVersionUID */
	private static final long serialVersionUID = 1L;

	long count = 0;
	
	private final Constraints constraints;
	
	private final Matcher matcher = new Matcher() {

		@Override
		public void update(Situation[] updates, long currentTime) {
			for ( Situation s : updates ) {
				if ( s != null && s.isFinished() )
					count++;
			}
		}
	};
	
	public long getCount() {
		return count;
	}

	/**
	 * Creates a new SituationCounter instance
	 * @param input
	 * @param window
	 * @param definitions
	 * @param output
	 */
	public SituationCounter(EPA input, long window, SituationDefinitions definitions, Constraints constraints, Output output) {
		super(input, window, definitions, output);
		this.constraints = constraints;
	}

	/**
	 * Creates a new SituationCounter instance
	 * @param input
	 * @param window
	 * @param definitions
	 * @param output
	 * @param partitionBy
	 */
	public SituationCounter(EPA input, long window, SituationDefinitions definitions, Constraints constraints, Output output,
			Partition partitionBy) {
		super(input, window, definitions, output, partitionBy);
		this.constraints = constraints;
	}

	/**
	 * @{inheritDoc}
	 */
	@Override
	protected PartitionFactory getPartitionFactory(EventSchema schemaIn, MatcherCallback callback) throws EPException {
		return new AbstractPartitionFactory(schemaIn, definitions, constraints, window, this) {
			@Override
			protected AbstractDeriver createDeriver(EVBooleanExpression condition, EVBindings aggreagtes) {
				return new ClassicDeriver(condition, aggreagtes);
			}

			@Override
			protected Matcher createMatcher(SituationDefinitions definitions, Constraints constraints,
					long window, Optimizer optimizer, MatcherCallback callback) {
				return matcher;
			}
		};
	}
}
