/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data;

import com.google.common.util.concurrent.RateLimiter;

import de.umr.ep.event.EventSchema;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.EVUtil;


/**
 * @author mika
 *
 */
public class ThrottledEVGen implements EventGenerator {

	private final EventGenerator source;
	
	private final RateLimiter rl;
	
	/**
	 * Creates a new ThrottledEVGen instance
	 * @param source
	 * @param eventsPerSecond
	 */
	public ThrottledEVGen(EventGenerator source, long eventsPerSecond) {
		this.source = source;
		this.rl = RateLimiter.create(eventsPerSecond);
	}

	/**
	 * @{inheritDoc}
	 */
	@Override
	public boolean hasNext() {
		return source.hasNext();
	}

	/**
	 * @{inheritDoc}
	 */
	@Override
	public EVEvent next() {
		rl.acquire();
		return source.next();
	}

	/**
	 * @{inheritDoc}
	 */
	@Override
	public String getName() {
		return source.getName();
	}

	/**
	 * @{inheritDoc}
	 */
	@Override
	public EventSchema getSchema() {
		return source.getSchema();
	}
	
	static boolean end = false;
	
	
	
	public static void main(String...args) {
		EventGenerator eg = new ThrottledEVGen(EVUtil.defaultGenerator(3, 100_000_000), 1000);
		
		long counter = 0;
		
		Thread stopThread = new Thread(new Runnable() {
			
			@Override
			public void run() {
				try {
					Thread.sleep(10_000);
				}
				catch (InterruptedException ex) {
				}
				finally {
					end = true;
				}
			}
		});
		
		System.out.print("Start... ");
		
		long time = -System.currentTimeMillis();
		stopThread.start();
		while ( eg.hasNext() && !end ) {
			eg.next();
			counter++;
		}
		time += System.currentTimeMillis();
		
		System.out.println("Rate: " + (counter / (time/1000.0)) + " events/s");
		
	}
}
