/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.adapter;

import com.espertech.esper.client.Configuration;
import com.espertech.esper.client.EPAdministrator;
import com.espertech.esper.client.EPServiceProvider;
import com.espertech.esper.client.EPServiceProviderManager;
import com.espertech.esper.client.EPStatement;
import com.espertech.esper.client.EventBean;
import com.espertech.esper.client.UpdateListener;
import com.espertech.esper.client.time.CurrentTimeEvent;
import com.espertech.esper.client.util.EventUnderlyingType;

import de.umr.ep.common.Attribute;
import de.umr.ep.common.Attribute.DataType;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.SchemaException;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.SituationGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.situations.Situation;

/**
 * @author Michael Körber
 *
 */
public class EsperAdapter implements EngineAdapter {
    
    private final EPServiceProvider epService;
    
    private final SituationGenerator gen;
    
    private long lastTick = Long.MIN_VALUE;
    
    private long results = 0;
    
    public EsperAdapter(SituationGenerator gen, String query) throws SchemaException {

        Configuration config = new Configuration();
        
        
        // disable internal clock
        config.getEngineDefaults().getThreading().setInternalTimerEnabled(false);
        // for resource utilization, enable view sharing (only recommended in single-threaded setups like this)
        config.getEngineDefaults().getViewResources().setShareViews(true);
        // for better performance, change internal event representation to object array
        config.getEngineDefaults().getEventMeta().setDefaultEventRepresentation(EventUnderlyingType.OBJECTARRAY);
        
        epService = EPServiceProviderManager.getProvider("esper-provier", config);
        this.gen = gen;
        
        EventSchema schema = new EventSchema(
                new Attribute("type",DataType.INTEGER),
                new Attribute("tstart", DataType.LONG),
                new Attribute("tend", DataType.LONG)
        );
        
        this.registerStream(gen.getName(), schema);
        addQuery(query);
    }
    
    private void registerStream(String name, EventSchema schema) {
        String[] names = new String[schema.getNumAttributes()];
        Object[] types = new Object[schema.getNumAttributes()];
        
        int idx = 0;
        for ( Attribute a : schema) {
            names[idx] = a.getName();
            switch (a.getType()) {
            case BYTE:      types[idx] = byte.class; break;
            case SHORT:     types[idx] = short.class; break;
            case INTEGER:   types[idx] = int.class; break;
            case LONG:      types[idx] = long.class; break;
            case FLOAT:     types[idx] = float.class; break;
            case DOUBLE:    types[idx] = double.class; break;
            case STRING:    types[idx] = String.class; break;
            default: throw new IllegalArgumentException("Unsupported data type: " + a.getType() );
            }
            idx++;
        }
        
        epService.getEPAdministrator().getConfiguration().addEventType(name, names, types);
    }
    
    public void run() {
        while ( gen.hasNext() ) {
            Situation s = gen.next();
            if ( s.interval.getEnd() > lastTick ) {
                lastTick = s.interval.getEnd();
                epService.getEPRuntime().sendEvent(new CurrentTimeEvent(lastTick));
            }
            Object[] payload = new Object[] { s.idx, s.interval.getStart(), s.interval.getEnd() };
            epService.getEPRuntime().sendEvent(payload, gen.getName());
        }
    }
    
    public long getNumberOfResults() {
        return results;
    }
    
    
    
    public void addQuery( String epl) {
        EPAdministrator admin = epService.getEPAdministrator();
        
        EPStatement stmt = admin.createEPL(epl);
        
        stmt.addListener( new UpdateListener() {
            @Override
            public void update(EventBean[] newEvents, EventBean[] oldEvents) {
                if ( newEvents != null ) 
                    results+=newEvents.length;
            }
        });
    }
    
    public void destroy() {
        epService.destroy();
    }

}
