/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.adapter;

import java.util.Iterator;

import com.espertech.esper.client.Configuration;
import com.espertech.esper.client.EPAdministrator;
import com.espertech.esper.client.EPServiceProvider;
import com.espertech.esper.client.EPServiceProviderManager;
import com.espertech.esper.client.EPStatement;
import com.espertech.esper.client.EventBean;
import com.espertech.esper.client.UpdateListener;
import com.espertech.esper.client.time.CurrentTimeEvent;
import com.espertech.esper.client.util.EventUnderlyingType;

import de.umr.ep.common.Attribute;
import de.umr.ep.event.Event;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.SchemaException;

/**
 * @author Michael Körber
 *
 */
public class EsperEventAdapter implements EngineAdapter {
    
    public static class SituationQuery {
        private String name;
        private EventSchema outputSchema;
        private String query;
        /**
         * @param name
         * @param outputSchema
         * @param query
         */
        public SituationQuery(String name, EventSchema outputSchema, String query) {
            this.name = name;
            this.outputSchema = outputSchema;
            this.query = query;
        }
    }
    

    private final EPServiceProvider epService;

    private final String            stream;

    private final Iterator<? extends Event>   gen;

    private long                    lastTick = Long.MIN_VALUE;

    private long                    results  = 0;

    public EsperEventAdapter(String stream, EventSchema schema, Iterator<? extends Event> events, String finalQuery, SituationQuery... sitQueries) throws SchemaException {

        Configuration config = new Configuration();

        // disable internal clock
        config.getEngineDefaults().getThreading().setInternalTimerEnabled(false);
        // for resource utilization, enable view sharing (only recommended in single-threaded setups like this)
        config.getEngineDefaults().getViewResources().setShareViews(true);
        // for better performance, change internal event representation to object array
        config.getEngineDefaults().getEventMeta().setDefaultEventRepresentation(EventUnderlyingType.OBJECTARRAY);

        epService = EPServiceProviderManager.getProvider("esper-provier", config);
        this.gen = events;
        this.stream = stream;

        this.registerStream(stream, schema);
        for ( SituationQuery sq : sitQueries ) {
            this.registerStream(sq.name, sq.outputSchema);
            addQuery(sq.query);
        }
        
        
        EPStatement stmt = addQuery(finalQuery);
        
        stmt.addListener(new UpdateListener() {
            @Override
            public void update(EventBean[] newEvents, EventBean[] oldEvents) {
                if (newEvents != null) {
                    results += newEvents.length;
//                    for ( EventBean eb : newEvents )
//                        if ( eb instanceof ObjectArrayEventBean ) 
//                            System.out.println(Arrays.toString(((ObjectArrayEventBean) eb).getProperties()));
//                        else
//                            System.out.println(eb.getUnderlying());
                }
            }
        });
        
    }

    private void registerStream(String name, EventSchema schema) {
        String[] names = new String[schema.getNumAttributes() + 2];
        Object[] types = new Object[schema.getNumAttributes() + 2];

        int idx = 0;
        for (Attribute a : schema) {
            names[idx] = a.getName();
            switch (a.getType()) {
            case BYTE:
                types[idx] = byte.class;
                break;
            case SHORT:
                types[idx] = short.class;
                break;
            case INTEGER:
                types[idx] = int.class;
                break;
            case LONG:
                types[idx] = long.class;
                break;
            case FLOAT:
                types[idx] = float.class;
                break;
            case DOUBLE:
                types[idx] = double.class;
                break;
            case STRING:
                types[idx] = String.class;
                break;
            default: throw new IllegalArgumentException("Unsupported data type: " + a.getType() );
            }
            idx++;
        }
        names[idx] = "TSTART";
        types[idx++] = long.class;
        names[idx] = "TEND";
        types[idx] = long.class;

        epService.getEPAdministrator().getConfiguration().addEventType(name, names, types);
    }

    public void run() {
        while (gen.hasNext()) {
            Event e = gen.next();
            if (e.getT1() > lastTick) {
                lastTick = e.getT1();
                epService.getEPRuntime().sendEvent(new CurrentTimeEvent(lastTick));
            }

            Object[] payload = new Object[e.getNumberOfAttributes() + 2];
            for ( int i = 0; i < e.getNumberOfAttributes(); i++ )
                payload[i] = e.get(i);
            payload[payload.length-2] = e.getT1();
            payload[payload.length-1] = e.getT2();
            epService.getEPRuntime().sendEvent(payload, stream);
        }
    }

    public long getNumberOfResults() {
        return results;
    }

    public EPStatement addQuery(String epl) {
        EPAdministrator admin = epService.getEPAdministrator();

        return admin.createEPL(epl);
    }

    public void destroy() {
        epService.destroy();
    }

}
