/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.adapter;

import java.util.concurrent.atomic.AtomicInteger;

import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.SituationGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.situations.Situation;
import edu.umass.cs.sase.engine.ConfigFlags;
import edu.umass.cs.sase.engine.EngineController;
import edu.umass.cs.sase.engine.Profiling;
import edu.umass.cs.sase.stream.Event;
import edu.umass.cs.sase.stream.Stream;
import net.sourceforge.jeval.EvaluationException;

/**
 * @author Michael Körber
 *
 */
public class SaseAdapter implements EngineAdapter {

    private final EngineController ec;

    public SaseAdapter(SituationGenerator gen, String query) {
        ec = new EngineController();
        ec.setNfaQuery(query);
        ec.initializeEngine();

        SaseStream ss = new SaseStream(gen);
        ec.setInput(ss);

        Profiling.resetProfiling();
        ConfigFlags.printResults = false;
    }

    public void run() {
        try {
            ec.runEngine();
        } catch (CloneNotSupportedException | EvaluationException e) {
            throw new RuntimeException(e);
        }
    }

    public long getNumberOfResults() {
        return Profiling.numberOfMatches;
    }

    private static class SaseStream extends Stream {
        private AtomicInteger      nextId = new AtomicInteger(0);

        private SituationGenerator gen;
        
        public SaseStream(SituationGenerator gen) {
            super(0);
            this.gen = gen;
        }

        @Override
        public Event popEvent() {
            if (gen.hasNext()) {
                return new SaseEvent(gen.next());
            }
            else
                return null;
        }

        @Override
        public Event[] getEvents() {
            throw new UnsupportedOperationException();
        }

        @Override
        public void setEvents(Event[] events) {
            throw new UnsupportedOperationException();
        }

        @Override
        public int getSize() {
            throw new UnsupportedOperationException();
        }

        @Override
        public void setSize(int size) {
            throw new UnsupportedOperationException();
        }

        private class SaseEvent implements Event, Cloneable {

            private int  id;
            private final long tstart;
            private final long tend;
            private final int  type;

            public SaseEvent(Situation s) {
                this.id = nextId.incrementAndGet();
                this.type = s.idx;
                this.tstart = s.interval.getStart();
                this.tend = s.interval.getEnd();
            }
            
            private SaseEvent(int id, long tstart, long tend, int type) {
                super();
                this.id = id;
                this.tstart = tstart;
                this.tend = tend;
                this.type = type;
            }

            /**
             * {@inheritDoc}
             */
            @Override
            public int getAttributeByName(String attributeName) {
                switch ( attributeName.toLowerCase() ) {
                case "tstart": return (int) tstart;
                case "tend": return (int) tend;
                case "type": return type;
                }
                throw new RuntimeException("No suche value: " + attributeName);
            }

            /**
             * {@inheritDoc}
             */
            @Override
            public double getAttributeByNameDouble(String attributeName) {
                throw new RuntimeException("No suche value");
            }

            /**
             * {@inheritDoc}
             */
            @Override
            public String getAttributeByNameString(String attributeName) {
                throw new RuntimeException("No suche value");
            }

            /**
             * {@inheritDoc}
             */
            @Override
            public int getAttributeValueType(String attributeName) {
                return 0;
            }

            /**
             * {@inheritDoc}
             */
            @Override
            public int getId() {
                return id;
            }

            /**
             * {@inheritDoc}
             */
            @Override
            public void setId(int id) {
                this.id = id;
            }

            /**
             * {@inheritDoc}
             */
            @Override
            public int getTimestamp() {
                return (int) tend;
            }

            /**
             * {@inheritDoc}
             */
            @Override
            public String getEventType() {
                return gen.getName() + type;
            }

            public Object clone() {
                return new SaseEvent(id,tstart,tend,type);
            }

            @Override
            public String toString() {
                return "SaseEvent [id=" + id + ", tstart=" + tstart + ", tend=" + tend + ", type=" + type + "]";
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void destroy() {
    }

}
