/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.adapter;

import java.util.Iterator;
import java.util.concurrent.atomic.AtomicInteger;

import de.umr.ep.event.EventSchema;
import de.umr.ep.event.SchemaException;
import edu.umass.cs.sase.engine.ConfigFlags;
import edu.umass.cs.sase.engine.EngineController;
import edu.umass.cs.sase.engine.Profiling;
import edu.umass.cs.sase.stream.Event;
import edu.umass.cs.sase.stream.Stream;
import net.sourceforge.jeval.EvaluationException;

/**
 * @author Michael Körber
 *
 */
public class SaseEventAdapter implements EngineAdapter {

    private final EngineController ec;

    public SaseEventAdapter(String stream, EventSchema schema, Iterator<? extends de.umr.ep.event.Event> events, String query) {
        ec = new EngineController();
        ec.setNfaQuery(query);
        ec.initializeEngine();

        SaseStream ss = new SaseStream(stream, schema, events);
        ec.setInput(ss);

        Profiling.resetProfiling();
        ConfigFlags.printResults = false;
    }

    public void run() {
        try {
            ec.runEngine();
        } catch (CloneNotSupportedException | EvaluationException e) {
            throw new RuntimeException(e);
        }
    }

    public long getNumberOfResults() {
        return Profiling.numberOfMatches;
    }

    private static class SaseStream extends Stream {
        private AtomicInteger                      nextId = new AtomicInteger(0);
        private final String                       stream;
        private final EventSchema                  schema;
        private Iterator<? extends de.umr.ep.event.Event> gen;

        public SaseStream(String stream, EventSchema schema, Iterator<? extends de.umr.ep.event.Event> gen) {
            super(0);
            this.stream = stream;
            this.schema = schema;
            this.gen = gen;
        }

        @Override
        public Event popEvent() {
            if (gen.hasNext()) {
                return new SaseEvent(stream,gen.next());
            } else
                return null;
        }

        @Override
        public Event[] getEvents() {
            throw new UnsupportedOperationException();
        }

        @Override
        public void setEvents(Event[] events) {
            throw new UnsupportedOperationException();
        }

        @Override
        public int getSize() {
            throw new UnsupportedOperationException();
        }

        @Override
        public void setSize(int size) {
            throw new UnsupportedOperationException();
        }

        private class SaseEvent implements Event, Cloneable {

            private int                            id;
            private final String                   type;
            private final de.umr.ep.event.Event event;

            public SaseEvent(String type, de.umr.ep.event.Event e) {
                this.id = nextId.incrementAndGet();
                this.type = type;
                this.event = e;
            }

            private SaseEvent(int id, String type, de.umr.ep.event.Event e) {
                super();
                this.id = id;
                this.type = type;
                this.event = e;
            }

            /**
             * {@inheritDoc}
             */
            @Override
            public int getAttributeByName(String attributeName) {
                try {
                    switch (attributeName.toLowerCase()) {
                    case "timestamp":
                        return (int) event.getT1();
                    case "accel":
                        return (int) (event.get(schema.getAttributeIndex(attributeName),Double.class) * 100_000);
                    default:
                        return ((Number) event.get(schema.getAttributeIndex(attributeName))).intValue();
                    }
                } catch (SchemaException e) {
                    throw new RuntimeException("No suche value: " + attributeName, e);
                }
            }

            /**
             * {@inheritDoc}
             */
            @Override
            public double getAttributeByNameDouble(String attributeName) {
                try {
                    return ((Number) event.get(schema.getAttributeIndex(attributeName))).doubleValue();
                } catch (SchemaException e) {
                    throw new RuntimeException("No suche value: " + attributeName, e);
                }
            }

            /**
             * {@inheritDoc}
             */
            @Override
            public String getAttributeByNameString(String attributeName) {
                try {
                    return event.get(schema.getAttributeIndex(attributeName), String.class);
                } catch (SchemaException e) {
                    throw new RuntimeException("No suche value: " + attributeName, e);
                }
            }

            /**
             * {@inheritDoc}
             */
            @Override
            public int getAttributeValueType(String attributeName) {
                try {
                    switch (schema.byName(attributeName).getType()) {
                        default:
                            return 0;
//                    case BYTE:
//                    case SHORT:
//                    case INTEGER:
//                    case LONG:
//                        return 0;
//                    case FLOAT:
//                    case DOUBLE:
//                        return 1;
//                    default:
//                        return 2;
                    }
                } catch (SchemaException e) {
                    throw new RuntimeException("No suche value: " + attributeName, e);
                }
            }

            /**
             * {@inheritDoc}
             */
            @Override
            public int getId() {
                return id;
            }

            /**
             * {@inheritDoc}
             */
            @Override
            public void setId(int id) {
                this.id = id;
            }

            /**
             * {@inheritDoc}
             */
            @Override
            public int getTimestamp() {
                return (int) event.getT1();
            }

            /**
             * {@inheritDoc}
             */
            @Override
            public String getEventType() {
                return type;
            }

            public Object clone() {
                return new SaseEvent(id, type, event);
            }

            @Override
            public String toString() {
                return "SaseEvent [id=" + id + ", type=" + type + ", raw=" + event + "]";
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void destroy() {
    }

}
