package de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.generator;

import java.util.Random;

import de.umr.ep.common.Attribute;
import de.umr.ep.common.Attribute.DataType;
import de.umr.ep.event.EventSchema;
import de.umr.ep.event.SchemaException;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.EVEvent;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.EventGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.situations.SituationDef;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.situations.duration.BooleanAttributeGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.situations.duration.BooleanAttributeValue;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.situations.duration.GaussianDurationGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.situations.duration.UniformDurationGenerator;

public class BaseEventGenerator implements EventGenerator{
    
    private final Random rand = new Random(System.nanoTime());
	
    private final String name;
    
	private final int numEvents;
	
	private final BooleanAttributeGenerator[] gens;
	
	private final BooleanAttributeValue[] currentValues;
	
	private final EventSchema schema;
	
	private final long freq;
	
	private int eventsProduced;
	
	private long timestamp;
	

	public BaseEventGenerator(String name, int numEvents, long starttime, long freq, SituationDef...defs) {
		super();
		this.name = name;
		this.numEvents = numEvents;
		this.freq = freq;
		this.timestamp = starttime;
		this.gens = new BooleanAttributeGenerator[defs.length];
		this.currentValues = new BooleanAttributeValue[defs.length];
		
		for ( int i = 0; i < defs.length; i++ ) {
			this.gens[i] = new BooleanAttributeGenerator(defs[i].durationGen, defs[i].gapGen);
			this.currentValues[i] = this.gens[i].next(rand);
		}
		
		Attribute[] as = new Attribute[defs.length + 1];
		as[0] = new Attribute("ID", DataType.INTEGER);
		for ( int i = 1; i < as.length; i++ ) {
			as[i] = new Attribute("A" + (i), DataType.INTEGER);
		}
		try {
			this.schema = new EventSchema(as);
		} catch (SchemaException e) {
			throw new IllegalStateException("Must not happen!",e);
		}
	}

	@Override
	public boolean hasNext() {
		return numEvents > eventsProduced;
	}

	@Override
	public EVEvent next() {
		Object[] payload = new Object[this.currentValues.length+1];
		payload[0] = 1;
		for ( int i = 0; i < this.currentValues.length; i++ ) {
			if ( !this.currentValues[i].isDataLeft() ) {
				this.currentValues[i] = this.gens[i].next(rand);
			}
			payload[i+1] = this.currentValues[i].getValue() ? 1 : 0;
		}
		timestamp += freq;
		eventsProduced++;
		return new EVEvent    (payload, timestamp, timestamp+1);
	}

	@Override
	public EventSchema getSchema() {
		return schema;
	}
	
	@Override
	public String getName() {
        return name;
    }
	
	public int getNumEvents() {
	    return numEvents;
	}
	
	public int getEventsProduced() {
	    return eventsProduced;
	}

    public static void main(String...args) {
		BaseEventGenerator bss = new BaseEventGenerator("Test", 100, 0, 1000,
			new SituationDef(new UniformDurationGenerator(8, 12), new UniformDurationGenerator(2, 4)),
			new SituationDef(new GaussianDurationGenerator(10, 2), new GaussianDurationGenerator(3, 1))
		);
		
		while ( bss.hasNext() ) {
			System.out.println(bss.next());
		}
		
	}
	

}
