package de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.generator;

import java.util.ArrayDeque;
import java.util.Queue;

import de.umr.ep.common.Attribute.DataType;
import de.umr.ep.event.Event;
import de.umr.ep.event.SchemaException;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.EventGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.SituationGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.situations.Situation;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.situations.SituationDef;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.situations.duration.GaussianDurationGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.data.situations.duration.UniformDurationGenerator;
import de.umr.jepc.v2.addons.epa.tpstream.partition.deriver.ClassicDeriver;
import de.umr.jepc.v2.bindings.EVBindings;
import de.umr.jepc.v2.expressions.arithmetic.constant.EVIntConstant;
import de.umr.jepc.v2.expressions.arithmetic.field.EVIntField;
import de.umr.jepc.v2.expressions.predicate.numeric.EVNumericEqual;

public class WrappingSituationGenerator implements SituationGenerator {
	
    private final String name;
    
	private EventGenerator egen;
	
	private Queue<Situation> sq = new ArrayDeque<>();
	
	private Situation next = null;
	
	private de.umr.jepc.v2.addons.epa.tpstream.Situation[] buf;
	
	private ClassicDeriver[] preds;
	
	public WrappingSituationGenerator(String name, EventGenerator egen, String attrib, String... attribs) throws SchemaException {
	    this.name = name;
		this.egen = egen;
		this.buf = new de.umr.jepc.v2.addons.epa.tpstream.Situation[1+attribs.length];
		
		
		this.preds = new ClassicDeriver[buf.length];
		
		preds[0] = new ClassicDeriver(
	            new EVNumericEqual(DataType.INTEGER, 
	            new EVIntField(egen.getSchema().getAttributeIndex(attrib)), 
	            new EVIntConstant(1)), new EVBindings());
		
		for ( int i = 1; i < buf.length; i++ ) {
		    preds[i] = new ClassicDeriver(
		            new EVNumericEqual(DataType.INTEGER, 
		            new EVIntField(egen.getSchema().getAttributeIndex(attribs[i-1])), 
		            new EVIntConstant(1)), new EVBindings());
		}
		
		this.next = computeNext();
	}
	
	/**
     * {@inheritDoc}
     */
    @Override
    public String getName() {
        return name;
    }
	
	/**
     * {@inheritDoc}
     */
    @Override
    public int getNumberOfSituations() {
        return egen.getSchema().getNumAttributes();
    }
	
	private Situation computeNext() {
		if ( !sq.isEmpty() )
			return sq.poll();
		else if ( egen.hasNext() ) {
			fillQueue();
			return computeNext();
		}
		return null;
	}
	
	private void fillQueue() {
		while ( sq.isEmpty() && egen.hasNext() ) {
			Event e = egen.next();
			for ( int i = 0; i < buf.length; i++ ) {
			    buf[i] = preds[i].update(e);
			    if ( buf[i] != null )
			        sq.add(new Situation(i,buf[i]));
			}
		}
	}
	

	@Override
	public boolean hasNext() {
		return next != null;
	}

	@Override
	public Situation next() {
		Situation res = next;
		next = computeNext();
		return res;
	}
	
	public static void main(String...args) throws SchemaException {
		BaseEventGenerator bss = new BaseEventGenerator("Test",100, 0, 1000,
			new SituationDef(new UniformDurationGenerator(8, 12), new UniformDurationGenerator(2, 4)),
			new SituationDef(new GaussianDurationGenerator(10, 2), new GaussianDurationGenerator(3, 1))
		);
		
		WrappingSituationGenerator wsg = new WrappingSituationGenerator("TestSit", bss, "A1", "A2");
		
		
		while ( wsg.hasNext() ) {
			System.out.println(wsg.next());
		}
	}
}
