/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.iseq;

import java.util.ArrayList;
import java.util.List;

import de.umr.ep.common.IncompatibleTypeException;
import de.umr.ep.event.EventSchema;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinition;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinitions;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.Matcher;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.Matcher.MatcherCallback;
import de.umr.jepc.v2.addons.epa.tpstream.partition.PartitionFactory;
import de.umr.jepc.v2.addons.epa.tpstream.partition.TPSPartition;
import de.umr.jepc.v2.addons.epa.tpstream.partition.deriver.AbstractDeriver;
import de.umr.jepc.v2.api.epa.pattern.symbol.Binding;
import de.umr.jepc.v2.api.epa.pattern.symbol.Bindings;
import de.umr.jepc.v2.bindings.EVBinding;
import de.umr.jepc.v2.bindings.EVBindings;
import de.umr.jepc.v2.expressions.bool.EVBooleanExpression;
import de.umr.jepc.v2.ql.ExpressionTranslator;
import de.umr.jepc.v2.ql.TranslatorException;

/**
 * Creates partitions for classic temporal pattern matching with closed interval semantics. 
 * @author Michael Körber
 *
 */
public abstract class AbstractISEQPartitionFactory implements PartitionFactory {
    
    
    /** The situation definitions */
    private final SituationDefinitions definitions;
    
    /** The window constraint */
    private final long window;
    
    /** The callback to invoke on new results */
    private final MatcherCallback callback;
    
    /** The set of compiled conditions */
    private final EVBooleanExpression[] conditions;
    
    /** The set of compiled aggregates */
    private final EVBindings[] aggregates;

    /**
     * @param schema the schema of incoming events
     * @param definitions the situation stream definitions
     * @param constraints the defined constraints
     * @param window the defined time-window
     * @param callback the callback to invoke on new results
     * @throws TranslatorException if conditions could not be translated
     * @throws IncompatibleTypeException if conditions could not be translated
     */
    public AbstractISEQPartitionFactory(EventSchema schema, SituationDefinitions definitions, long window, MatcherCallback callback)
            throws TranslatorException, IncompatibleTypeException {
        this.definitions = definitions;
        this.window = window;
        this.callback = callback;
        this.conditions = new EVBooleanExpression[definitions.getNumberOfDefinitions()];
        this.aggregates = new EVBindings[definitions.getNumberOfDefinitions()];
        
        int i = 0;
        for ( SituationDefinition s : definitions ) {
            // Do not allow bindings to be accessed!
            conditions[i] = ExpressionTranslator.INSTANCE.translateBooleanExpression(s.getCondition(), schema, new Bindings());
            List<EVBinding> tmp = new ArrayList<>();
            for (Binding b : s.getAggregates()) {
                tmp.add(ExpressionTranslator.INSTANCE.translateVariableBinding(b, schema, new Bindings()));
            }
            this.aggregates[i] = new EVBindings( tmp.toArray(new EVBinding[tmp.size()]) );
            i++;
        }
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public TPSPartition create() {
        AbstractDeriver[] derivers = new AbstractDeriver[definitions.getNumberOfDefinitions()];
        for ( int i = 0; i < derivers.length; i++ ) {
            derivers[i] = createDeriver(conditions[i], aggregates[i]);
        }
        return new TPSPartition(derivers, createMatcher(definitions,window,callback));
    }
    
    

    protected abstract AbstractDeriver createDeriver(EVBooleanExpression condition, EVBindings bindings);

    protected abstract Matcher createMatcher(SituationDefinitions definitions, long window, MatcherCallback callback);
}
