/**
 * 
 */
package de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.iseq;

import de.umr.ep.common.EPException;
import de.umr.ep.event.Event;
import de.umr.ep.event.EventSchema;
import de.umr.jepc.v2.addons.epa.tpstream.Situation;
import de.umr.jepc.v2.addons.epa.tpstream.SituationDefinitions;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.WallTime;
import de.umr.jepc.v2.addons.epa.tpstream.evaluation.util.iseq.synthetic.SyntheticISEQFactory;
import de.umr.jepc.v2.addons.epa.tpstream.matcher.Matcher.MatcherCallback;
import de.umr.jepc.v2.addons.epa.tpstream.partition.PartitionFactory;
import de.umr.jepc.v2.addons.epa.tpstream.partition.deriver.AbstractDeriver;
import de.umr.jepc.v2.addons.epa.tpstream.partition.deriver.MSituation;
import de.umr.jepc.v2.addons.epa.tpstream.partition.deriver.MeasuringISEQDeriver;
import de.umr.jepc.v2.api.bridge.EventChannel;
import de.umr.jepc.v2.api.epa.EPA;
import de.umr.jepc.v2.api.epa.pattern.Output;
import de.umr.jepc.v2.api.epa.pattern.Partition;
import de.umr.jepc.v2.bindings.EVBindings;
import de.umr.jepc.v2.expressions.bool.EVBooleanExpression;


/**
 * @author mika
 *
 */
public class MeasuringSyntheticISEQ extends SyntheticISEQ {

	/** The serialVersionUID */
	private static final long serialVersionUID = 1L;

	private double eventLatencyAccum = 0;
	
	private double processingLatencyAccum = 0;
	
	private long resultCount = 0;
	
	private long inputCount = 0;
	
	/**
	 * Creates a new MeasuringSyntheticISEQ instance
	 * @param input
	 * @param window
	 * @param definitions
	 * @param output
	 * @param partitionBy
	 */
	public MeasuringSyntheticISEQ(EPA input, long window, SituationDefinitions definitions, Output output,
			Partition partitionBy) {
		super(input, window, definitions, output, partitionBy);
	}

	/**
	 * Creates a new MeasuringSyntheticISEQ instance
	 * @param input
	 * @param window
	 * @param definitions
	 * @param output
	 */
	public MeasuringSyntheticISEQ(EPA input, long window, SituationDefinitions definitions, Output output) {
		super(input, window, definitions, output);
	}
	
	

	/**
	 * @{inheritDoc}
	 */
	@Override
	public void process(EventChannel input, Event event) {
		inputCount++;
		WallTime.update();
		super.process(input, event);
	}

	/**
	 * @{inheritDoc}
	 */
	@Override
	public void onResult(Situation[] result, long currentTime) {
		processingLatencyAccum += (System.nanoTime() - WallTime.getLastTimestamp());
		long bend = ((MSituation) result[1]).getWallEnd();
		long cend = ((MSituation) result[2]).getWallEnd();
		resultCount++;
		eventLatencyAccum += (cend-bend);
		super.onResult(result, currentTime);
	}

	/**
	 * @{inheritDoc}
	 */
	@Override
	protected PartitionFactory getPartitionFactory(EventSchema schemaIn, MatcherCallback callback) throws EPException {
		return new SyntheticISEQFactory(schemaIn, definitions, window, callback) {

			/**
			 * @{inheritDoc}
			 */
			@Override
			protected AbstractDeriver createDeriver(EVBooleanExpression condition, EVBindings bindings) {
				return new MeasuringISEQDeriver(condition, bindings);
			}
			
		};
	}
	
	public double getAverageLatency() {
//		return (latencyAccum - WallTime.measureCosts(inputCount+resultCount)) / resultCount;
		return (eventLatencyAccum+processingLatencyAccum) / resultCount;
	}
	
	public double getAverageEventLatency() {
		return eventLatencyAccum / resultCount;
	}
	
	public double getAverageProcessingLatency() {
		return processingLatencyAccum / resultCount;
	}
	
	/**
	 * @return the resultCount
	 */
	public long getResultCount() {
		return this.resultCount;
	}
	
	/**
	 * @return the inputCount
	 */
	public long getInputCount() {
		return this.inputCount;
	}
}
